@@ -9,12 +9,10 @@ representations to libgps structures.
 
 ***************************************************************************/
 
-#include <math.h>
-#include <assert.h>
-#include <string.h>
-#include <stddef.h>
-#include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdbool.h>
+#include <stdlib.h>
 
 #include "gpsd.h"
 #include "gps_json.h"
