@@ -13,8 +13,8 @@
 /*
  * The True North compass fails with current gpsd versions for reasons
  * the dev team has been unable to diagnose due to not having test hardware.
- * The sup[port for it is conditioned out in order to simplify moving 
- * to the new JSON-based oprotocol and reduce startup time.
+ * The support for it is conditioned out in order to simplify moving 
+ * to the new JSON-based protocol and reduce startup time.
  */
 #undef TRUENORTH
 
@@ -37,6 +37,9 @@
    from the 'datawin' window for the GPS mode. */
 #define DATAWIN_GPS_FIELDS 9
 
+/* Count of optional fields that we'll display if we have the room. */
+#define DATAWIN_OPTIONAL_FIELDS 5
+
 /* This is how many display fields are output in the 'datawin' window
    when in COMPASS mode.  Change this value if you add or remove fields
    from the 'datawin' window for the COMPASS mode. */
@@ -68,6 +71,9 @@
    GPS mode. */
 #define MIN_GPS_DATAWIN_SIZE (DATAWIN_GPS_FIELDS + DATAWIN_OVERHEAD)
 
+/* And the maximum size we'll try to use */
+#define MAX_GPS_DATAWIN_SIZE (DATAWIN_GPS_FIELDS + DATAWIN_OPTIONAL_FIELDS + DATAWIN_OVERHEAD)
+
 /* This is the minimum size we'll accept for the 'datawin' window in
    COMPASS mode. */
 #define MIN_COMPASS_DATAWIN_SIZE (DATAWIN_COMPASS_FIELDS + DATAWIN_OVERHEAD)
@@ -78,29 +84,20 @@
 /* This is the maximum size we need for the 'satellites' window. */
 #define MAX_SATWIN_SIZE (MAX_POSSIBLE_SATS + SATWIN_OVERHEAD)
 
-#include <sys/types.h>
-#include <sys/select.h>
-#ifndef S_SPLINT_S
-#include <sys/socket.h>
-#include <unistd.h>
-#endif /* S_SPLINT_S */
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
-#include <ctype.h>
 #include <math.h>
 #include <errno.h>
-#include <assert.h>
+#include <curses.h>
+#include <time.h>
 #include <signal.h>
-#include <stdbool.h>
+#ifndef S_SPLINT_S
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 
 #include "gpsd_config.h"
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif /* HAVE_NCURSES_H */
-
 #include "gps.h"
 #include "gpsdclient.h"
 #include "revision.h"
@@ -132,6 +129,7 @@ static bool compass_flag = false;
 #define CGPS_QUIT	0	/* voluntary yterminastion */
 #define GPS_GONE	-1	/* GPS device went away */
 #define GPS_ERROR	-2	/* low-level failure in GPS read */
+#define GPS_TIMEOUT	-3	/* low-level failure in GPS waiting */
 
 /* Convert true heading to magnetic.  Taken from the Aviation
    Formulary v1.43.  Valid to within two degrees within the
@@ -204,18 +202,17 @@ static float true2magnetic(double lat, double lon, double heading)
 /* Function to call when we're all done.  Does a bit of clean-up. */
 static void die(int sig)
 {
-    /* Ignore signals. */
-    (void)signal(SIGINT, SIG_IGN);
-    (void)signal(SIGHUP, SIG_IGN);
+    if (!isendwin())
+    {
+	/* Move the cursor to the bottom left corner. */
+	(void)mvcur(0, COLS - 1, LINES - 1, 0);
 
-    /* Move the cursor to the bottom left corner. */
-    (void)mvcur(0, COLS - 1, LINES - 1, 0);
+	/* Put input attributes back the way they were. */
+	(void)echo();
 
-    /* Put input attributes back the way they were. */
-    (void)echo();
-
-    /* Done with curses. */
-    (void)endwin();
+	/* Done with curses. */
+	(void)endwin();
+    }
 
     /* We're done talking to gpsd. */
     (void)gps_close(&gpsdata);
@@ -229,26 +226,31 @@ static void die(int sig)
     case GPS_ERROR:
 	(void)fprintf(stderr, "cgps: GPS read returned error\n");
 	break;
+    case GPS_TIMEOUT:
+	(void)fprintf(stderr, "cgps: GPS timeout\n");
+	break;
     default:
 	(void)fprintf(stderr, "cgps: caught signal %d\n", sig);
+	break;
     }
 
     /* Bye! */
     exit(0);
 }
 
-
 static enum deg_str_type deg_type = deg_dd;
 
 /*@ -globstate @*/
 static void windowsetup(void)
+/* inotialize curses and set up screen windows */
 {
     /* Set the window sizes per the following criteria:
      * 
      * 1.  Set the window size to display the maximum number of
-     * satellites possible, but not more than the size required to
-     * display the maximum number of satellites gpsd is capable of
-     * tracking (MAXCHANNELS - 2).
+     * satellites possible, but not more than can be fit in a 
+     * window the size of the GPS report window. We have to set
+     * the limit that way because MAXCHANNELS has been made large
+     * in order to prepare for survey-grade receivers..
      * 
      * 2.  If the screen size will not allow for the full complement of
      * satellites to be displayed, set the windows sizes smaller, but
@@ -264,6 +266,9 @@ static void windowsetup(void)
      */
     int xsize, ysize;
 
+    /* Fire up curses. */
+    (void)initscr();
+    (void)noecho();
     getmaxyx(stdscr, ysize, xsize);
 
 #ifdef TRUENORTH
@@ -287,26 +292,18 @@ static void windowsetup(void)
     } else
 #endif /* TRUENORTH */
     {
-	if (ysize == MAX_SATWIN_SIZE) {
+	if (ysize > MAX_GPS_DATAWIN_SIZE) {
+	    raw_flag = true;
+	    window_length = MAX_GPS_DATAWIN_SIZE;
+	} else if (ysize == MAX_GPS_DATAWIN_SIZE) {
 	    raw_flag = false;
-	    window_length = MAX_SATWIN_SIZE;
-	    display_sats = MAX_POSSIBLE_SATS;
-	} else if (ysize == MAX_SATWIN_SIZE + 1) {
-	    raw_flag = true;
-	    window_length = MAX_SATWIN_SIZE;
-	    display_sats = MAX_POSSIBLE_SATS;
-	} else if (ysize > MAX_SATWIN_SIZE + 2) {
-	    raw_flag = true;
-	    window_length = MAX_SATWIN_SIZE;
-	    display_sats = MAX_POSSIBLE_SATS;
+	    window_length = MAX_GPS_DATAWIN_SIZE;
 	} else if (ysize > MIN_GPS_DATAWIN_SIZE) {
-	    raw_flag = false;
-	    window_length = ysize - (int)raw_flag;
-	    display_sats = window_length - SATWIN_OVERHEAD - (int)raw_flag;
+	    raw_flag = true;
+	    window_length = MIN_GPS_DATAWIN_SIZE;
 	} else if (ysize == MIN_GPS_DATAWIN_SIZE) {
 	    raw_flag = false;
 	    window_length = MIN_GPS_DATAWIN_SIZE;
-	    display_sats = window_length - SATWIN_OVERHEAD - 1;
 	} else {
 	    (void)mvprintw(0, 0,
 			   "Your screen must be at least 80x%d to run cgps.",
@@ -317,6 +314,7 @@ static void windowsetup(void)
 	    (void)sleep(5);
 	    die(0);
 	}
+	display_sats = window_length - SATWIN_OVERHEAD - (int)raw_flag;
     }
 
 #ifdef TRUENORTH
@@ -390,7 +388,7 @@ static void windowsetup(void)
 	 * there in the first place because I arbitrarily thought they
 	 * sounded interesting. ;^) */
 
-	if (window_length >= (MIN_GPS_DATAWIN_SIZE + 5)) {
+	if (window_length == MAX_GPS_DATAWIN_SIZE) {
 	    (void)mvwprintw(datawin, 10, DATAWIN_DESC_OFFSET,
 			    "Longitude Err:");
 	    (void)mvwprintw(datawin, 11, DATAWIN_DESC_OFFSET,
@@ -409,10 +407,19 @@ static void windowsetup(void)
 
 /*@ +globstate @*/
 
+static void resize(int sig UNUSED)
+/* cope with terminal resize */
+{
+    if (!isendwin()) 
+    {
+	(void)endwin();
+	windowsetup();
+    }
+}
+
 #ifdef TRUENORTH
 /* This gets called once for each new compass sentence. */
-static void update_compass_panel(struct gps_data_t *gpsdata,
-				 char *message, size_t len UNUSED)
+static void update_compass_panel(struct gps_data_t *gpsdata)
 {
     char scr[128];
     /* Print time/date. */
@@ -466,19 +473,13 @@ static void update_compass_panel(struct gps_data_t *gpsdata,
 #endif /* TRUENORTH */
 
 /* This gets called once for each new GPS sentence. */
-static void update_gps_panel(struct gps_data_t *gpsdata,
-			     char *message, size_t len UNUSED)
+static void update_gps_panel(struct gps_data_t *gpsdata)
 {
     int i, j, n;
     int newstate;
     char scr[128];
     bool usedflags[MAXCHANNELS];
 
-    /* this is where we implement source-device filtering */
-    if (gpsdata->dev.path[0] != '\0' && source.device != NULL
-	&& strcmp(source.device, gpsdata->dev.path) != 0)
-	return;
-
     /* must build bit vector of which statellites are used from list */
     for (i = 0; i < MAXCHANNELS; i++) {
 	usedflags[i] = false;
@@ -630,22 +631,11 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
     (void)mvwprintw(datawin, 8, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
     /* Fill in receiver type. */
-    if (gpsdata->set & (DEVICE_SET | DEVICELIST_SET)) {
-#ifdef CLIENTDEBUG_ENABLE
-	if (debug > 0)
-	    (void)fprintf(stderr, "Device ID or list set.\n");
-#endif
-	if (gpsdata->set & DEVICE_SET) {
-	    (void)snprintf(scr, sizeof(scr), "%s", gpsdata->dev.driver);
-	} else if (gpsdata->devices.ndevices == 1) {
-	    (void)snprintf(scr, sizeof(scr), "%s",
-			   gpsdata->devices.list[0].driver);
-	} else {
-	    (void)snprintf(scr, sizeof(scr), "%d devices",
-			   gpsdata->devices.ndevices);
-	}
-	(void)mvwprintw(datawin, 9, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
-    }
+    for (i = 0; i < gpsdata->devices.ndevices; i++)
+	if (strcmp(gpsdata->devices.list[i].path, gpsdata->dev.path) == 0)
+	   break;
+    (void)snprintf(scr, sizeof(scr), "%s", gpsdata->devices.list[i].driver);
+    (void)mvwprintw(datawin, 9, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
     /* Note that the following four fields are exceptions to the
      * sizing rule.  The minimum window size does not include these
      * fields, if the window is too small, they get excluded.  This
@@ -703,7 +693,7 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
 
     /* Be quiet if the user requests silence. */
     if (!silent_flag && raw_flag) {
-	(void)waddstr(messages, message);
+	(void)waddstr(messages, gps_data(gpsdata));
     }
 
     /* Reset the status_timer if the state has changed. */
@@ -745,10 +735,7 @@ int main(int argc, char *argv[])
 {
     int option;
     int c;
-
-    struct timeval timeout;
-    fd_set rfds;
-    int data;
+    unsigned int flags = WATCH_ENABLE;
 
     /*@ -observertrans @*/
     switch (gpsd_units()) {
@@ -777,7 +764,7 @@ int main(int argc, char *argv[])
     /*@ +observertrans @*/
 
     /* Process the options.  Print help if requested. */
-    while ((option = getopt(argc, argv, "hVl:smuD:")) != -1) {
+    while ((option = getopt(argc, argv, "hVl:smu:D:")) != -1) {
 	switch (option) {
 #ifdef CLIENTDEBUG_ENABLE
 	case 'D':
@@ -851,57 +838,43 @@ int main(int argc, char *argv[])
 	gpsd_source_spec(NULL, &source);
 
     /* Open the stream to gpsd. */
-    if (gps_open_r(source.server, source.port, &gpsdata) != 0) {
+    if (gps_open(source.server, source.port, &gpsdata) != 0) {
 	(void)fprintf(stderr,
 		      "cgps: no gpsd running or network error: %d, %s\n",
 		      errno, gps_errstr(errno));
 	exit(2);
     }
 
-    /* Fire up curses. */
-    (void)initscr();
-    (void)noecho();
+    /* note: we're assuming BSD-style reliable signals here */
     (void)signal(SIGINT, die);
     (void)signal(SIGHUP, die);
+    (void)signal(SIGWINCH, resize);
 
     windowsetup();
 
-    /* Here's where updates go now that things are established. */
-#ifdef TRUENORTH
-    if (compass_flag) {
-	gps_set_raw_hook(&gpsdata, update_compass_panel);
-    } else
-#endif /* TRUENORTH */
-    {
-	gps_set_raw_hook(&gpsdata, update_gps_panel);
-    }
-
     status_timer = time(NULL);
 
-    (void)gps_stream(&gpsdata, WATCH_ENABLE, NULL);
+    if (source.device != NULL)
+	flags |= WATCH_DEVICE;
+    (void)gps_stream(&gpsdata, flags, source.device);
 
     /* heart of the client */
     for (;;) {
-
-	/* watch to see when it has input */
-	FD_ZERO(&rfds);
-	FD_SET(gpsdata.gps_fd, &rfds);
-
-	/* wait up to five seconds. */
-	timeout.tv_sec = 5;
-	timeout.tv_usec = 0;
-
-	/* check if we have new information */
-	data = select(gpsdata.gps_fd + 1, &rfds, NULL, NULL, &timeout);
-
-	if (data == -1) {
-	    fprintf(stderr, "cgps: socket error 3\n");
-	    exit(2);
-	} else if (data) {
+	if (!gps_waiting(&gpsdata, 5000000)) {
+	    die(GPS_TIMEOUT);
+	} else {
 	    errno = 0;
 	    if (gps_read(&gpsdata) == -1) {
 		fprintf(stderr, "cgps: socket error 4\n");
 		die(errno == 0 ? GPS_GONE : GPS_ERROR);
+	    } else {
+		/* Here's where updates go now that things are established. */
+#ifdef TRUENORTH
+		if (compass_flag)
+		    update_compass_panel(&gpsdata);
+		else
+#endif /* TRUENORTH */
+		    update_gps_panel(&gpsdata);
 	    }
 	}
 
@@ -927,6 +900,5 @@ int main(int argc, char *argv[])
 	default:
 	    break;
 	}
-
     }
 }
