@@ -10,17 +10,9 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/types.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
 #include <ctype.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <time.h>
-#include <stdio.h>
 
 #include "gpsd.h"
 #include "bits.h"
@@ -62,7 +54,7 @@ static void from_sixbit(char *bitvec, uint start, int count, char *to)
 
 /*@ +charint -fixedformalarray -usedef -branchstate @*/
 bool aivdm_decode(const char *buf, size_t buflen,
-		  struct aivdm_context_t ais_contexts[AIVDM_CHANNELS], 
+		  struct aivdm_context_t ais_contexts[AIVDM_CHANNELS],
 		  struct ais_t *ais)
 {
 #ifdef __UNUSED_DEBUG__
@@ -124,9 +116,18 @@ bool aivdm_decode(const char *buf, size_t buflen,
     switch (field[4][0]) {
     /* FIXME: if fields[4] == "12", it doesn't detect the error */
     case '\0':
+	/*
+	 * Apparently an empty channel is normal for AIVDO sentences,
+	 * which makes sense as they don't come in over radio.  This
+	 * is going to break if there's ever an AIVDO type 24, though.
+	 */
+	if (strncmp((const char *)field[0], "!AIVDO", 6) != 0)
+	    gpsd_report(LOG_ERROR, "invalid empty AIS channel. Assuming 'A'\n");
+	ais_context = &ais_contexts[0];
+	break;
     case '1':
-	gpsd_report(LOG_ERROR, "invalid AIS channel '%c'. Assuming 'A'\n",
-	                       field[4][0]);
+	gpsd_report(LOG_ERROR, "invalid AIS channel 0x%0x '%c'. Assuming 'A'\n",
+	                       field[4][0], (field[4][0] != '\0' ? field[4][0]:' '));
 	/*@fallthrough@*/
     case 'A':
 	ais_context = &ais_contexts[0];
@@ -138,7 +139,7 @@ bool aivdm_decode(const char *buf, size_t buflen,
 	ais_context = &ais_contexts[1];
 	break;
     default:
-	gpsd_report(LOG_ERROR, "invalid AIS channel.\n");
+	gpsd_report(LOG_ERROR, "invalid AIS channel 0x%0X .\n", field[4][0]);
 	return false;
     }
 
@@ -341,18 +342,18 @@ bool aivdm_decode(const char *buf, size_t buflen,
 	case 7: /* Binary acknowledge */
 	case 13: /* Safety Related Acknowledge */
 	{
-	    unsigned int mmsi[4];
+	    unsigned int u, mmsi[4];
 	    if (ais_context->bitlen < 72 || ais_context->bitlen > 168) {
 		gpsd_report(LOG_WARN, "AIVDM message type %d size is out of range (%zd).\n",
 			    ais->type,
 			    ais_context->bitlen);
 		return false;
 	    }
-	    for (i = 0; i < sizeof(mmsi)/sizeof(mmsi[0]); i++)
-		if (ais_context->bitlen > 40 + 32*i)
-		    mmsi[i] = UBITS(40 + 32*i, 30);
+	    for (u = 0; u < sizeof(mmsi)/sizeof(mmsi[0]); u++)
+		if (ais_context->bitlen > 40 + 32*u)
+		    mmsi[u] = UBITS(40 + 32*u, 30);
 		else
-		    mmsi[i] = 0;
+		    mmsi[u] = 0;
 	    /*@ -usedef @*/
 	    ais->type7.mmsi1 = mmsi[0];
 	    ais->type7.mmsi2 = mmsi[1];
@@ -372,9 +373,70 @@ bool aivdm_decode(const char *buf, size_t buflen,
 	    ais->type8.dac            = UBITS(40, 10);
 	    ais->type8.fid            = UBITS(40, 6);
 	    ais->type8.bitcount       = ais_context->bitlen - 56;
+#ifdef __UNUSED__
+	    if (ais->type8.dac == 1)
+		/*
+		 * The strange order in which these FIDs occur is
+		 * direct from IMO circular 289.
+		 */
+		switch (ais->type8.fid) {
+		case 31:        /* Meteorological-Hydrological data */
+		    ais->type8.dac1fid31.lon		= SBITS(56, 25);
+		    ais->type8.dac1fid31.lat		= SBITS(81, 24);
+		    ais->type8.dac1fid31.accuracy       = (bool)UBITS(105, 1);
+		    ais->type8.dac1fid31.day		= UBITS(106, 5);
+		    ais->type8.dac1fid31.hour		= UBITS(111, 5);
+		    ais->type8.dac1fid31.minute		= UBITS(116, 6);
+		    ais->type8.dac1fid31.wspeed		= UBITS(122, 7);
+		    ais->type8.dac1fid31.wgust		= UBITS(129, 7);
+		    ais->type8.dac1fid31.wdir		= UBITS(129, 7);
+		    ais->type8.dac1fid31.wgustdir	= UBITS(136, 7); 
+		    ais->type8.dac1fid31.temperature	= SBITS(143, 11);
+		    ais->type8.dac1fid31.humidity	= UBITS(154, 7);
+		    ais->type8.dac1fid31.dewpoint	= SBITS(154, 10);
+		    ais->type8.dac1fid31.pressure	= UBITS(164, 9);
+		    ais->type8.dac1fid31.pressuretend	= UBITS(173, 3);
+		    ais->type8.dac1fid31.visgreater	= (bool)UBITS(174, 1);
+		    ais->type8.dac1fid31.visibility	= UBITS(175, 8);
+		    ais->type8.dac1fid31.waterlevel	= UBITS(183, 12);
+		    goto nocopy;
+		case 25:        /* Dangerous Cargo Indication */
+		    goto nocopy;
+		case 32:        /* Tidal Window */
+		    goto nocopy;
+		case 24:        /* Extended ship static & voyage-related data */
+		    goto nocopy;
+		case 16:        /* Number of persons on board */
+		    goto nocopy;
+		case 17:        /* VTS-generated/synthetic targets */
+		    goto nocopy;
+		case 18:        /* Clearance time to enter port */
+		    goto nocopy;
+		case 19:        /* Marine Traffic Signal */
+		    goto nocopy;
+		case 20:        /* Berthing Data */
+		    goto nocopy;
+		case 21:        /* Weather observation report from ship */
+		    goto nocopy;
+		case 22:        /* Area notice - broadcast */
+		    goto nocopy;
+		case 23:        /* Area notice - addressed */
+		    goto nocopy;
+		case 27:        /* Route information - broadcast */
+		    goto nocopy;
+		case 29:        /* Text Description - broadcast */
+		    goto nocopy;
+		case 30:        /* Text Description - addressed */
+		    goto nocopy;
+		}
+	    /* land here if we failed to match a known DAC/FID */
+#endif /* __UNUSED_ */
 	    (void)memcpy(ais->type8.bitdata,
 			 (char *)ais_context->bits + (56 / BITS_PER_BYTE),
 			 (ais->type8.bitcount + 7) / 8);
+#ifdef __UNUSED__
+	nocopy:
+#endif /* __UNUSED_ */
 	    gpsd_report(LOG_INF, "dac=%u, fid=%u, cnt=%zd\n",
 			ais->type8.dac,
 			ais->type8.fid,
@@ -542,7 +604,7 @@ bool aivdm_decode(const char *buf, size_t buflen,
 			ais->type18.course,
 			ais->type18.heading,
 			ais->type18.second);
-	    break;	
+	    break;
 	case 19:	/* Extended Class B CS Position Report */
 	    if (ais_context->bitlen != 312) {
 		gpsd_report(LOG_WARN, "AIVDM message type 19 size not 312 bits (%zd).\n",
@@ -612,11 +674,11 @@ bool aivdm_decode(const char *buf, size_t buflen,
 		return false;
 	    }
 	    ais->type21.aid_type = UBITS(38, 5);
-	    from_sixbit((char *)ais_context->bits, 
+	    from_sixbit((char *)ais_context->bits,
 			43, 21, ais->type21.name);
 	    if (strlen(ais->type21.name) == 20 && ais_context->bitlen > 272)
-		from_sixbit((char *)ais_context->bits, 
-			    272, (ais_context->bitlen - 272)/6, 
+		from_sixbit((char *)ais_context->bits,
+			    272, (ais_context->bitlen - 272)/6,
 			    ais->type21.name+20);
 	    ais->type21.accuracy     = UBITS(163, 1);
 	    ais->type21.lon          = SBITS(164, 28);
@@ -721,7 +783,7 @@ bool aivdm_decode(const char *buf, size_t buflen,
 			            ais->mmsi);
 		    return false;
 		}
-		(void)strlcpy(ais->type24.shipname, 
+		(void)strlcpy(ais->type24.shipname,
 			      ais_context->shipname24,
 			      sizeof(ais_context->shipname24));
 		ais->type24.shiptype = UBITS(40, 8);
@@ -766,7 +828,7 @@ bool aivdm_decode(const char *buf, size_t buflen,
 	    }
 	    ais->type25.addressed	= (bool)UBITS(38, 1);
 	    ais->type25.structured	= (bool)UBITS(39, 1);
-	    if (ais_context->bitlen < (40 + (16*ais->type25.structured) + (30*ais->type25.addressed))) {
+	    if (ais_context->bitlen < (unsigned)(40 + (16*ais->type25.structured) + (30*ais->type25.addressed))) {
 		gpsd_report(LOG_WARN, "AIVDM message type 25 too short for mode.\n");
 		return false;
 	    }
@@ -792,7 +854,7 @@ bool aivdm_decode(const char *buf, size_t buflen,
 			ais->type25.structured,
 			ais->type25.dest_mmsi,
 			ais->type25.app_id,
-			ais->type25.bitcount);		
+			ais->type25.bitcount);
 	    break;
 	case 26:	/* Binary Message, Multiple Slot */
 	    if (ais_context->bitlen < 60 || ais_context->bitlen > 1004) {
@@ -802,6 +864,10 @@ bool aivdm_decode(const char *buf, size_t buflen,
 	    }
 	    ais->type26.addressed	= (bool)UBITS(38, 1);
 	    ais->type26.structured	= (bool)UBITS(39, 1);
+	    if ((signed)ais_context->bitlen < 40 + 16*ais->type26.structured + 30*ais->type26.addressed + 20) {
+		gpsd_report(LOG_WARN, "AIVDM message type 26 too short for mode.\n");
+		return false;
+	    }
 	    if (ais->type26.addressed)
 		ais->type26.dest_mmsi   = UBITS(40, 30);
 	    if (ais->type26.structured)
