@@ -1,8 +1,18 @@
 /*
  *
- * This is the gpsd driver for EverMore GPSes operating in binary mode.
- * About the only thing this gives us that NMEA won't is TDOP.
- * But we'll get atomic position reports from it, which is good.
+ * This is the gpsd driver for EverMore GPSes.  They have both an NMEA and
+ * a binary reporting mode, with the interesting property that they will 
+ * cheerfully accept binary commands (such as speed changes) while in NMEA
+ * mode.
+ *
+ * Binary mode would give us atomic fix reports, but it has one large drawback:
+ * the Navigation Data Out message doesn't report a leap-second offset, so ut
+ * is not actually possible to colloect a leap-second offset from it. Therefore
+ * we'll normally run the driver in NMEA mode.  
+ *
+ * About the only thing binary mode gives that NMEA won't is TDOP and raw
+ * pseudoranges, but gpsd does its own DOPs from skyview. By default we'll
+ * trade away raw data to get accurate time.
  *
  * The vendor site is <http://www.emt.com.tw>.
  *
@@ -41,7 +51,7 @@
  * 100303 - date of firmware release DDMMYY
  * 180 -  datum ID; 001 is WGS-84
  * 05 - default elevation mask; see message 0x86
- * 1 - default DOP select (1 is auto DOP mask); see message 0x87
+ * 1 - default DOP select, 1 is auto DOP mask; see message 0x87
  * 20 - default GDOP; see message 0x87
  * 15 - default PDOP
  * 08 - default HDOP
@@ -114,17 +124,10 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 
-#include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <math.h>
-#include <ctype.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <time.h>
-#include <stdio.h>
 
 #include "gpsd.h"
 #if defined(EVERMORE_ENABLE) && defined(BINARY_ENABLE)
@@ -183,21 +186,19 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 
     switch (type) {
     case 0x02:			/* Navigation Data Output */
-	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
-	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
-	/*@ ignore @*//*@ splint is confused @ */
-	session->newdata.time =
-	    gpstime_to_unix(session->context->gps_week,
-			    session->context->gps_tow) -
-	    session->context->leap_seconds;
-	/*@ end @*/
+	session->newdata.time = gpsd_gpstime_resolve(session,
+	  (unsigned short)getleu16(buf2, 2),
+	  (double)getleu32(buf2, 4) * 0.01);
 	ecef_to_wgs84fix(&session->newdata, &session->gpsdata.separation,
-			 getlesl(buf2, 8) * 1.0, getlesl(buf2, 12) * 1.0,
-			 getlesl(buf2, 16) * 1.0, getlesw(buf2, 20) / 10.0,
-			 getlesw(buf2, 22) / 10.0, getlesw(buf2, 24) / 10.0);
+			 (double)getles32(buf2, 8) * 1.0, 
+			 (double)getles32(buf2, 12) * 1.0,
+			 (double)getles32(buf2, 16) * 1.0,
+			 (double)getles16(buf2, 20) / 10.0,
+			 (double)getles16(buf2, 22) / 10.0, 
+			 (double)getles16(buf2, 24) / 10.0);
 	used = (unsigned char)getub(buf2, 26) & 0x0f;
 	//visible = (getub(buf2, 26) & 0xf0) >> 4;
-	version = (uint) getleuw(buf2, 27) / 100.0;
+	version = (uint) getleu16(buf2, 27) / 100.0;
 	/* that's all the information in this packet */
 	if (used < 3)
 	    session->newdata.mode = MODE_NO_FIX;
@@ -223,18 +224,13 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	return mask | CLEAR_IS | REPORT_IS;
 
     case 0x04:			/* DOP Data Output */
-	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
-	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
-	/*@ ignore @*//*@ splint is confused @ */
-	session->newdata.time =
-	    gpstime_to_unix(session->context->gps_week,
-			    session->context->gps_tow) -
-	    session->context->leap_seconds;
-	/*@ end @*/
+	session->newdata.time = gpsd_gpstime_resolve(session,
+	    (unsigned short)getleu16(buf2, 2),
+						  (double)getleu32(buf2, 4) * 0.01);
 	/*
 	 * We make a deliberate choice not to clear DOPs from the
 	 * last skyview here, but rather to treat this as a supplement
-	 * to our calculations from the visiniolity matrix, trusting
+	 * to our calculations from the visibility matrix, trusting
 	 * the firmware algorithms over ours.
 	 */
 	session->gpsdata.dop.gdop = (double)getub(buf2, 8) * 0.1;
@@ -272,14 +268,9 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	return mask;
 
     case 0x06:			/* Channel Status Output */
-	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
-	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
-	/*@ ignore @*//*@ splint is confused @ */
-	session->gpsdata.skyview_time =
-	    gpstime_to_unix(session->context->gps_week,
-			    session->context->gps_tow) -
-	    session->context->leap_seconds;
-	/*@ end @*/
+	session->gpsdata.skyview_time = gpsd_gpstime_resolve(session,
+	    (unsigned short)getleu16(buf2, 2),
+	    (double)getleu32(buf2, 4) * 0.01);
 	session->gpsdata.satellites_visible = (int)getub(buf2, 8);
 	gpsd_zero_satellites(&session->gpsdata);
 	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
@@ -299,7 +290,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 		continue;	/* satellite record is not valid */
 	    session->gpsdata.PRN[satcnt] = prn;
 	    session->gpsdata.azimuth[satcnt] =
-		(int)getleuw(buf2, 7 * i + 7 + 4);
+		(int)getleu16(buf2, 7 * i + 7 + 4);
 	    session->gpsdata.elevation[satcnt] =
 		(int)getub(buf2, 7 * i + 7 + 6);
 	    session->gpsdata.ss[satcnt] = (float)getub(buf2, 7 * i + 7 + 7);
@@ -331,15 +322,10 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 
     case 0x08:			/* Measurement Data Output */
 	/* clock offset is a manufacturer diagnostic */
-	/* (int)getleuw(buf2, 8);  clock offset, 29000..29850 ?? */
-	session->context->gps_week = (unsigned short)getleuw(buf2, 2);
-	session->context->gps_tow = (double)getleul(buf2, 4) * 0.01;
-	/*@ ignore @*//*@ splint is confused @ */
-	session->newdata.time =
-	    gpstime_to_unix(session->context->gps_week,
-			    session->context->gps_tow) -
-	    session->context->leap_seconds;
-	/*@ end @*/
+	/* (int)getleu16(buf2, 8);  clock offset, 29000..29850 ?? */
+	session->newdata.time = gpsd_gpstime_resolve(session,
+	    (unsigned short)getleu16(buf2, 2),
+	    (double)getleu32(buf2, 4) * 0.01);
 	visible = (unsigned char)getub(buf2, 10);
 	/*
 	 * Note: This code is untested. It was written from the manual.
@@ -393,7 +379,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	gpsd_report(LOG_WARN,
 		    "unknown EverMore packet EID 0x%02x, length %zd: %s\n",
 		    buf2[0], datalen, gpsd_hexdump_wrapper(buf2, datalen,
-							   LOG_IO));
+							   LOG_WARN));
 	return 0;
     }
 }
@@ -534,16 +520,24 @@ static void evermore_mode(struct gps_device_t *session, int mode)
 
 static void evermore_event_hook(struct gps_device_t *session, event_t event)
 {
+    if (session->context->readonly)
+	return;
     /*
      * FIX-ME: It might not be necessary to call this on reactivate.
      * Experiment to see if the holds its settings through a close.
      */
     if (event == event_identified || event == event_reactivate) {
-	if (session->packet.type == NMEA_PACKET) {
-	    (void)evermore_nmea_config(session, 1);	/* configure NMEA messages for gpsd (GPGSV every 5s) */
-	}
-	(void)evermore_mode(session, 1);	/* switch GPS to binary mode */
-	session->back_to_nmea = true;
+	/*
+	 * We used to run this driver in binary mode, but that has the
+	 * problem that Evermore binary mode doesn't report a
+	 * leap-second correction in the Navigation Data Out sentence.
+	 * So, run it in NMEA mode to getbUTC corrected by firmware.
+	 * Fortunately the Evermore firmware interprets binary
+	 * commands in NMEA mode, so nothing else needs to change.
+	 */
+	(void)evermore_mode(session, 0);	/* switch GPS to NMEA mode */
+	(void)evermore_nmea_config(session, 1);	/* configure NMEA messages for gpsd (GPGSV every 5s) */
+	session->back_to_nmea = false;
     } else if (event == event_deactivate) {
 	(void)evermore_nmea_config(session, 0);	/* configure NMEA messages to default */
     }
@@ -621,12 +615,13 @@ const struct gps_type_t evermore_binary =
 {
     .type_name      = "EverMore binary",	/* full name of type */
     .packet_type    = EVERMORE_PACKET,		/* lexer packet type */
+    .flags	    = DRIVER_NOFLAGS,		/* no flags set */
     .trigger        = "$PEMT,", 		/* recognize the type */
     .channels       = EVERMORE_CHANNELS,	/* consumer-grade GPS */
     .probe_detect   = NULL,			/* no probe */
     .get_packet     = generic_get,		/* use generic one */
     .parse_packet   = evermore_parse_input,	/* parse message packets */
-    .rtcm_writer    = pass_rtcm,		/* send RTCM data straight */
+    .rtcm_writer    = gpsd_write,		/* send RTCM data straight */
     .event_hook     = evermore_event_hook,	/* lifetime event handler */
 #ifdef ALLOW_RECONFIGURE
     .speed_switcher = evermore_speed,		/* we can change baud rates */
