@@ -106,31 +106,14 @@ invalid data.
 
 ***************************************************/
 
-#include <sys/types.h>
 
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <math.h>
-
 #include <string.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <errno.h>
-#include <inttypes.h>
-
-#include "gpsd_config.h"
-#if defined (HAVE_SYS_SELECT_H)
-#include <sys/select.h>
-#endif
-
-#if defined(HAVE_STRINGS_H)
 #include <strings.h>
-#endif
 
 #include "gpsd.h"
-#include "gps.h"
-#include "timebase.h"
 
 #ifdef GARMINTXT_ENABLE
 
@@ -307,7 +290,7 @@ gps_mask_t garmintxt_parse(struct gps_device_t * session)
 	if (0 != gar_int_decode(buf + 0, 2, 0, 99, &result))
 	    break;
 	session->driver.garmintxt.date.tm_year =
-	    (CENTURY_BASE + (int)result) - 1900;
+	    (session->context->century + (int)result) - 1900;
 	/* month */
 	if (0 != gar_int_decode(buf + 2, 2, 1, 12, &result))
 	    break;
@@ -331,7 +314,7 @@ gps_mask_t garmintxt_parse(struct gps_device_t * session)
 	session->driver.garmintxt.date.tm_sec = (int)result;
 	session->driver.garmintxt.subseconds = 0;
 	session->newdata.time =
-	    (double)mkgmtime(&session->driver.garmintxt.date) +
+	    (timestamp_t)mkgmtime(&session->driver.garmintxt.date) +
 	    session->driver.garmintxt.subseconds;
 	mask |= TIME_IS;
     } while (0);
