@@ -2,20 +2,15 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/types.h>
 #include <stdio.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
 #include <stdlib.h>
+#include <stdbool.h>
 #include <math.h>
 #include <string.h>
 #include <stdarg.h>
-#include <ctype.h>
 #include <time.h>
 
 #include "gpsd.h"
-#include "timebase.h"
 
 #ifdef NMEA_ENABLE
 /**************************************************************************
@@ -58,13 +53,14 @@ static void do_lat_lon(char *field[], struct gps_fix_t *out)
  * Scary timestamp fudging begins here
  *
  * Four sentences, GGA and GLL and RMC and ZDA, contain timestamps.
- * GGA/GLL/RMC timestamps look like hhmmss.ss, with the trailing .ss part
- * optional.  RMC has a date field, in the format ddmmyy.  ZDA has
- * separate fields for day/month/year, with a 4-digit year.  This
+ * GGA/GLL/RMC timestamps look like hhmmss.ss, with the trailing .ss
+ * part optional.  RMC has a date field, in the format ddmmyy.  ZDA
+ * has separate fields for day/month/year, with a 4-digit year.  This
  * means that for RMC we must supply a century and for GGA and GLL we
  * must supply a century, year, and day.  We get the missing data from
- * a previous RMC or ZDA; century in RMC is supplied by a constant if
- * there has been no previous ZDA.
+ * a previous RMC or ZDA; century in RMC is supplied from the daemon's
+ * context (initialized at startup time) if there has been no previous
+ * ZDA.
  *
  **************************************************************************/
 
@@ -73,28 +69,26 @@ static void do_lat_lon(char *field[], struct gps_fix_t *out)
 static void merge_ddmmyy(char *ddmmyy, struct gps_device_t *session)
 /* sentence supplied ddmmyy, but no century part */
 {
-    int yy = DD(ddmmyy + 4), year = session->driver.nmea.date.tm_year;
+    int yy = DD(ddmmyy + 4);
     int mon = DD(ddmmyy + 2);
     int mday = DD(ddmmyy);
+    int year;
 
-    if (year <= 0) {
-	year = (CENTURY_BASE + yy) - 1900;
-    } else if (year % 100 != yy) {
-	/* update year */
-	if (year % 100 == 99 && yy == 0)
-	    yy += 100;		/* century change */
-	year = year / 100 * 100 + yy;
+    /* check for century wrap */
+    if (session->driver.nmea.date.tm_year % 100 == 99 && yy == 0) {
+	session->context->century += 100;
+	gpsd_report(LOG_WARN, "century rollover detected.\n");
     }
-    if ( (1 > year ) || (2200 < year ) ) {
-	gpsd_report(LOG_WARN, "merge_ddmmyy(), bad year: %d\n",  year);
-    } else if ( (1 > mon ) || (12 < mon ) ) {
-	gpsd_report(LOG_WARN, "merge_ddmmyy(), malformed month: %2s\n",  ddmmyy + 2);
+    year = (session->context->century + yy);
+
+    if ( (1 > mon ) || (12 < mon ) ) {
+	gpsd_report(LOG_WARN, "merge_ddmmyy(%s), malformed month\n",  ddmmyy);
     } else if ( (1 > mday ) || (31 < mday ) ) {
-	gpsd_report(LOG_WARN, "merge_ddmmyy(), malformed day: %2s\n",  ddmmyy);
+	gpsd_report(LOG_WARN, "merge_ddmmyy(%s), malformed day\n",  ddmmyy);
     } else {
-	gpsd_report(LOG_DATA, "merge_ddmmyy(ddmmyy) sets year %d from %s\n",
-		    year, ddmmyy);
-	session->driver.nmea.date.tm_year = year;
+	gpsd_report(LOG_DATA, "merge_ddmmyy(%s) sets year %d\n",
+		    ddmmyy, year);
+	session->driver.nmea.date.tm_year = year - 1900;
 	session->driver.nmea.date.tm_mon = mon - 1;
 	session->driver.nmea.date.tm_mday = mday;
     }
@@ -181,7 +175,7 @@ static gps_mask_t processGPRMC(int count, char *field[],
 	mask |= ONLINE_IS;
     } else if (strcmp(field[2], "A") == 0) {
 	/*
-	 * The MKT3301, Royaltek RGM-3800, and possibly other
+	 * The MTK3301, Royaltek RGM-3800, and possibly other
 	 * devices deliver bogus time values when the navigation
 	 * warning bit is set.
 	 */
@@ -259,7 +253,7 @@ static gps_mask_t processGPGLL(int count, char *field[],
      *
      * Unless you care about the FAA indicator, this sentence supplies nothing
      * that GPRMC doesn't already.  But at least one Garmin GPS -- the 48
-     * actually ships updates in GPLL that aren't redundant.
+     * actually ships updates in GLL that aren't redundant.
      */
     char *status = field[7];
     gps_mask_t mask = 0;
@@ -333,6 +327,25 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[],
 
     session->gpsdata.status = atoi(field[6]);
     mask = STATUS_IS;
+    /*
+     * There are some receivers (the Trimble Placer 450 is an example) that
+     * don't ship a GSA with mode 1 when they lose satellite lock. Instead
+     * they just keep reporting GGA and GSA on subsequent cycles with the
+     * timestamp not advancing and a bogus mode.  On the assumption that GGA
+     * is only issued once per cycle we can detect this here (it would be
+     * nicer to do it on GSA but GSA has no timestamp).
+     */
+    session->driver.nmea.latch_mode = strncmp(field[1], 
+					      session->driver.nmea.last_gga_timestamp,
+					      sizeof(session->driver.nmea.last_gga_timestamp))==0;
+    if (session->driver.nmea.latch_mode) {
+	session->gpsdata.status = STATUS_NO_FIX;
+	session->newdata.mode = MODE_NO_FIX;
+    } else
+	(void)strncpy(session->driver.nmea.last_gga_timestamp, 
+		       field[1],
+		       sizeof(session->driver.nmea.last_gga_timestamp));
+    /* if we have a fix and the mode latch is off, go... */
     if (session->gpsdata.status > STATUS_NO_FIX) {
 	char *altitude;
 
@@ -393,6 +406,53 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[],
     return mask;
 }
 
+
+static gps_mask_t processGPGST(int count, char *field[], struct gps_device_t *session)
+/* GST - GPS Pseudorange Noise Statistics */
+{
+    /*
+     * GST,hhmmss.ss,x,x,x,x,x,x,x,*hh
+     * 1 TC time of associated GGA fix
+     * 2 Total RMS standard deviation of ranges inputs to the navigation solution
+     * 3 Standard deviation (meters) of semi-major axis of error ellipse
+     * 4 Standard deviation (meters) of semi-minor axis of error ellipse
+     * 5 Orientation of semi-major axis of error ellipse (true north degrees)
+     * 6 Standard deviation (meters) of latitude error
+     * 7 Standard deviation (meters) of longitude error
+     * 8 Standard deviation (meters) of altitude error
+     * 9 Checksum
+*/
+    if (count < 8) {
+      return 0;
+    }
+
+#define PARSE_FIELD(n) (*field[n]!='\0' ? atof(field[n]) : NAN)
+    session->gpsdata.gst.utctime             = PARSE_FIELD(1);
+    session->gpsdata.gst.rms_deviation       = PARSE_FIELD(2);
+    session->gpsdata.gst.smajor_deviation    = PARSE_FIELD(3);
+    session->gpsdata.gst.sminor_deviation    = PARSE_FIELD(4);
+    session->gpsdata.gst.smajor_orientation  = PARSE_FIELD(5);
+    session->gpsdata.gst.lat_err_deviation   = PARSE_FIELD(6);
+    session->gpsdata.gst.lon_err_deviation   = PARSE_FIELD(7);
+    session->gpsdata.gst.alt_err_deviation   = PARSE_FIELD(8);
+#undef PARSE_FIELD
+    register_fractional_time(field[0], field[1], session);
+
+    gpsd_report(LOG_DATA,
+		"GST: utc = %.2f, rms = %.2f, maj = %.2f, min = %.2f, ori = %.2f, lat = %.2f, lon = %.2f, alt = %.2f\n",
+                session->gpsdata.gst.utctime,
+                session->gpsdata.gst.rms_deviation,
+                session->gpsdata.gst.smajor_deviation,
+                session->gpsdata.gst.sminor_deviation,
+                session->gpsdata.gst.smajor_orientation,
+                session->gpsdata.gst.lat_err_deviation,
+                session->gpsdata.gst.lon_err_deviation,
+                session->gpsdata.gst.alt_err_deviation);
+
+    return NOISE_IS | ONLINE_IS;
+}
+
+
 static gps_mask_t processGPGSA(int count, char *field[],
 			       struct gps_device_t *session)
 /* GPS DOP and Active Satellites */
@@ -421,6 +481,9 @@ static gps_mask_t processGPGSA(int count, char *field[],
     if (count < 17) {
 	gpsd_report(LOG_DATA, "GPGSA: malformed, setting ONLINE_IS only.\n");
 	mask = ONLINE_IS;
+    } else if (session->driver.nmea.latch_mode) {
+	/* last GGA had a non-advancing timestamp; don't trust this GSA */
+	mask = ONLINE_IS;
     } else {
 	int i;
 	session->newdata.mode = atoi(field[2]);
@@ -476,28 +539,32 @@ static gps_mask_t processGPGSV(int count, char *field[],
      */
     int n, fldnum;
     if (count <= 3) {
+	gpsd_report(LOG_WARN, "malformed GPGSV - fieldcount %d <= 3\n",
+		    count);
 	gpsd_zero_satellites(&session->gpsdata);
 	session->gpsdata.satellites_visible = 0;
-	return 0;
+	return ONLINE_IS;
     }
     if (count % 4 != 0) {
 	gpsd_report(LOG_WARN, "malformed GPGSV - fieldcount %d %% 4 != 0\n",
 		    count);
 	gpsd_zero_satellites(&session->gpsdata);
 	session->gpsdata.satellites_visible = 0;
-	return 0;
+	return ONLINE_IS;
     }
 
     session->driver.nmea.await = atoi(field[1]);
     if (sscanf(field[2], "%d", &session->driver.nmea.part) < 1) {
+	gpsd_report(LOG_WARN, "malformed GPGSV - bad part\n");
 	gpsd_zero_satellites(&session->gpsdata);
-	return 0;
+	return ONLINE_IS;
     } else if (session->driver.nmea.part == 1)
 	gpsd_zero_satellites(&session->gpsdata);
 
     for (fldnum = 4; fldnum < count;) {
 	if (session->gpsdata.satellites_visible >= MAXCHANNELS) {
-	    gpsd_report(LOG_ERROR, "internal error - too many satellites!\n");
+	    gpsd_report(LOG_ERROR, "internal error - too many satellites [%d]!\n",
+                    session->gpsdata.satellites_visible);
 	    gpsd_zero_satellites(&session->gpsdata);
 	    break;
 	}
@@ -528,7 +595,7 @@ static gps_mask_t processGPGSV(int count, char *field[],
     if (session->driver.nmea.part < session->driver.nmea.await) {
 	gpsd_report(LOG_PROG, "Partial satellite data (%d of %d).\n",
 		    session->driver.nmea.part, session->driver.nmea.await);
-	return 0;
+	return ONLINE_IS;
     }
     /*
      * This sanity check catches an odd behavior of SiRFstarII receivers.
@@ -544,7 +611,7 @@ static gps_mask_t processGPGSV(int count, char *field[],
     gpsd_report(LOG_WARN, "Satellite data no good (%d of %d).\n",
 		session->driver.nmea.part, session->driver.nmea.await);
     gpsd_zero_satellites(&session->gpsdata);
-    return 0;
+    return ONLINE_IS;
   sane:
     session->gpsdata.skyview_time = NAN;
     gpsd_report(LOG_DATA, "GSV: Satellite data OK (%d of %d).\n",
@@ -658,7 +725,7 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[],
 	|| field[4][0] == '\0') {
 	gpsd_report(LOG_WARN, "malformed ZDA\n");
     } else {
-    	int year, mon, mday;
+    	int year, mon, mday, century;
 
 	merge_hhmmss(field[1], session);
 	/*
@@ -670,6 +737,7 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[],
 	year = atoi(field[4]);
 	mon = atoi(field[3]);
 	mday = atoi(field[2]);
+	century = year - year % 100;
 	if ( (1900 > year ) || (2200 < year ) ) {
 	    gpsd_report(LOG_WARN, "malformed ZDA year: %s\n",  field[4]);
 	} else if ( (1 > mon ) || (12 < mon ) ) {
@@ -677,8 +745,24 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[],
 	} else if ( (1 > mday ) || (31 < mday ) ) {
 	    gpsd_report(LOG_WARN, "malformed ZDA day: %s\n",  field[2]);
 	} else {
-	    year -= 1900;	
-	    session->driver.nmea.date.tm_year = year;
+	    if (century > session->context->century) {
+		/*
+		 * This mismatch is almost certainly not due to a GPS week
+		 * rollover, because that would throw the ZDA report backward
+		 * into the last rollover period instead of forward.  Almost
+		 * certainly it means that a century mark has passed while
+		 * gpsd was running, and we should trust the new ZDA year.
+		 */
+		gpsd_report(LOG_WARN, "century rollover detected.\n");
+		session->context->century = century;
+	    } else if (session->context->start_time >= GPS_EPOCH && century < session->context->century) {
+		/*
+		 * This looks like a GPS week-counter rollover.
+		 */
+		gpsd_report(LOG_WARN, "ZDA year %d less than clock year, "  
+			    "probable GPS week rollover lossage\n", year);
+	    }
+	    session->driver.nmea.date.tm_year = year - 1900;
 	    session->driver.nmea.date.tm_mon = mon - 1;
 	    session->driver.nmea.date.tm_mday = mday;
 	    mask = TIME_IS;
@@ -926,12 +1010,13 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 	{
 	"RMC", 8, processGPRMC}, {
 	"GGA", 13, processGPGGA}, {
+        "GST", 8, processGPGST}, {
 	"GLL", 7, processGPGLL}, {
 	"GSA", 17, processGPGSA}, {
 	"GSV", 0, processGPGSV}, {
 	"VTG", 0, NULL},	/* ignore Velocity Track made Good */
 	{
-	"ZDA", 7, processGPZDA}, {
+	"ZDA", 4, processGPZDA}, {
 	"GBS", 7, processGPGBS},
 #ifdef TNT_ENABLE
 	{
@@ -963,7 +1048,8 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
      * packets.  This may be a generic bug of all Garmin chipsets.
      */
     if (strlen(sentence) > NMEA_MAX) {
-	gpsd_report(LOG_WARN, "Overlong packet rejected.\n");
+	gpsd_report(LOG_WARN, "Overlong packet of %zd chars rejected.\n",
+		    strlen(sentence));
 	return ONLINE_IS;
     }
 
@@ -1020,7 +1106,7 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 					      session);
 		strncpy(session->gpsdata.tag, nmea_phrase[i].name, MAXTAGLEN);
 		/*
-		 * Must force this to be nz, as we're gong to rely on a zero
+		 * Must force this to be nz, as we're going to rely on a zero
 		 * value to mean "no previous tag" later.
 		 */
 		thistag = i + 1;
@@ -1030,31 +1116,22 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 	}
     }
 
-    /* general handler for MKT3301 vendor specifics */
-#ifdef MKT3301_ENABLE
-    if (strncmp("PMTK", session->driver.nmea.field[0], 4) == 0)
-	retval = processMKT3301(count, session->driver.nmea.field, session);
-#endif /* MKT3301_ENABLE */
-    /*@ +usedef @*/
-
     /* timestamp recording for fixes happens here */
     if ((retval & TIME_IS) != 0) {
+	session->newdata.time = gpsd_utc_resolve(session);
 	/*
 	 * WARNING: This assumes time is always field 0, and that field 0
 	 * is a timestamp whenever TIME_IS is set.
 	 */
-	session->newdata.time =
-	    (double)mkgmtime(&session->driver.nmea.date) +
-	    session->driver.nmea.subseconds;
 	gpsd_report(LOG_DATA,
-		    "%s time (nearest sec) is %2f = %d-%d-%dT%d:%d%d\n",
+		    "%s time is %2f = %d-%02d-%02dT%02d:%02d:%02.2fZ\n",
 		    session->driver.nmea.field[0], session->newdata.time,
 		    1900 + session->driver.nmea.date.tm_year,
 		    session->driver.nmea.date.tm_mon + 1,
 		    session->driver.nmea.date.tm_mday,
 		    session->driver.nmea.date.tm_hour,
 		    session->driver.nmea.date.tm_min,
-		    session->driver.nmea.date.tm_sec);
+		    session->driver.nmea.date.tm_sec + session->driver.nmea.subseconds);
     }
 
     /*
