@@ -12,7 +12,9 @@
  * messages that gpsd needs. Some protocols transmit error estimates
  * separately from the navigation solution; if developing a driver for
  * such a protocol you will need to add a decoder function for that
- * message.
+ * message. Be extra careful when using sizeof(<type>) to extract part
+ * of packets (ie. don't do it). This idiom creates portability problems
+ * between 32 and 64 bit systems.
  *
  * For anyone hacking this driver skeleton: "_PROTO_" and "_proto_" are now
  * reserved tokens. We suggest that they only ever be used as prefixes,
@@ -33,20 +35,10 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-
-#include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
-#include <math.h>
-#include <ctype.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <time.h>
-#include <stdio.h>
 
-#include "gpsd.h"
 #if defined(_PROTO__ENABLE) && defined(BINARY_ENABLE)
 
 #include "bits.h"
@@ -91,7 +83,7 @@ _proto__msg_navsol(struct gps_device_t *session, unsigned char *buf, size_t data
     /* extract ECEF navigation solution here */
     /* or extract the local tangential plane (ENU) solution */
     [Px, Py, Pz, Vx, Vy, Vz] = GET_ECEF_FIX();
-    ecef_to_wgs84fix(&session->newdata,  &session->separation,
+    ecef_to_wgs84fix(&session->newdata,  &session->gpsdata.separation,
 		     Px, Py, Pz, Vx, Vy, Vz);
     mask |= LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | CLIMB_IS  ;
 
@@ -159,13 +151,8 @@ _proto__msg_utctime(struct gps_device_t *session, unsigned char *buf, size_t dat
 
     tow = GET_MS_TIMEOFWEEK();
     gps_week = GET_WEEKNUMBER();
-    session->context->gps_week = gps_week;
     session->context->leap_seconds = GET_GPS_LEAPSECONDS();
-    session->context->gps_tow = tow / 1000.0;
-
-    t = gpstime_to_unix(gps_week, session->context->gps_tow)
-	- session->context->leap_seconds;
-    session->newdata.time = t;
+    session->newdata.time = gpsd_gpstime_resolve(session, gps_week, tow / 1000.0);
 
     return TIME_IS | ONLINE_IS;
 }
@@ -300,7 +287,7 @@ gps_mask_t _proto__dispatch(struct gps_device_t *session, unsigned char *buf, si
 
     /* we may need to dump the raw packet */
     gpsd_report(LOG_RAW, "raw _proto_ packet type 0x%02x length %d: %s\n",
-	type, len, gpsd_hexdump_wrapper(buf, len, LOG_WARN));
+	type, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
 
    /*
     * The tag field is only 8 bytes; be careful you do not overflow.
@@ -376,12 +363,15 @@ static ssize_t _proto__control_send(struct gps_device_t *session,
 #ifdef ALLOW_RECONFIGURE
 static void _proto__event_hook(struct gps_device_t *session, event_t event)
 {
+    if (session->context->readonly)
+	return;
+
     if (event == event_wakeup) {
        /*
-	* Code to make the device ready to communicate. This is
-	* run every time we are about to try a different baud
-	* rate in the autobaud sequence. Only needed if the
-	* device is in some kind of sleeping state.
+	* Code to make the device ready to communicate.  Only needed if the
+	* device is in some kind of sleeping state, and only shipped to
+	* RS232C (so that gpsd won't send strings to unidentified USB devices)
+	* that might not be GPSes at all.
 	*/
     }
     if (event == event_identified) {
@@ -523,6 +513,8 @@ const struct gps_type_t _proto__binary = {
     .type_name        = "_proto_ binary",
     /* Associated lexer packet type */
     .packet_type      = _PROTO__PACKET,
+    /* Driver tyoe flags */
+    .flags	      = DRIVER_NOFLAGS,
     /* Response string that identifies device (not active) */
     .trigger          = NULL,
     /* Number of satellite channels supported by the device */
