@@ -27,21 +27,20 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
+#include <strings.h>
 #include <math.h>
 #include <ctype.h>
+#include <termios.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#include <time.h>
-#include <stdio.h>
 
 #include "gpsd.h"
 #include "bits.h"
-#include "timebase.h"
 #if defined(SIRF_ENABLE) && defined(BINARY_ENABLE)
 
 #define HI(n)		((n) >> 8)
@@ -134,17 +133,15 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *, unsigned char *,
 				   size_t);
 static gps_mask_t sirf_msg_svinfo(struct gps_device_t *, unsigned char *,
 				  size_t);
-#ifdef ALLOW_RECONFIGURE
 static gps_mask_t sirf_msg_swversion(struct gps_device_t *, unsigned char *,
 				     size_t);
 static gps_mask_t sirf_msg_sysparam(struct gps_device_t *, unsigned char *,
 				    size_t);
-#endif /* ALLOW_RECONFIGURE */
 static gps_mask_t sirf_msg_ublox(struct gps_device_t *, unsigned char *,
 				 size_t);
 
 
-static bool sirf_write(int fd, unsigned char *msg)
+static bool sirf_write(struct gps_device_t *session, unsigned char *msg)
 {
     unsigned int crc;
     size_t i, len;
@@ -162,13 +159,8 @@ static bool sirf_write(int fd, unsigned char *msg)
     msg[len + 4] = (unsigned char)((crc & 0xff00) >> 8);
     msg[len + 5] = (unsigned char)(crc & 0x00ff);
 
-    gpsd_report(LOG_IO, "SiRF: Writing control type %02x:%s\n", msg[4],
-		gpsd_hexdump_wrapper(msg, len + 8, LOG_IO));
-    ok = (write(fd, msg, len + 8) == (ssize_t) (len + 8));
-    if (!ok) {
-	gpsd_report(LOG_WARN, "SiRF: Writing error.\n");
-    }
-    (void)tcdrain(fd);
+    gpsd_report(LOG_IO, "SiRF: Writing control type %02x:\n", msg[4]);
+    ok = (gpsd_write(session, (const char *)msg, len+8) == (ssize_t) (len+8));
     return (ok);
 }
 
@@ -187,7 +179,7 @@ static ssize_t sirf_control_send(struct gps_device_t *session, char *msg,
     session->msgbuflen = len + 8;
 
     /* *INDENT-OFF* */
-    return sirf_write(session->gpsdata.gps_fd,
+    return sirf_write(session,
 	      (unsigned char *)session->msgbuf) ? (int)session->msgbuflen : -1;
     /* *INDENT-ON* */
     /*@ -charint -matchanyintegral +initallelements +mayaliasunique @*/
@@ -195,7 +187,7 @@ static ssize_t sirf_control_send(struct gps_device_t *session, char *msg,
 #endif /* ALLOW_CONTROLSEND */
 
 #ifdef ALLOW_RECONFIGURE
-static bool sirf_speed(int ttyfd, speed_t speed, char parity, int stopbits)
+static bool sirfbin_speed(struct gps_device_t *session, speed_t speed, char parity, int stopbits)
 /* change speed in binary mode */
 {
     /*@ +charint @*/
@@ -214,8 +206,8 @@ static bool sirf_speed(int ttyfd, speed_t speed, char parity, int stopbits)
     /*@ -charint @*/
     gpsd_report(LOG_PROG, "SiRF: sirf_speed(%d,%c,%d)\n",
 		speed, parity, stopbits);
-    if (9600 > speed) {
-	gpsd_report(LOG_WARN, "NTPD: SiRF may lag at less than 9600bps\n");
+    if (9600 >= speed) {
+	gpsd_report(LOG_WARN, "SiRF may lag at 9600bps or less.\n");
     }
 
     switch (parity) {
@@ -237,10 +229,10 @@ static bool sirf_speed(int ttyfd, speed_t speed, char parity, int stopbits)
     msg[8] = (unsigned char)LO(speed);
     msg[10] = (unsigned char)stopbits;
     msg[11] = (unsigned char)parity;
-    return (sirf_write(ttyfd, msg));
+    return (sirf_write(session, msg));
 }
 
-static bool sirf_to_nmea(int ttyfd, speed_t speed)
+static bool sirf_to_nmea(struct gps_device_t *session, speed_t speed)
 /* switch from binary to NMEA at specified baud */
 {
     /*@ +charint @*/
@@ -264,15 +256,14 @@ static bool sirf_to_nmea(int ttyfd, speed_t speed)
 
     msg[26] = (unsigned char)HI(speed);
     msg[27] = (unsigned char)LO(speed);
-    return (sirf_write(ttyfd, msg));
+    return (sirf_write(session, msg));
 }
 
 static void sirfbin_mode(struct gps_device_t *session, int mode)
 {
     char parity = '0';
     if (mode == MODE_NMEA) {
-	(void)sirf_to_nmea(session->gpsdata.gps_fd,
-			   session->gpsdata.dev.baudrate);
+	(void)sirf_to_nmea(session, session->gpsdata.dev.baudrate);
     } else if (mode == MODE_BINARY) {
 	switch (session->gpsdata.dev.parity) {
 	default:
@@ -288,7 +279,7 @@ static void sirfbin_mode(struct gps_device_t *session, int mode)
 
 	}
 	// gpsd only supports 8[NO]1 or 7[EO]2
-	// thus the strange us of stopbits
+	// thus the strange use of stopbits
 	(void)nmea_send(session,
 			"$PSRF100,0,%d,%d,%d,%c",
 			session->gpsdata.dev.baudrate,
@@ -351,11 +342,11 @@ static gps_mask_t sirf_msg_debug(unsigned char *buf, size_t len)
 
 static gps_mask_t sirf_msg_errors(unsigned char *buf, size_t len UNUSED)
 {
-    switch (getbeuw(buf, 1)) {
+    switch (getbeu16(buf, 1)) {
     case 2:
 	gpsd_report(LOG_PROG,
 		    "SiRF: EID 0x0a type 2: Subframe %u error on PRN %u\n",
-		    getbeul(buf, 9), getbeul(buf, 5));
+		    getbeu32(buf, 9), getbeu32(buf, 5));
 	break;
 
     case 4107:
@@ -365,15 +356,15 @@ static gps_mask_t sirf_msg_errors(unsigned char *buf, size_t len UNUSED)
 
     default:
 	gpsd_report(LOG_PROG, "SiRF: EID 0x0a: Error MID %d\n",
-		    getbeuw(buf, 1));
+		    getbeu16(buf, 1));
 	break;
     }
     return 0;
 }
 
 /* Navigation Library Measurement Data MID 28 */
-static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session,
-				unsigned char *buf, size_t len)
+static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session UNUSED,
+				unsigned char *buf UNUSED, size_t len)
 {
 
     double gps_tow = 0.0;
@@ -383,7 +374,7 @@ static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session,
 
     /* oh barf, SiRF claims to be IEEE754 but supports two
      * different double orders, neither IEEE754 */
-    /* Todo - decode the time, since this is the first MID with a
+    /* FIXME - decode the time, since this is the first MID with a
      * good time stamp this will be good for ntpshm time */
     gpsd_report(LOG_PROG, "SiRF: MID 0x1c, NLMD, gps_tow: %f, %s\n",
 		(double)gps_tow, gpsd_hexdump_wrapper(&gps_tow, 8, LOG_PROG));
@@ -391,18 +382,20 @@ static gps_mask_t sirf_msg_nlmd(struct gps_device_t *session,
     return 0;
 }
 
-#ifdef ALLOW_RECONFIGURE
 static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
 				     unsigned char *buf, size_t len)
 {
     double fv;
+    unsigned char *cp;
 
     if (len < 20)
 	return 0;
 
     (void)strlcpy(session->subtype, (char *)buf + 1,
 		  sizeof(session->subtype));
-    fv = atof((char *)(buf + 1));
+    for (cp = buf+1; *cp!=(unsigned char)'\0' && isdigit(*cp)==0; cp++)
+	continue;
+    fv = atof((const char *)cp);
     if (fv < 231) {
 	session->driver.sirf.driverstate |= SIRF_LT_231;
 	if (fv > 200)
@@ -410,11 +403,15 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
     } else if (fv < 232) {
 	session->driver.sirf.driverstate |= SIRF_EQ_231;
     } else {
-	gpsd_report(LOG_PROG, "SiRF: Enabling PPS message...\n");
-	(void)sirf_write(session->gpsdata.gps_fd, enablemid52);
 	session->driver.sirf.driverstate |= SIRF_GE_232;
-	session->context->valid |= LEAP_SECOND_VALID;
     }
+#ifdef ALLOW_RECONFIGURE
+    if (!session->context->readonly) {
+	gpsd_report(LOG_PROG, "SiRF: Enabling PPS message...\n");
+	(void)sirf_write(session, enablemid52);
+    }
+#endif /* ALLOW_RECONFIGURE */
+
     if (strstr((char *)(buf + 1), "ES"))
 	gpsd_report(LOG_INF, "SiRF: Firmware has XTrac capability\n");
     gpsd_report(LOG_PROG, "SiRF: fv: %0.2f, Driver state flags are: %0x\n",
@@ -422,20 +419,23 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
 #ifdef NTPSHM_ENABLE
     session->driver.sirf.time_seen = 0;
 #endif /* NTPSHM_ENABLE */
-    if (session->gpsdata.dev.baudrate >= 38400) {
+#ifdef ALLOW_RECONFIGURE
+    if (!session->context->readonly && session->gpsdata.dev.baudrate >= 38400) {
+        /* some USB devices are also too slow, no way to tell which ones */
 	gpsd_report(LOG_PROG, "SiRF: Enabling subframe transmission...\n");
-	(void)sirf_write(session->gpsdata.gps_fd, enablesubframe);
+	(void)sirf_write(session, enablesubframe);
     }
-    gpsd_report(LOG_DATA, "SiRF: FV 0x06: subtype='%s' mask={DEVICEID}\n",
+#endif /* ALLOW_RECONFIGURE */
+    gpsd_report(LOG_DATA, "SiRF: FV MID 0x06: subtype='%s' mask={DEVICEID}\n",
 		session->subtype);
     return DEVICEID_IS;
 }
-#endif /* ALLOW_RECONFIGURE */
 
 static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
 				   unsigned char *buf, size_t len)
 {
-    unsigned int i, words[10], chan, svid;
+    unsigned int i, chan, svid;
+    uint32_t words[10];
 
     if (len != 43)
 	return 0;
@@ -444,18 +444,20 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
     svid = (unsigned int)getub(buf, 2);
 
     for (i = 0; i < 10; i++) {
-	words[i] = (unsigned int)getbeul(buf, 4 * i + 3);
+	words[i] = (uint32_t)getbeu32(buf, 4 * i + 3);
     }
 
-    (void)gpsd_interpret_subframe_raw(session, words);
 
 #ifdef ALLOW_RECONFIGURE
-    if (session->gpsdata.dev.baudrate < 38400) {
-	gpsd_report(LOG_PROG, "SiRF: Disabling subframe transmission...\n");
-	(void)sirf_write(session->gpsdata.gps_fd, disablesubframe);
+    if (!session->context->readonly && session->gpsdata.dev.baudrate < 38400) {
+        /* some USB are also too slow, no way to tell which ones */
+	gpsd_report(LOG_WARN, 
+		"WARNING: SiRF: link too slow, disabling subframes.\n");
+	(void)sirf_write(session, disablesubframe);
     }
 #endif /* ALLOW_RECONFIGURE */
-    return 0;
+
+    return gpsd_interpret_subframe_raw(session, svid, words);
 }
 
 #define SIRF_CHANNELS	12	/* max channels allowed in SiRF format */
@@ -469,15 +471,11 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
     if (len != 188)
 	return 0;
 
+    session->gpsdata.skyview_time = gpsd_gpstime_resolve(session,
+	(unsigned short)getbes16(buf, 1),
+	(unsigned int)getbeu32(buf, 3) * 1e-2);
+
     gpsd_zero_satellites(&session->gpsdata);
-    session->context->gps_week = (unsigned short)getbesw(buf, 1);
-    session->context->gps_tow = (double)getbeul(buf, 3) * 1e-2;
-    /*@ ignore @*//*@ splint is confused @ */
-    session->gpsdata.skyview_time
-	=
-	gpstime_to_unix(session->context->gps_week, session->context->gps_tow)
-	- session->context->leap_seconds;
-    /*@ end @*/
     for (i = st = 0; i < SIRF_CHANNELS; i++) {
 	int off = 8 + 15 * i;
 	bool good;
@@ -500,7 +498,7 @@ static gps_mask_t sirf_msg_svinfo(struct gps_device_t *session,
 		    getub(buf, off),
 		    getub(buf, off + 2) / 2.0,
 		    (getub(buf, off + 1) * 3) / 2.0,
-		    cn / 10, getbeuw(buf, off + 3), good ? '*' : ' ');
+		    cn / 10, getbeu16(buf, off + 3), good ? '*' : ' ');
 #endif /* UNUSED */
 	if (good != 0)
 	    st += 1;
@@ -606,9 +604,12 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
 	session->gpsdata.used[i] = (int)getub(buf, 29 + i);
     /* position/velocity is bytes 1-18 */
     ecef_to_wgs84fix(&session->newdata, &session->gpsdata.separation,
-		     getbesl(buf, 1) * 1.0, getbesl(buf, 5) * 1.0,
-		     getbesl(buf, 9) * 1.0, getbesw(buf, 13) / 8.0,
-		     getbesw(buf, 15) / 8.0, getbesw(buf, 17) / 8.0);
+		     (double)getbes32(buf, 1) * 1.0, 
+		     (double)getbes32(buf, 5) * 1.0,
+		     (double)getbes32(buf, 9) * 1.0, 
+		     (double)getbes16(buf, 13) / 8.0,
+		     (double)getbes16(buf, 15) / 8.0, 
+		     (double)getbes16(buf, 17) / 8.0);
     /* fix status is byte 19 */
     navtype = (unsigned short)getub(buf, 19);
     session->gpsdata.status = STATUS_NO_FIX;
@@ -628,14 +629,8 @@ static gps_mask_t sirf_msg_navsol(struct gps_device_t *session,
 		navtype, session->gpsdata.status, session->newdata.mode);
     /* byte 20 is HDOP, see below */
     /* byte 21 is "mode 2", not clear how to interpret that */
-    session->context->gps_week = (unsigned short)getbesw(buf, 22);
-    session->context->gps_tow = (double)getbeul(buf, 24) * 1e-2;
-    /*@ ignore @*//*@ splint is confused @ */
-    session->newdata.time =
-	gpstime_to_unix(session->context->gps_week,
-			session->context->gps_tow) -
-	session->context->leap_seconds;
-    /*@ end @*/
+    session->newdata.time = gpsd_gpstime_resolve(session,
+	(unsigned short)getbes16(buf, 22), (double)getbeu32(buf, 24) * 1e-2);
 #ifdef NTPSHM_ENABLE
     if (session->newdata.mode <= MODE_NO_FIX) {
 	gpsd_report(LOG_PROG, "SiRF: NTPD no fix, mode: %d\n",
@@ -705,7 +700,7 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
     session->gpsdata.sentence_length = 91;
     (void)strlcpy(session->gpsdata.tag, "GND", MAXTAGLEN + 1);
 
-    navtype = (unsigned short)getbeuw(buf, 3);
+    navtype = (unsigned short)getbeu16(buf, 3);
     session->gpsdata.status = STATUS_NO_FIX;
     session->newdata.mode = MODE_NO_FIX;
     if (navtype & 0x80)
@@ -722,18 +717,18 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 		navtype, session->gpsdata.status, session->newdata.mode);
     mask |= STATUS_IS | MODE_IS;
 
-    session->newdata.latitude = getbesl(buf, 23) * 1e-7;
-    session->newdata.longitude = getbesl(buf, 27) * 1e-7;
+    session->newdata.latitude = getbes32(buf, 23) * 1e-7;
+    session->newdata.longitude = getbes32(buf, 27) * 1e-7;
     if (session->newdata.latitude != 0 && session->newdata.latitude != 0)
 	mask |= LATLON_IS;
 
-    if ((eph = getbesl(buf, 50) * 1e-2) > 0) {
+    if ((eph = getbes32(buf, 50) * 1e-2) > 0) {
 	session->newdata.epx = session->newdata.epy = eph / sqrt(2);
 	mask |= HERR_IS;
     }
-    if ((session->newdata.epv = getbesl(buf, 54) * 1e-2) > 0)
+    if ((session->newdata.epv = getbes32(buf, 54) * 1e-2) > 0)
 	mask |= VERR_IS;
-    if ((session->newdata.eps = getbesw(buf, 62) * 1e-2) > 0)
+    if ((session->newdata.eps = getbes16(buf, 62) * 1e-2) > 0)
 	mask |= SPEEDERR_IS;
 
     /* HDOP should be available at byte 89, but in 231 it's zero. */
@@ -775,15 +770,15 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 	 * Documentation of this field was corrected in the 1.6 version
 	 * of the protocol manual.
 	 */
-	unpacked_date.tm_year = (int)getbeuw(buf, 11) - 1900;
+	unpacked_date.tm_year = (int)getbeu16(buf, 11) - 1900;
 	unpacked_date.tm_mon = (int)getub(buf, 13) - 1;
 	unpacked_date.tm_mday = (int)getub(buf, 14);
 	unpacked_date.tm_hour = (int)getub(buf, 15);
 	unpacked_date.tm_min = (int)getub(buf, 16);
 	unpacked_date.tm_sec = 0;
-	subseconds = getbeuw(buf, 17) * 1e-3;
+	subseconds = getbeu16(buf, 17) * 1e-3;
 	/*@ -compdef -unrecog */
-	session->newdata.time = (double)timegm(&unpacked_date) + subseconds;
+	session->newdata.time = (timestamp_t)timegm(&unpacked_date) + subseconds;
 	/*@ +compdef +unrecog */
 	gpsd_report(LOG_PROG, "SiRF: GND 0x29 UTC: %lf\n",
 		    session->newdata.time);
@@ -806,12 +801,12 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 
 #endif /* NTPSHM_ENABLE */
 	/* skip 4 bytes of satellite map */
-	session->newdata.altitude = getbesl(buf, 35) * 1e-2;
+	session->newdata.altitude = getbes32(buf, 35) * 1e-2;
 	/* skip 1 byte of map datum */
-	session->newdata.speed = getbeuw(buf, 40) * 1e-2;
-	session->newdata.track = getbeuw(buf, 42) * 1e-2;
+	session->newdata.speed = getbeu16(buf, 40) * 1e-2;
+	session->newdata.track = getbeu16(buf, 42) * 1e-2;
 	/* skip 2 bytes of magnetic variation */
-	session->newdata.climb = getbesw(buf, 46) * 1e-2;
+	session->newdata.climb = getbes16(buf, 46) * 1e-2;
 	mask |= TIME_IS | SPEED_IS | TRACK_IS;
 	if (session->newdata.mode == MODE_3D)
 	    mask |= ALTITUDE_IS | CLIMB_IS;
@@ -830,7 +825,6 @@ static gps_mask_t sirf_msg_geodetic(struct gps_device_t *session,
 }
 #endif /* __UNUSED__ */
 
-#ifdef ALLOW_RECONFIGURE
 static gps_mask_t sirf_msg_sysparam(struct gps_device_t *session,
 				    unsigned char *buf, size_t len)
 {
@@ -842,16 +836,19 @@ static gps_mask_t sirf_msg_sysparam(struct gps_device_t *session,
     session->driver.sirf.nav_parameters_seen = true;
     session->driver.sirf.altitude_hold_mode = (unsigned char)getub(buf, 5);
     session->driver.sirf.altitude_hold_source = (unsigned char)getub(buf, 6);
-    session->driver.sirf.altitude_source_input = getbesw(buf, 7);
+    session->driver.sirf.altitude_source_input = getbes16(buf, 7);
     session->driver.sirf.degraded_mode = (unsigned char)getub(buf, 9);
     session->driver.sirf.degraded_timeout = (unsigned char)getub(buf, 10);
     session->driver.sirf.dr_timeout = (unsigned char)getub(buf, 11);
     session->driver.sirf.track_smooth_mode = (unsigned char)getub(buf, 12);
-    gpsd_report(LOG_PROG, "SiRF: Setting Navigation Parameters\n");
-    (void)sirf_write(session->gpsdata.gps_fd, modecontrol);
+#ifdef ALLOW_RECONFIGURE
+    if (!session->context->readonly) {
+	gpsd_report(LOG_PROG, "SiRF: Setting Navigation Parameters\n");
+	(void)sirf_write(session, modecontrol);
+    }
+#endif /* ALLOW_RECONFIGURE */
     return 0;
 }
-#endif /* ALLOW_RECONFIGURE */
 
 static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 				 unsigned char *buf, size_t len UNUSED)
@@ -865,16 +862,16 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
     /* this packet is only sent by uBlox firmware from version 1.32 */
     mask = LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | CLIMB_IS |
 	STATUS_IS | MODE_IS | DOP_IS;
-    session->newdata.latitude = getbesl(buf, 1) * RAD_2_DEG * 1e-8;
-    session->newdata.longitude = getbesl(buf, 5) * RAD_2_DEG * 1e-8;
+    session->newdata.latitude = (double)getbes32(buf, 1) * RAD_2_DEG * 1e-8;
+    session->newdata.longitude = (double)getbes32(buf, 5) * RAD_2_DEG * 1e-8;
     session->gpsdata.separation =
 	wgs84_separation(session->newdata.latitude,
 			 session->newdata.longitude);
     session->newdata.altitude =
-	getbesl(buf, 9) * 1e-3 - session->gpsdata.separation;
-    session->newdata.speed = getbesl(buf, 13) * 1e-3;
-    session->newdata.climb = getbesl(buf, 17) * 1e-3;
-    session->newdata.track = getbesl(buf, 21) * RAD_2_DEG * 1e-8;
+	(double)getbes32(buf, 9) * 1e-3 - session->gpsdata.separation;
+    session->newdata.speed = (double)getbes32(buf, 13) * 1e-3;
+    session->newdata.climb = (double)getbes32(buf, 17) * 1e-3;
+    session->newdata.track = (double)getbes32(buf, 21) * RAD_2_DEG * 1e-8;
 
     navtype = (unsigned short)getub(buf, 25);
     session->gpsdata.status = STATUS_NO_FIX;
@@ -895,15 +892,15 @@ static gps_mask_t sirf_msg_ublox(struct gps_device_t *session,
 	struct tm unpacked_date;
 	double subseconds;
 	mask |= TIME_IS;
-	unpacked_date.tm_year = (int)getbeuw(buf, 26) - 1900;
+	unpacked_date.tm_year = (int)getbeu16(buf, 26) - 1900;
 	unpacked_date.tm_mon = (int)getub(buf, 28) - 1;
 	unpacked_date.tm_mday = (int)getub(buf, 29);
 	unpacked_date.tm_hour = (int)getub(buf, 30);
 	unpacked_date.tm_min = (int)getub(buf, 31);
 	unpacked_date.tm_sec = 0;
-	subseconds = ((unsigned short)getbeuw(buf, 32)) * 1e-3;
+	subseconds = ((unsigned short)getbeu16(buf, 32)) * 1e-3;
 	/*@ -compdef */
-	session->newdata.time = (double)mkgmtime(&unpacked_date) + subseconds;
+	session->newdata.time = (timestamp_t)mkgmtime(&unpacked_date) + subseconds;
 	/*@ +compdef */
 #ifdef NTPSHM_ENABLE
 	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_UTC_2)) {
@@ -953,20 +950,12 @@ static gps_mask_t sirf_msg_ppstime(struct gps_device_t *session,
 	unpacked_date.tm_sec = (int)getub(buf, 3);
 	unpacked_date.tm_mday = (int)getub(buf, 4);
 	unpacked_date.tm_mon = (int)getub(buf, 5) - 1;
-	unpacked_date.tm_year = (int)getbeuw(buf, 6) - 1900;
+	unpacked_date.tm_year = (int)getbeu16(buf, 6) - 1900;
 	/*@ -compdef */
-	session->newdata.time = (double)mkgmtime(&unpacked_date);
+	session->newdata.time = (timestamp_t)mkgmtime(&unpacked_date);
 	/*@ +compdef */
-	session->context->leap_seconds = (int)getbeuw(buf, 8);
-	if (LEAP_SECONDS > session->context->leap_seconds) {
-	    /* something wrong */
-	    gpsd_report(LOG_ERROR, "SiRF: Invalid leap_seconds: %d\n",
-			session->context->leap_seconds);
-	    session->context->leap_seconds = LEAP_SECONDS;
-	    session->context->valid &= ~LEAP_SECOND_VALID;
-	} else {
-	    session->context->valid |= LEAP_SECOND_VALID;
-	}
+	session->context->leap_seconds = (int)getbeu16(buf, 8);
+	session->context->valid |= LEAP_SECOND_VALID;
 #ifdef NTPSHM_ENABLE
 	if (0 == (session->driver.sirf.time_seen & TIME_SEEN_UTC_2)) {
 	    gpsd_report(LOG_RAW, "SiRF: NTPD just SEEN_UTC_2\n");
@@ -1015,10 +1004,8 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 	gpsd_report(LOG_PROG, "SiRF: unused Raw Tracker Data 0x05\n");
 	return 0;
 
-#ifdef ALLOW_RECONFIGURE
     case 0x06:			/* Software Version String MID 6 */
 	return sirf_msg_swversion(session, buf, len);
-#endif /* ALLOW_RECONFIGURE */
 
     case 0x07:			/* Clock Status Data MID 7 */
 	gpsd_report(LOG_PROG, "SiRF: unused CLK 0x07\n");
@@ -1041,9 +1028,9 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
     case 0x09:			/* CPU Throughput MID 9 */
 	gpsd_report(LOG_PROG,
 		    "SiRF: THR 0x09: SegStatMax=%.3f, SegStatLat=%3.f, AveTrkTime=%.3f, Last MS=%u\n",
-		    (float)getbeuw(buf, 1) / 186, (float)getbeuw(buf,
+		    (float)getbeu16(buf, 1) / 186, (float)getbeu16(buf,
 								 3) / 186,
-		    (float)getbeuw(buf, 5) / 186, getbeuw(buf, 7));
+		    (float)getbeu16(buf, 5) / 186, getbeu16(buf, 7));
 	return 0;
 
     case 0x0a:			/* Error ID Data MID 10 */
@@ -1081,10 +1068,8 @@ gps_mask_t sirf_parse(struct gps_device_t * session, unsigned char *buf,
 		    getub(buf, 1));
 	return 0;
 
-#ifdef ALLOW_RECONFIGURE
     case 0x13:			/* Navigation Parameters MID 19 */
 	return sirf_msg_sysparam(session, buf, len);
-#endif /* ALLOW_RECONFIGURE */
 
     case 0x1b:			/* DGPS status (undocumented) MID 27 */
 	gpsd_report(LOG_PROG, "SiRF: unused DGPSF 0x1b %s\n",
@@ -1189,6 +1174,9 @@ static gps_mask_t sirfbin_parse_input(struct gps_device_t *session)
 
 static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 {
+    if (session->context->readonly)
+	return;
+
     if (event == event_identified || event == event_reactivate) {
 	if (session->packet.type == NMEA_PACKET) {
 	    gpsd_report(LOG_PROG, "SiRF: Switching chip mode to binary.\n");
@@ -1285,30 +1273,30 @@ static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 			session->gpsdata.dev.baudrate);
 	    (void)usleep(3330);	/* guessed settling time */
 	    gpsd_report(LOG_PROG, "SiRF: unset MID 30...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, unsetmid30);
+	    (void)sirf_write(session, unsetmid30);
 	    (void)usleep(3330);	/* guessed settling time */
 
 	    gpsd_report(LOG_PROG,
 			"SiRF: Requesting periodic ecef reports...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, requestecef);
+	    (void)sirf_write(session, requestecef);
 	    gpsd_report(LOG_PROG,
 			"SiRF: Requesting periodic tracker reports...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, requesttracker);
+	    (void)sirf_write(session, requesttracker);
 	    gpsd_report(LOG_PROG,
 			"SiRF: Setting DGPS control to use SBAS...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, dgpscontrol);
+	    (void)sirf_write(session, dgpscontrol);
 	    gpsd_report(LOG_PROG,
 			"SiRF: Setting SBAS to auto/integrity mode...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, sbasparams);
+	    (void)sirf_write(session, sbasparams);
 
 	    gpsd_report(LOG_PROG, "SiRF: unset MID 29...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, unsetmid29);
+	    (void)sirf_write(session, unsetmid29);
 
 	    gpsd_report(LOG_PROG, "SiRF: Probing for firmware version...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, versionprobe);
+	    (void)sirf_write(session, versionprobe);
 	    gpsd_report(LOG_PROG,
 			"SiRF: Requesting navigation parameters...\n");
-	    (void)sirf_write(session->gpsdata.gps_fd, navparams);
+	    (void)sirf_write(session, navparams);
 	}
     }
     if (event == event_deactivate) {
@@ -1330,7 +1318,7 @@ static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 	};
 	/*@ -charint -shiftimplementation @*/
 	putbyte(moderevert, 7, session->driver.sirf.degraded_mode);
-	putbeword(moderevert, 10, session->driver.sirf.altitude_source_input);
+	putbe16(moderevert, 10, session->driver.sirf.altitude_source_input);
 	putbyte(moderevert, 12, session->driver.sirf.altitude_hold_mode);
 	putbyte(moderevert, 13, session->driver.sirf.altitude_hold_source);
 	putbyte(moderevert, 15, session->driver.sirf.degraded_timeout);
@@ -1338,30 +1326,23 @@ static void sirfbin_event_hook(struct gps_device_t *session, event_t event)
 	putbyte(moderevert, 17, session->driver.sirf.track_smooth_mode);
 	/*@ +shiftimplementation @*/
 	gpsd_report(LOG_PROG, "SiRF: Reverting navigation parameters...\n");
-	(void)sirf_write(session->gpsdata.gps_fd, moderevert);
+	(void)sirf_write(session, moderevert);
     }
 }
 
-#ifdef ALLOW_RECONFIGURE
-static bool sirfbin_speed(struct gps_device_t *session,
-			  speed_t speed, char parity, int stopbits)
-{
-    return sirf_speed(session->gpsdata.gps_fd, speed, parity, stopbits);
-}
-#endif /* ALLOW_RECONFIGURE */
-
 /* this is everything we export */
 /* *INDENT-OFF* */
 const struct gps_type_t sirf_binary =
 {
     .type_name      = "SiRF binary",	/* full name of type */
     .packet_type    = SIRF_PACKET,	/* associated lexer packet type */
+    .flags	    = DRIVER_NOFLAGS,	/* no flags set */
     .trigger	    = NULL,		/* no trigger */
     .channels       = SIRF_CHANNELS,	/* consumer-grade GPS */
     .probe_detect   = NULL,		/* no probe */
     .get_packet     = sirf_get,		/* be prepared for SiRF or NMEA */
     .parse_packet   = sirfbin_parse_input,/* parse message packets */
-    .rtcm_writer    = pass_rtcm,	/* send RTCM data straight */
+    .rtcm_writer    = gpsd_write,	/* send RTCM data straight */
     .event_hook     = sirfbin_event_hook,/* lifetime event handler */
 #ifdef ALLOW_RECONFIGURE
     .speed_switcher = sirfbin_speed,	/* we can change baud rate */
