@@ -2,20 +2,11 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/types.h>
-
-#include "gpsd_config.h"
-
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdbool.h>
+#include <time.h>
 #include <string.h>
 #include <math.h>
-#include <ctype.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <time.h>
-#include <stdio.h>
 
 #include "gpsd.h"
 
@@ -106,8 +97,8 @@ superstar2_msg_navsol_lla(struct gps_device_t *session,
     tm.tm_sec = (int)d;
     tm.tm_mday = (int)getub(buf, 14);
     tm.tm_mon = (int)getub(buf, 15) - 1;
-    tm.tm_year = (int)getleuw(buf, 16) - 1900;
-    session->newdata.time = timegm(&tm) + (d - tm.tm_sec);
+    tm.tm_year = (int)getleu16(buf, 16) - 1900;
+    session->newdata.time = (timestamp_t)timegm(&tm) + (d - tm.tm_sec);
     mask |= TIME_IS;
 
     /* extract the local tangential plane (ENU) solution */
@@ -120,8 +111,8 @@ superstar2_msg_navsol_lla(struct gps_device_t *session,
     mask |= LATLON_IS | ALTITUDE_IS | SPEED_IS | TRACK_IS | CLIMB_IS;
 
     session->gpsdata.satellites_used = (int)getub(buf, 71) & 0x0f;
-    /*@i3@*/ session->gpsdata.dop.hdop = getleuw(buf, 66) * 0.1;
-    /*@i3@*/ session->gpsdata.dop.vdop = getleuw(buf, 68) * 0.1;
+    /*@i3@*/ session->gpsdata.dop.hdop = getleu16(buf, 66) * 0.1;
+    /*@i3@*/ session->gpsdata.dop.vdop = getleu16(buf, 68) * 0.1;
     /* other DOP if available */
     mask |= DOP_IS | USED_IS;
 
@@ -268,13 +259,13 @@ superstar2_msg_timing(struct gps_device_t *session, unsigned char *buf,
 	(void)memset(&tm, '\0', sizeof(tm));
 	tm.tm_mday = (int)getsb(buf, 37);
 	tm.tm_mon = (int)getsb(buf, 38) - 1;
-	tm.tm_year = (int)getlesw(buf, 39) - 1900;
+	tm.tm_year = (int)getles16(buf, 39) - 1900;
 
 	tm.tm_hour = (int)getsb(buf, 41);
 	tm.tm_min = (int)getsb(buf, 42);
 	d = getled(buf, 43);
 	tm.tm_sec = (int)d;
-	session->newdata.time = timegm(&tm);
+	session->newdata.time = (timestamp_t)timegm(&tm);
 	session->context->leap_seconds = (int)getsb(buf, 20);
 	mask = TIME_IS;
     }
@@ -291,7 +282,6 @@ superstar2_msg_measurement(struct gps_device_t *session, unsigned char *buf,
 			   size_t data_len UNUSED)
 {
     gps_mask_t mask = 0;
-#ifdef RAW_ENABLE
     int i, n;
     unsigned long ul;
     double t;
@@ -310,8 +300,8 @@ superstar2_msg_measurement(struct gps_device_t *session, unsigned char *buf,
 	session->gpsdata.PRN[i] = (int)getub(buf, 11 * i + 15) & 0x1f;
 	session->gpsdata.ss[i] = (double)getub(buf, 11 * i * 15 + 1) / 4.0;
 	session->gpsdata.raw.codephase[i] =
-	    (double)getleul(buf, 11 * i * 15 + 2);
-	ul = (unsigned long)getleul(buf, 11 * i * 15 + 6);
+	    (double)getleu32(buf, 11 * i * 15 + 2);
+	ul = (unsigned long)getleu32(buf, 11 * i * 15 + 6);
 
 	session->gpsdata.raw.satstat[i] = (unsigned int)(ul & 0x03L);
 	session->gpsdata.raw.carrierphase[i] = (double)((ul >> 2) & 0x03ffL);
@@ -319,7 +309,6 @@ superstar2_msg_measurement(struct gps_device_t *session, unsigned char *buf,
     }
 
     mask |= RAW_IS;
-#endif /* RAW_ENABLE */
     return mask;
 }
 
@@ -437,43 +426,19 @@ superstar2_dispatch(struct gps_device_t * session, unsigned char *buf,
  * Externally called routines below here
  *
  **********************************************************/
-/*@ +charint @*/
-/* canned config messages */
-/* Initiate Link ID# 63 */
-static unsigned char link_msg[] = { 0x01, 0x3f, 0xc0, 0x08,
-    0x55, 0x47, 0x50, 0x53, 0x2d, 0x30, 0x30, 0x30,
-    0x00, 0x00
-};
-
-/* Request Hardware/Software Identification ID# 45 */
-static unsigned char version_msg[] = { 0x01, 0x2d, 0xd2, 0x00, 0x00, 0x01 };
-
-/*@ -charint @*/
 
 static void superstar2_event_hook(struct gps_device_t *session, event_t event)
 {
-    if (event == event_wakeup) {
-	(void)superstar2_write(session, (char *)link_msg, sizeof(link_msg));
-	(void)usleep(320000);
-	(void)superstar2_write(session, (char *)version_msg,
-			       sizeof(version_msg));
+    if (session->context->readonly)
 	return;
-    }
 
-    /* query firmware version */
-    if (event == event_identified)
-	(void)superstar2_write(session, (char *)version_msg,
-			       sizeof(version_msg));
-
-    /* FIX-ME: check to see if this really needs to be resent on reactivation */
-    if (event == event_identified || event == event_reactivate) {
+    if (event == event_identified) {
 	/*@ +charint @*/
-	unsigned char svinfo_msg[] = { 0x01, 0xa1, 0x5e, 0x00, 0x00, 0x01 };
-	unsigned char timing_msg[] = { 0x01, 0xf1, 0x0e, 0x00, 0x00, 0x01 };
-	unsigned char navsol_lla_msg[] =
-	    { 0x01, 0x94, 0x6b, 0x00, 0x00, 0x01 };
-	unsigned char ephemeris_msg[] =
-	    { 0x01, 0x96, 0x69, 0x00, 0x00, 0x01 };
+	unsigned char version_msg[]    = { 0x01, 0x2d, 0xd2, 0x00, 0x00, 0x01 };
+	unsigned char svinfo_msg[]     = { 0x01, 0xa1, 0x5e, 0x00, 0x00, 0x01 };
+	unsigned char timing_msg[]     = { 0x01, 0xf1, 0x0e, 0x00, 0x00, 0x01 };
+	unsigned char navsol_lla_msg[] = { 0x01, 0x94, 0x6b, 0x00, 0x00, 0x01 };
+	unsigned char ephemeris_msg[]  = { 0x01, 0x96, 0x69, 0x00, 0x00, 0x01 };
 	unsigned char measurement_msg[] =
 	    { 0x01, 0x97, 0x68, 0x01, 0x00, 0x01, 0x01 };
 	/*@ -charint @*/
@@ -581,6 +546,8 @@ const struct gps_type_t superstar2_binary = {
     .type_name		= "SuperStarII binary",
     /* Associated lexer packet type */
     .packet_type        = SUPERSTAR2_PACKET,
+    /* Driver type flags */
+    .flags	         = DRIVER_NOFLAGS,
     /* Response string that identifies device (not active) */
     .trigger		= NULL,
     /* Number of satellite channels supported by the device */
@@ -592,7 +559,7 @@ const struct gps_type_t superstar2_binary = {
     /* Parse message packets */
     .parse_packet	= superstar2_parse_input,
     /* RTCM handler (using default routine) */
-    .rtcm_writer	= pass_rtcm,
+    .rtcm_writer	= gpsd_write,
     /* Fire on various lifetime events */
     .event_hook		= superstar2_event_hook,
 #ifdef ALLOW_RECONFIGURE
