@@ -35,11 +35,18 @@ extern "C" {
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
 #endif
 
-#define GPSD_API_MAJOR_VERSION	4	/* bump on incompatible changes */
-#define GPSD_API_MINOR_VERSION	1	/* bump on compatible changes */
+/*
+ * 4.1 - Base version for initial JSON protocol (Dec 2009, release 2.90)
+ * 4.2 - AIS application IDs split into DAC and FID (July 2010, release 2.95)
+ * 5.0 - MAXCHANNELS bumped from 20 to 32 for GLONASS (July 2010)
+ *       gps_open() becomes reentrant, what gps_open_r() used to be. 
+ *       gps_poll() removed in favor of gps_read().  The raw hook is gone.
+ */
+#define GPSD_API_MAJOR_VERSION	5	/* bump on incompatible changes */
+#define GPSD_API_MINOR_VERSION	0	/* bump on compatible changes */
 
 #define MAXTAGLEN	8	/* maximum length of sentence tag name */
-#define MAXCHANNELS	20	/* maximum GPS channels (*not* satellites!) */
+#define MAXCHANNELS	72	/* must be > 12 GPS + 12 GLONASS + 2 WAAS */
 #define GPS_PRNMAX	32	/* above this number are SBAS satellites */
 #define GPS_PATH_MAX	64	/* dev files usually have short names */
 #define MAXUSERDEVS	4	/* max devices per user */
@@ -70,8 +77,10 @@ extern "C" {
  *
  * Error estimates are at 95% confidence.
  */
+typedef double timestamp_t;	/* Unix time in seconds with fractional part */
+
 struct gps_fix_t {
-    double time;	/* Time of update, seconds since Unix epoch */
+    timestamp_t time;	/* Time of update */
     int    mode;	/* Mode of fix */
 #define MODE_NOT_SEEN	0	/* mode update not seen yet */
 #define MODE_NO_FIX	1	/* none */
@@ -92,6 +101,20 @@ struct gps_fix_t {
     double epc;		/* Vertical speed uncertainty */
 };
 
+/* 
+ * The structure describing the pseudorange errors (GPGST)
+ */
+struct gst_t {
+    double utctime;
+    double rms_deviation;
+    double smajor_deviation;
+    double sminor_deviation;
+    double smajor_orientation;
+    double lat_err_deviation;
+    double lon_err_deviation;
+    double alt_err_deviation;
+};
+
 /*  
  * From the RCTM104 2.x standard:
  *
@@ -142,14 +165,14 @@ struct rtcm2_t {
     union {
 	struct {
 	    unsigned int nentries;
-	    struct rangesat_t {		/* data from messages 1 & 9 */
+	    struct gps_rangesat_t {	/* data from messages 1 & 9 */
 		unsigned ident;		/* satellite ID */
 		unsigned udre;		/* user diff. range error */
-		unsigned issuedata;	/* issue of data */
-		double rangerr;		/* range error */
-		double rangerate;	/* range error rate */
+		unsigned iod;		/* issue of data */
+		double prc;		/* range error */
+		double rrc;		/* range error rate */
 	    } sat[MAXCORRECTIONS];
-	} ranges;
+	} gps_ranges;
 	struct {		/* data for type 3 messages */
 	    bool valid;		/* is message well-formed? */
 	    double x, y, z;
@@ -193,6 +216,28 @@ struct rtcm2_t {
 		unsigned int bitrate;		/* of station transmissions */
 	    } station[MAXSTATIONS];
 	} almanac;
+	struct {		/* data for type 13 messages */
+	    bool status;		/* expect a text message */
+	    bool rangeflag;		/* station range altered? */
+	    double lat, lon;		/* station longitude/latitude */
+	    unsigned int range;		/* transmission range in km */
+	} xmitter;
+	struct {		/* data from type 14 messages */
+	    unsigned int week;			/* GPS week (0-1023) */
+	    unsigned int hour;			/* Hour in week (0-167) */
+	    unsigned int leapsecs;		/* Leap seconds (0-63) */
+	} gpstime;
+	struct {
+	    unsigned int nentries;
+	    struct glonass_rangesat_t {		/* data from message type 31 */
+		unsigned ident;		/* satellite ID */
+		unsigned udre;		/* user diff. range error */
+		unsigned tod;		/* issue of data */
+		bool change;		/* ephemeris change bit */
+		double prc;		/* range error */
+		double rrc;		/* range error rate */
+	    } sat[MAXCORRECTIONS];
+	} glonass_ranges;
 	/* data from type 16 messages */
 	char message[(RTCM2_WORDS_MAX-2) * sizeof(isgps30bits_t)];
 	/* data from messages of unknown type */
@@ -268,19 +313,19 @@ struct rtcm3_t {
 	/* 1001-1013 were present in the 3.0 version */
 	struct {
 	    struct rtcm3_rtk_hdr	header;
-	    struct {
+	    struct rtcm3_1001_t {
 		unsigned ident;			/* Satellite ID */
 		struct rtcm3_basic_rtk L1;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1001;
-	struct {
+	struct rtcm3_1002_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;			/* Satellite ID */
 		struct rtcm3_extended_rtk L1;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1002;
-	struct {
+	struct rtcm3_1003_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;			/* Satellite ID */
@@ -288,7 +333,7 @@ struct rtcm3_t {
 		struct rtcm3_basic_rtk L2;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1003;
-	struct {
+	struct rtcm3_1004_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;			/* Satellite ID */
@@ -296,14 +341,14 @@ struct rtcm3_t {
 		struct rtcm3_extended_rtk L2;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1004;
-	struct {
+	struct rtcm3_1005_t {
 	    unsigned int station_id;		/* Reference Station ID */
 	    int system;				/* Which system is it? */
 	    bool reference_station;		/* Reference-station indicator */
 	    bool single_receiver;		/* Single Receiver Oscillator */
 	    double ecef_x, ecef_y, ecef_z;	/* ECEF antenna location */
 	} rtcm3_1005;
-	struct {
+	struct rtcm3_1006_t {
 	    unsigned int station_id;		/* Reference Station ID */
 	    int system;				/* Which system is it? */
 	    bool reference_station;		/* Reference-station indicator */
@@ -311,32 +356,32 @@ struct rtcm3_t {
 	    double ecef_x, ecef_y, ecef_z;	/* ECEF antenna location */
 	    double height;			/* Antenna height */
 	} rtcm3_1006;
-	struct {
+	struct rtcm3_1007_t {
 	    unsigned int station_id;			/* Reference Station ID */
 	    char descriptor[RTCM3_MAX_DESCRIPTOR+1];	/* Description string */
 	    unsigned char setup_id;
 	} rtcm3_1007;
-	struct {
+	struct rtcm3_1008_t {
 	    unsigned int station_id;			/* Reference Station ID */
 	    char descriptor[RTCM3_MAX_DESCRIPTOR+1];	/* Description string */
 	    unsigned char setup_id;
 	    char serial[RTCM3_MAX_DESCRIPTOR+1];	/* Serial # string */
 	} rtcm3_1008;
-	struct {
+	struct rtcm3_1009_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;		/* Satellite ID */
 		struct rtcm3_basic_rtk L1;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1009;
-	struct {
+	struct rtcm3_1010_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;		/* Satellite ID */
 		struct rtcm3_extended_rtk L1;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1010;
-	struct {
+	struct rtcm3_1011_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;			/* Satellite ID */
@@ -344,7 +389,7 @@ struct rtcm3_t {
 		struct rtcm3_extended_rtk L2;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1011;
-	struct {
+	struct rtcm3_1012_t {
 	    struct rtcm3_rtk_hdr	header;
 	    struct {
 		unsigned ident;			/* Satellite ID */
@@ -352,7 +397,7 @@ struct rtcm3_t {
 		struct rtcm3_extended_rtk L2;
 	    } rtk_data[RTCM3_MAX_SATELLITES];
 	} rtcm3_1012;
-	struct {
+	struct rtcm3_1013_t {
 	    unsigned int station_id;	/* Reference Station ID */
 	    unsigned short mjd;		/* Modified Julian Day (MJD) Number */
 	    unsigned int sod;		/* Seconds of Day (UTC) */
@@ -365,7 +410,7 @@ struct rtcm3_t {
 	    } announcements[RTCM3_MAX_ANNOUNCEMENTS];
 	} rtcm3_1013;
 	/* 1014-1017 were added in the 3.1 version */
-	struct {
+	struct rtcm3_1014_t {
 	    unsigned int network_id;	/* Network ID */
 	    unsigned int subnetwork_id;	/* Subnetwork ID */
 	    unsigned char stationcount;	/* # auxiliary stations transmitted */
@@ -373,20 +418,20 @@ struct rtcm3_t {
 	    unsigned int aux_id;	/* Auxilary Reference Station ID */
 	    double d_lat, d_lon, d_alt;	/* Aux-master location delta */
 	} rtcm3_1014;
-	struct {
+	struct rtcm3_1015_t {
 	    struct rtcm3_network_rtk_header	header;
 	    struct rtcm3_correction_diff corrections[RTCM3_MAX_SATELLITES];
 	} rtcm3_1015;
-	struct {
+	struct rtcm3_1016_t {
 	    struct rtcm3_network_rtk_header	header;
 	    struct rtcm3_correction_diff corrections[RTCM3_MAX_SATELLITES];
 	} rtcm3_1016;
-	struct {
+	struct rtcm3_1017_t {
 	    struct rtcm3_network_rtk_header	header;
 	    struct rtcm3_correction_diff corrections[RTCM3_MAX_SATELLITES];
 	} rtcm3_1017;
 	/* 1018-1029 were in the 3.0 version */
-	struct {
+	struct rtcm3_1019_t {
 	    unsigned int ident;		/* Satellite ID */
 	    unsigned int week;		/* GPS Week Number */
 	    unsigned char sv_accuracy;	/* GPS SV ACCURACY */
@@ -419,7 +464,7 @@ struct rtcm3_t {
 	    bool p_data;
 	    bool fit_interval;
 	} rtcm3_1019;
-	struct {
+	struct rtcm3_1020_t {
 	    unsigned int ident;		/* Satellite ID */
 	    unsigned short channel;	/* Satellite Frequency Channel Number */
 	    /* ephemeris fields, not scaled */
@@ -457,18 +502,322 @@ struct rtcm3_t {
 	    signed int M_tau_sub_GPS;
 	    bool M_l_sub_n;
 	} rtcm3_1020;
-	struct {
+	struct rtcm3_1029_t {
 	    unsigned int station_id;	/* Reference Station ID */
 	    unsigned short mjd;		/* Modified Julian Day (MJD) Number */
 	    unsigned int sod;		/* Seconds of Day (UTC) */
-	    unsigned char len;		/* # Chars to follow */
-	    unsigned char unicode_units;
+	    size_t len;			/* # chars to follow */
+	    size_t unicode_units;	/* # Unicode units in text */
 	    unsigned char text[128];
 	} rtcm3_1029;
+	char data[1024];		/* Max RTCM3 msg length is 1023 bytes */
     } rtcmtypes;
 };
 
-typedef /*@unsignedintegraltype@*/ unsigned int gps_mask_t;
+/*
+ * Raw IS_GPS subframe data
+ */
+
+/* The almanac is a subset of the clock and ephemeris data, with reduced 
+ * precision. See IS-GPS-200E, Table 20-VI  */
+struct almanac_t
+{
+    uint8_t sv;  /* The satellite this refers to */
+    /* toa, almanac reference time, 8 bits unsigned, seconds */
+    uint8_t toa;
+    long l_toa;
+    /* SV health data, 8 bit unsigned bit map */
+    uint8_t svh;
+    /* deltai, correction to inclination, 16 bits signed, semi-circles */
+    int16_t deltai;
+    double d_deltai;
+    /* M0, Mean Anomaly at Reference Time, 24 bits signed, semi-circles */
+    int32_t M0;
+    double d_M0;
+    /* Omega0, Longitude of Ascending Node of Orbit Plane at Weekly Epoch,
+     * 24 bits signed, semi-circles */
+    int32_t Omega0;
+    double d_Omega0;
+    /* omega, Argument of Perigee, 24 bits signed, semi-circles */
+    int32_t omega;
+    double d_omega;
+    /* af0, SV clock correction constant term
+     * 11 bits signed, seconds */
+    int16_t af0;
+    double d_af0;
+    /* af1, SV clock correction first order term
+     * 11 bits signed, seconds/second */
+    int16_t af1;
+    double d_af1;
+    /* eccentricity, 16 bits, unsigned, dimensionless */
+    uint16_t e;
+    double d_eccentricity;
+    /* sqrt A, Square Root of the Semi-Major Axis
+     * 24 bits unsigned, square_root(meters) */
+    uint32_t sqrtA;
+    double d_sqrtA;
+    /* Omega dot, Rate of Right Ascension, 16 bits signed, semi-circles/sec */
+    int16_t Omegad;
+    double d_Omegad;
+};
+
+struct subframe_t {
+    /* subframe number, 3 bits, unsigned, 1 to 5 */
+    uint8_t subframe_num;
+    /* data_id, denotes the NAV data structure of D(t), 2 bits, in
+     * IS-GPS-200E always == 0x1 */
+    uint8_t data_id;
+    /* SV/page id used for subframes 4 & 5, 6 bits */
+    uint8_t pageid; 
+    /* tSVID, SV ID of the sat that transmitted this frame, 6 bits unsigned */
+    uint8_t tSVID;
+    /* TOW, Time of Week of NEXT message, 17 bits unsigned, scale 6, seconds */
+    uint32_t TOW17;
+    long l_TOW17;
+    /* integrity, URA bounds flag, 1 bit */
+    bool integrity;
+    /* alert, alert flag, SV URA and/or the SV User Differential Range 
+     * Accuracy (UDRA) may be worse than indicated, 1 bit */
+    bool alert;
+    /* antispoof, A-S mode is ON in that SV, 1 bit */
+    bool antispoof;
+    int is_almanac;
+    union {
+        /* subframe 1, part of ephemeris, see IS-GPS-200E, Table 20-II
+	 * and Table 20-I */
+	struct {
+	    /* WN, Week Number, 10 bits unsigned, scale 1, weeks */
+	    uint16_t WN;
+	    /* IODC, Issue of Data, Clock, 10 bits, unsigned, 
+	     * issued in 8 data ranges at the same time */
+	    uint16_t IODC;
+	    /* toc, clock data reference time, 16 bits, unsigned, seconds
+	     * scale 2**4, issued in 8 data ranges at the same time */
+	    uint16_t toc;
+	    long l_toc;
+	    /* l2, code on L2, 2 bits, bit map */
+	    uint8_t l2;
+	    /* l2p, L2 P data flag, 1 bit */
+	    uint8_t l2p;
+	    /* ura, SV accuracy, 4 bits unsigned index */
+	    unsigned int ura;
+	    /* hlth, SV health, 6 bits unsigned bitmap */
+	    unsigned int hlth;
+	    /* af0, SV clock correction constant term
+	     * 22 bits signed, scale 2**-31, seconds */
+	    int32_t af0;
+	    double d_af0;
+	    /* af1, SV clock correction first order term
+	     * 22 bits signed, scale 2**-43, seconds/second */
+	    int16_t af1;
+	    double d_af1;
+	    /* af2, SV clock correction second order term
+	     * 8 bits signed, scale 2**-55, seconds/second**2 */
+	    int8_t af2;
+	    double d_af2;
+	    /* Tgd,  L1-L2 correction term, 8 bits signed,  scale 2**-31,
+	     * seconds */
+	    int8_t Tgd;
+	    double d_Tgd;
+	} sub1;
+        /* subframe 2, part of ephemeris, see IS-GPS-200E, Table 20-II
+	 * and Table 20-III */
+	struct {
+	    /* Issue of Data (Ephemeris), 
+	     * equal to the 8 LSBs of the 10 bit IODC of the same data set */
+	    uint8_t IODE;
+	    /* Age of Data Offset for the NMCT, 6 bits, scale 900,
+	     * ignore if all ones, seconds */
+	    uint8_t AODO;
+	    uint16_t u_AODO;
+	    /* fit, FIT interval flag, indicates a fit interval greater than
+	     * 4 hour, 1 bit */
+	    uint8_t fit;
+	    /* toe, Reference Time Ephemeris, 16 bits unsigned, scale 2**4,
+	     * seconds */
+	    uint16_t toe;
+	    long l_toe;
+	    /* Crs, Amplitude of the Sine Harmonic Correction Term to the 
+	     * Orbit Radius, 16 bits, scale 2**-5, signed, meters */
+	    int16_t Crs;
+	    double d_Crs;
+	    /* Cus, Amplitude of the Sine Harmonic Correction Term to the 
+	     * Argument of Latitude, 16 bits, signed, scale 2**-29, radians */
+	    int16_t Cus;
+	    double d_Cus;
+	    /* Cuc, Amplitude of the Cosine Harmonic Correction Term to the 
+	     * Argument of Latitude, 16 bits, signed, scale 2**-29, radians */
+	    int16_t Cuc;
+	    double d_Cuc;
+	    /* deltan, Mean Motion Difference From Computed Value
+	     * Mean Motion Difference From Computed Value
+	     * 16 bits, signed, scale 2**-43, semi-circles/sec */
+	    int16_t deltan;
+	    double d_deltan;
+	    /* M0, Mean Anomaly at Reference Time, 32 bits signed, 
+	     * scale 2**-31, semi-circles */
+	    int32_t M0;
+	    double d_M0;
+	    /* eccentricity, 32 bits, unsigned, scale 2**-33, dimensionless */
+	    uint32_t e;
+	    double d_eccentricity;
+	    /* sqrt A, Square Root of the Semi-Major Axis
+	     * 32 bits unsigned, scale 2**-19, square_root(meters) */
+	    uint32_t sqrtA;
+	    double d_sqrtA;
+	} sub2;
+        /* subframe 3, part of ephemeris, see IS-GPS-200E, Table 20-II,
+	 * Table 20-III */
+	struct {
+	    /* Issue of Data (Ephemeris), 8 bits, unsigned 
+	     * equal to the 8 LSBs of the 10 bit IODC of the same data set */
+	    uint8_t IODE;
+	    /* Rate of Inclination Angle, 14 bits signed, scale2**-43,
+	     * semi-circles/sec */
+	    uint16_t IDOT;
+	    double d_IDOT;
+	    /* Cic, Amplitude of the Cosine Harmonic Correction Term to the 
+	     * Angle of Inclination, 16 bits signed, scale 2**-29, radians*/
+	    uint16_t Cic;
+	    double d_Cic;
+	    /* Cis, Amplitude of the Sine Harmonic Correction Term to the
+	     * Angle of Inclination, 16 bits, unsigned, scale 2**-29, radians */
+	    int16_t Cis;
+	    double d_Cis;
+            /* Crc, Amplitude of the Cosine Harmonic Correction Term to the
+	     * Orbit Radius, 16 bits signed, scale 2**-5, meters */
+	    int16_t Crc;
+	    double d_Crc;
+	    /* i0, Inclination Angle at Reference Time, 32 bits, signed,
+	     * scale 2**-31, semi-circles */
+	    int32_t i0;
+	    double d_i0;
+	    /* Omega0, Longitude of Ascending Node of Orbit Plane at Weekly 
+	     * Epoch, 32 bits signed, semi-circles */
+	    int32_t Omega0;
+	    double d_Omega0;
+	    /* omega, Argument of Perigee, 32 bits signed, scale 2**-31,
+	     * semi-circles */
+	    int32_t omega;
+	    double d_omega;
+	    /* Omega dot, Rate of Right Ascension, 24 bits signed, 
+	     * scale 2**-43, semi-circles/sec */
+	    int32_t Omegad;
+	    double d_Omegad;
+	} sub3;
+	struct {
+	    struct almanac_t almanac;
+	} sub4;
+	/* subframe 4, page 13 */
+	struct {
+	    /* mapping ord ERD# to SV # is non trivial
+	     * leave it alone.  See IS-GPS-200E Section 20.3.3.5.1.9 */
+	    /* Estimated Range Deviation, 6 bits signed, meters */
+	    char ERD[33];
+	    /* ai, Availability Indicator, 2bits, bit map */
+	    unsigned char ai;
+	} sub4_13;
+	/* subframe 4, page 17, system message, 23 chars, plus nul */
+	struct {
+	    char str[24];
+	} sub4_17;
+	/* subframe 4, page 18 */
+	struct {
+	    /* ionospheric and UTC data */
+	    /* A0, Bias coefficient of GPS time scale relative to UTC time 
+	     * scale, 32 bits signed, scale 2**-30, seconds */
+	    int32_t A0;
+	    double d_A0;
+	    /* A1, Drift coefficient of GPS time scale relative to UTC time 
+	     * scale, 24 bits signed, scale 2**-50, seconds/second */
+	    int32_t A1;
+	    double d_A1;
+
+	    /* alphaX, the four coefficients of a cubic equation representing 
+	     * the amplitude of the vertical delay */
+
+	    /* alpha0, 8 bits signed, scale w**-30, seconds */
+	    int8_t alpha0;
+	    double d_alpha0;
+	    /* alpha1, 8 bits signed, scale w**-27, seconds/semi-circle */
+	    int8_t alpha1;
+	    double d_alpha1;
+	    /* alpha2, 8 bits signed, scale w**-24, seconds/semi-circle**2 */
+	    int8_t alpha2;
+	    double d_alpha2;
+	    /* alpha3, 8 bits signed, scale w**-24, seconds/semi-circle**3 */
+	    int8_t alpha3;
+	    double d_alpha3;
+
+	    /* betaX, the four coefficients of a cubic equation representing 
+	     * the period of the model */
+
+	    /* beta0, 8 bits signed, scale w**11, seconds */
+	    int8_t beta0;
+	    double d_beta0;
+	    /* beta1, 8 bits signed, scale w**14, seconds/semi-circle */
+	    int8_t beta1;
+	    double d_beta1;
+	    /* beta2, 8 bits signed, scale w**16, seconds/semi-circle**2 */
+	    int8_t beta2;
+	    double d_beta2;
+	    /* beta3, 8 bits signed, scale w**16, seconds/semi-circle**3 */
+	    int8_t beta3;
+	    double d_beta3;
+	    
+	    /* leap (delta t ls), current leap second, 8 bits signed, 
+	     * scale 1, seconds */
+	    int8_t leap;
+	    /* lsf (delta t lsf), future leap second, 8 bits signed, 
+	     * scale 1, seconds */
+	    int8_t lsf;
+
+	    /* tot, reference time for UTC data,
+	     * 8 bits unsigned, scale 2**12, seconds */
+	    uint8_t tot;
+	    double d_tot;
+
+	    /* WNt, UTC reference week number, 8 bits unsigned, scale 1, 
+	     * weeks */
+	    uint8_t WNt;
+	    /* WNlsf, Leap second reference Week Number,
+	     * 8 bits unsigned, scale 1, weeks */
+	    uint8_t WNlsf;
+	    /* DN, Leap second reference Day Number , 8 bits unsigned,
+	     * scale 1, days */
+	    uint8_t DN;
+	} sub4_18;
+	/* subframe 4, page 25 */
+	struct {
+	    /* svf, A-S status and the configuration code of each SV
+	     * 4 bits unsigned, bitmap */
+	    unsigned char svf[33];
+	    /* svh, SV health data for SV 25 through 32
+	     * 6 bits unsigned bitmap */
+	    uint8_t svhx[8];
+	} sub4_25;
+	struct {
+	    struct almanac_t almanac;
+	} sub5;
+	struct {
+	    /* toa, Almanac reference Time, 8 bits unsigned, scale 2**12,
+	     * seconds */
+	    uint8_t toa;
+	    long l_toa;
+	    /* WNa, Week Number almanac, 8 bits, scale 2, GPS Week 
+	     * Number % 256 */
+	    uint8_t WNa;
+	    /* sv, SV health status, 6 bits, bitmap */
+	    uint8_t sv[25];
+	} sub5_25;
+    };
+};
+
+#ifndef S_SPLINT_S
+typedef uint64_t gps_mask_t;
+#else
+typedef /*@unsignedintegraltype@*/ unsigned long long gps_mask_t;
+#endif /* S_SPLINT_S */
 
 /* 
  * Is an MMSI number that of an auxiliary associated with a mother ship?
@@ -565,7 +914,8 @@ struct ais_t
 	    unsigned int dest_mmsi;	/* destination MMSI */
 	    bool retransmit;		/* retransmit flag */
 	    //unsigned int spare;	spare bit(s) */
-	    unsigned int app_id;        /* Application ID */
+	    unsigned int dac;           /* Application ID */
+	    unsigned int fid;           /* Functional ID */
 #define AIS_TYPE6_BINARY_MAX	920	/* 920 bits */
 	    size_t bitcount;		/* bit count of the data */
 	    char bitdata[(AIS_TYPE6_BINARY_MAX + 7) / 8];
@@ -581,10 +931,48 @@ struct ais_t
 	/* Type 8 - Broadcast Binary Message */
 	struct {
 	    //unsigned int spare;	spare bit(s) */
-	    unsigned int app_id;       	/* Application ID */
+	    unsigned int dac;       	/* Designated Area Code */
+	    unsigned int fid;       	/* Functional ID */
 #define AIS_TYPE8_BINARY_MAX	952	/* 952 bits */
 	    size_t bitcount;		/* bit count of the data */
-	    char bitdata[(AIS_TYPE8_BINARY_MAX + 7) / 8];
+	    union {
+		char bitdata[(AIS_TYPE8_BINARY_MAX + 7) / 8];
+#ifdef __UNUSED__
+		struct {
+#define DAC1FID31_LATLON_SCALE	1000
+		    int lon;		/* longitude in minutes * .001 */
+#define DAC1FID31_LON_NOT_AVAILABLE	(181*60*DAC1FID31_LATLON_SCALE)
+		    int lat;		/* longitude in minutes * .001 */
+#define DAC1FID31_LAT_NOT_AVAILABLE	(91*60*DAC1FID31_LATLON_SCALE)
+		    bool accuracy;	/* position accuracy, <10m if true */
+		    unsigned int day;		/* UTC day */
+		    unsigned int hour;		/* UTC hour */
+		    unsigned int minute;	/* UTC minute */
+		    unsigned int wspeed;	/* average wind speed */
+		    unsigned int wgust;		/* wind gust */
+#define DAC1FID31_WIND_HIGH			126
+#define DAC1FID31_WIND_NOT_AVAILABLE		127
+		    unsigned int wdir;		/* wind direction */
+		    unsigned int wgustdir;	/* wind gust direction */
+#define DAC1FID31_DIR_NOT_AVAILABLE		360
+		    int temperature;		/* temperature, units 0.1C */
+#define DAC1FID31_AIRTEMP_NOT_AVAILABLE		-1024
+		    unsigned int humidity;	/* relative humidity, % */
+#define DAC1FID31_HUMIDITY_NOT_AVAILABLE	101
+		    int dewpoint;		/* dew point, units 0.1C */
+#define DAC1FID31_DEWPOINT_NOT_AVAILABLE	501
+		    unsigned int pressure;	/* air pressure, hpa */
+#define DAC1FID31_PRESSURE_NOT_AVAILABLE	511
+#define DAC1FID31_PRESSURE_HIGH			402
+		    unsigned int pressuretend;	/* tendency */
+		    bool visgreater;		/* vis. > than following */
+		    unsigned int visibility;	/* units 0.1 nautical miles */
+#define DAC1FID31_VISIBILITY_NOT_AVAILABLE	127
+		    unsigned int waterlevel;	/* decimeters + 100 */
+#define DAC1FID31_WATERLEVEL_NOT_AVAILABLE	4001
+		} dac1fid31;
+#endif /* __UNUSED_ */
+	    };
 	} type8;
 	/* Type 9 - Standard SAR Aircraft Position Report */
 	struct {
@@ -905,7 +1293,8 @@ struct policy_t {
     int raw;				/* requesting raw data? */
     bool scaled;			/* requesting report scaling? */ 
     bool timing;			/* requesting timing info */
-    char devpath[GPS_PATH_MAX];		/* specific device to watch */   
+    int loglevel;			/* requested log level of messages */
+    char devpath[GPS_PATH_MAX];		/* specific device to watch */
 };
 
 /* 
@@ -917,17 +1306,17 @@ struct policy_t {
 typedef int socket_t;
 
 /* mode flags for setting streaming policy */
-#define WATCH_ENABLE	0x0001u	/* enable streaming */
-#define WATCH_JSON	0x0002u	/* enable JSON output */
-#define WATCH_NMEA	0x0004u	/* enable output in NMEA */
-#define WATCH_RARE	0x0008u	/* enable output of packets in hex */
-#define WATCH_RAW	0x0010u	/* enable output of raw packets */
-#define WATCH_SCALED	0x0020u	/* scale output to floats, when applicable */ 
-#define WATCH_NEWSTYLE	0x0040u	/* force JSON streaming */
-#define WATCH_OLDSTYLE	0x0080u	/* force old-style streaming */
-#define WATCH_DEVICE	0x0100u	/* watch specific device */
-#define WATCH_DISABLE	0x0200u	/* disable watching */
-#define POLL_NONBLOCK	0x1000u	/* set non-blocking poll (experimental!) */
+#define WATCH_ENABLE	0x000001u	/* enable streaming */
+#define WATCH_DISABLE	0x000002u	/* disable watching */
+#define WATCH_JSON	0x000010u	/* JSON output */
+#define WATCH_NMEA	0x000020u	/* output in NMEA */
+#define WATCH_RARE	0x000040u	/* output of packets in hex */
+#define WATCH_RAW	0x000080u	/* output of raw packets */
+#define WATCH_SCALED	0x000100u	/* scale output to floats */ 
+#define WATCH_TIMING	0x000200u	/* timing information */
+#define WATCH_DEVICE	0x000800u	/* watch specific device */
+#define WATCH_NEWSTYLE	0x010000u	/* force JSON streaming */
+#define WATCH_OLDSTYLE	0x020000u	/* force old-style streaming */
 
 /* 
  * Main structure that includes all previous substructures
@@ -935,38 +1324,38 @@ typedef int socket_t;
 
 struct gps_data_t {
     gps_mask_t set;	/* has field been set since this was last cleared? */
-#define ONLINE_SET	0x00000001u
-#define TIME_SET	0x00000002u
-#define TIMERR_SET	0x00000004u
-#define LATLON_SET	0x00000008u
-#define ALTITUDE_SET	0x00000010u
-#define SPEED_SET	0x00000020u
-#define TRACK_SET	0x00000040u
-#define CLIMB_SET	0x00000080u
-#define STATUS_SET	0x00000100u
-#define MODE_SET	0x00000200u
-#define DOP_SET  	0x00000400u
-#define VERSION_SET	0x00000800u
-#define HERR_SET	0x00001000u
-#define VERR_SET	0x00002000u
-#define ATTITUDE_SET	0x00004000u
-#define POLICY_SET	0x00008000u
-#define SATELLITE_SET	0x00010000u
-#define RAW_SET		0x00020000u
-#define USED_SET	0x00040000u
-#define SPEEDERR_SET	0x00080000u
-#define TRACKERR_SET	0x00100000u
-#define CLIMBERR_SET	0x00200000u
-#define DEVICE_SET	0x00400000u
-#define DEVICELIST_SET	0x00800000u
-#define DEVICEID_SET	0x01000000u
-#define ERROR_SET	0x02000000u
-#define RTCM2_SET	0x04000000u
-#define RTCM3_SET	0x08000000u
-#define AIS_SET 	0x10000000u
-#define PACKET_SET	0x20000000u
-#define AUXDATA_SET	0x80000000u	/* reserved */
-    double online;		/* NZ if GPS is on line, 0 if not.
+#define ONLINE_SET	(1u<<1)
+#define TIME_SET	(1u<<2)
+#define TIMERR_SET	(1u<<3)
+#define LATLON_SET	(1u<<4)
+#define ALTITUDE_SET	(1u<<5)
+#define SPEED_SET	(1u<<6)
+#define TRACK_SET	(1u<<7)
+#define CLIMB_SET	(1u<<8)
+#define STATUS_SET	(1u<<9)
+#define MODE_SET	(1u<<10)
+#define DOP_SET  	(1u<<11)
+#define HERR_SET	(1u<<12)
+#define VERR_SET	(1u<<13)
+#define ATTITUDE_SET	(1u<<14)
+#define SATELLITE_SET	(1u<<15)
+#define SPEEDERR_SET	(1u<<16)
+#define TRACKERR_SET	(1u<<17)
+#define CLIMBERR_SET	(1u<<18)
+#define DEVICE_SET	(1u<<19)
+#define DEVICELIST_SET	(1u<<20)
+#define DEVICEID_SET	(1u<<21)
+#define RTCM2_SET	(1u<<22)
+#define RTCM3_SET	(1u<<23)
+#define AIS_SET 	(1u<<24)
+#define PACKET_SET	(1u<<25)
+#define SUBFRAME_SET	(1u<<26)
+#define GST_SET 	(1u<<27)
+#define VERSION_SET	(1u<<28)
+#define POLICY_SET	(1u<<29)
+#define LOGMESSAGE_SET	(1u<<30)
+#define ERROR_SET	(1u<<31)
+    timestamp_t online;		/* NZ if GPS is on line, 0 if not.
 				 *
 				 * Note: gpsd clears this time when sentences
 				 * fail to show up within the GPS's normal
@@ -1000,7 +1389,7 @@ struct gps_data_t {
     double epe;  /* spherical position error, 95% confidence (meters)  */
 
     /* satellite status -- valid when satellites_visible > 0 */
-    double skyview_time;	/* skyview timestamp */
+    timestamp_t skyview_time;	/* skyview timestamp */
     int satellites_visible;	/* # of satellites in view */
     int PRN[MAXCHANNELS];	/* PRNs of satellite */
     int elevation[MAXCHANNELS];	/* elevation of satellite */
@@ -1013,42 +1402,40 @@ struct gps_data_t {
 
     char tag[MAXTAGLEN+1];	/* tag of last sentence processed */
 
-    void (*raw_hook)(struct gps_data_t *, char *, size_t len);	/* Raw-mode hook for GPS data. */
-
     /* pack things never reported together to reduce structure size */ 
-#define UNION_SET	(RTCM2_SET|RTCM3_SET|AIS_SET|VERSION_SET|DEVICELIST_SET|ERROR_SET)
+#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|VERSION_SET|DEVICELIST_SET|LOGMESSAGE_SET|ERROR_SET|GST_SET)
     union {
 	/* unusual forms of sensor data that might come up the pipe */ 
 	struct rtcm2_t	rtcm2;
 	struct rtcm3_t	rtcm3;
+	struct subframe_t subframe;
 	struct ais_t ais;
 	struct attitude_t attitude;
 	struct rawdata_t raw;
+	struct gst_t gst;
 	/* "artificial" structures for various protocol responses */
 	struct version_t version;
 	struct {
-	    double time;
+	    timestamp_t time;
 	    int ndevices;
 	    struct devconfig_t list[MAXUSERDEVS];
 	} devices;
-	char error[80];
+	char error[256];
     };
 
     /* Private data - client code must not set this */
     void *privdata;
 };
 
-extern int gps_open_r(const char *host, const char *, 
+extern int gps_open(/*@null@*/const char *, /*@null@*/const char *, 
 		      /*@out@*/struct gps_data_t *);
-extern /*@null@*/struct gps_data_t *gps_open(const char *, const char *);
 extern int gps_close(struct gps_data_t *);
 extern int gps_send(struct gps_data_t *, const char *, ... );
-extern int gps_poll(struct gps_data_t *);
-extern bool gps_waiting(struct gps_data_t *);
+extern int gps_read(/*@out@*/struct gps_data_t *);
+extern bool gps_waiting(struct gps_data_t *, int);
 extern int gps_stream(struct gps_data_t *, unsigned int, /*@null@*/void *);
-extern void gps_set_raw_hook(struct gps_data_t *, 
-			     void (*)(struct gps_data_t *, char *, size_t));
-extern char /*@observer@*/ *gps_errstr(const int);
+extern const char /*@observer@*/ *gps_data(struct gps_data_t *);
+extern const char /*@observer@*/ *gps_errstr(const int);
 
 /* this only needs to be visible for the unit tests */
 extern int gps_unpack(char *, struct gps_data_t *);
@@ -1063,12 +1450,13 @@ extern void gps_enable_debug(int, FILE *);
 extern /*@observer@*/const char *gps_maskdump(gps_mask_t);
 
 extern time_t mkgmtime(register struct tm *);
-extern double timestamp(void);
-extern double iso8601_to_unix(char *);
-extern /*@observer@*/char *unix_to_iso8601(double t, /*@ out @*/char[], size_t len);
-extern double gpstime_to_unix(int, double);
-extern void unix_to_gpstime(double, /*@out@*/int *, /*@out@*/double *);
+extern timestamp_t timestamp(void);
+extern timestamp_t iso8601_to_unix(char *);
+extern /*@observer@*/char *unix_to_iso8601(timestamp_t t, /*@ out @*/char[], size_t len);
 extern double earth_distance(double, double, double, double);
+extern double earth_distance_and_bearings(double, double, double, double, 
+					  /*@null@*//*@out@*/double *, 
+					  /*@null@*//*@out@*/double *);
 extern double wgs84_separation(double, double);
 
 /* some multipliers for interpreting GPS output */
@@ -1092,7 +1480,7 @@ extern double wgs84_separation(double, double);
 #define WGS84F 298.257223563	/* flattening */
 #define WGS84B 6356752.3142	/* polar radius */
 
-/* gps_open() errno return values */
+/* netlib_connectsock() errno return values */
 #define NL_NOSERVICE	-1	/* can't get service entry */
 #define NL_NOHOST	-2	/* can't get host entry */
 #define NL_NOPROTO	-3	/* can't get protocol entry */
