@@ -5,80 +5,50 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <stdlib.h>
-#include "gpsd_config.h"
+
+#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/time.h>		/* for select() */
+#include <stdio.h>
+#include <time.h>
+#include <string.h>
+#include <stdlib.h>
+#include <stdbool.h>
+#include <stdarg.h>
+#include <ctype.h>
+#include <setjmp.h>
+#include <assert.h>
+#include <pwd.h>
+#include <grp.h>
+#include <math.h>
+#include <syslog.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <locale.h>
+#ifndef S_SPLINT_S
+#include <netdb.h>
+#ifndef AF_UNSPEC
+#include <sys/socket.h>
+#endif /* AF_UNSPEC */
+#ifndef INADDR_ANY
+#include <netinet/in.h>
+#endif /* INADDR_ANY */
+#include <sys/un.h>
+#include <arpa/inet.h>     /* for htons() and friends */
+#endif /* S_SPLINT_S */
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif /* HAVE_SYSLOG_H */
-#include <signal.h>
-#include <errno.h>
-#include <ctype.h>
-#include <fcntl.h>
-#include <string.h>
-#ifndef S_SPLINT_S
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#else
-#define AF_UNSPEC 0
-#endif /* HAVE_SYS_SOCKET_H */
-#ifdef HAVE_SYS_UN_H
-#include <sys/un.h>
-#endif /* HAVE_SYS_UN_H */
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif /* HAVE_NETINET_IN_H */
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif /* HAVE_ARPA_INET_H */
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif /* HAVE_NETDB_H */
-#endif /* S_SPLINT_S */
-#include <stdarg.h>
-#include <setjmp.h>
-#include <stdio.h>
-#include <assert.h>
-#ifdef HAVE_PWD_H
-#include <pwd.h>
-#endif /* HAVE_PWD_H */
-#ifdef HAVE_GRP_H
-#include <grp.h>
-#endif /* HAVE_GRP_H */
-#include <stdbool.h>
-#include <math.h>
 
-#if defined (HAVE_PATH_H)
-#include <paths.h>
-#else
-#if !defined (_PATH_DEVNULL)
-#define _PATH_DEVNULL    "/dev/null"
-#endif
-#endif
-#if defined (HAVE_SYS_SELECT_H)
-#include <sys/select.h>
-#endif
-#if defined (HAVE_SYS_STAT_H)
-#include <sys/stat.h>
-#endif
-#if defined(HAVE_SYS_TIME_H)
-#include <sys/time.h>
-#endif
-#ifdef HAVE_SETLOCALE
-#include <locale.h>
-#endif
+#include "gpsd_config.h"
 
 #ifdef DBUS_ENABLE
-#include <gpsd_dbus.h>
+#include "gpsd_dbus.h"
 #endif
 
 #include "gpsd.h"
 #include "sockaddr.h"
 #include "gps_json.h"
-#include "timebase.h"
 #include "revision.h"
 
 /*
@@ -157,35 +127,7 @@ static bool in_background = false;
 static bool listen_global = false;
 static bool nowait = false;
 static jmp_buf restartbuf;
-
-/* *INDENT-OFF* */
-/*@ -initallelements -nullassign -nullderef @*/
-struct gps_context_t context = {
-    .valid	    = 0,
-    .readonly	    = false,
-    .sentdgps	    = false,
-    .netgnss_service  = netgnss_none,
-    .fixcnt	    = 0,
-    .dsock	    = -1,
-    .netgnss_privdata = NULL,
-    .rtcmbytes	    = 0,
-    .rtcmbuf	    = {'\0'},
-    .rtcmtime	    = 0,
-    .leap_seconds   = LEAP_SECONDS,
-    .gps_week	    = 0,
-    .gps_tow        = 0,
-    .century	    = CENTURY_BASE,
-#ifdef NTPSHM_ENABLE
-    .enable_ntpshm  = false,
-    .shmTime	    = {0},
-    .shmTimeInuse   = {0},
-# ifdef PPS_ENABLE
-    .shmTimePPS	    = false,
-# endif /* PPS_ENABLE */
-#endif /* NTPSHM_ENABLE */
-};
-/*@ +initallelements +nullassign +nullderef @*/
-/* *INDENT-ON* */
+static struct gps_context_t context;
 
 static volatile sig_atomic_t signalled;
 
@@ -195,39 +137,6 @@ static void onsig(int sig)
     signalled = (sig_atomic_t) sig;
 }
 
-static int daemonize(void)
-{
-    int fd;
-    pid_t pid;
-
-    /*@ -type @*//* weirdly, splint 3.1.2 is confused by fork() */
-    switch (pid = fork()) {
-    case -1:
-	return -1;
-    case 0:			/* child side */
-	break;
-    default:			/* parent side */
-	exit(0);
-    }
-    /*@ +type @*/
-
-    if (setsid() == -1)
-	return -1;
-    if (chdir("/") == -1)
-	return -1;
-    /*@ -nullpass @*/
-    if ((fd = open(_PATH_DEVNULL, O_RDWR, 0)) != -1) {
-	(void)dup2(fd, STDIN_FILENO);
-	(void)dup2(fd, STDOUT_FILENO);
-	(void)dup2(fd, STDERR_FILENO);
-	if (fd > 2)
-	    (void)close(fd);
-    }
-    /*@ +nullpass @*/
-    in_background = true;
-    return 0;
-}
-
 #if defined(PPS_ENABLE)
 static pthread_mutex_t report_mutex;
 #endif /* PPS_ENABLE */
@@ -238,11 +147,12 @@ static void visibilize(/*@out@*/char *buf2, size_t len, const char *buf)
 
     buf2[0] = '\0';
     for (sp = buf; *sp != '\0' && strlen(buf2)+4 < len; sp++)
-	if (isprint(*sp) || (sp[0] == '\n' && sp[1] == '\0') || (sp[0] == '\r' && sp[2] == '\0'))
+	if (isprint(*sp) || (sp[0] == '\n' && sp[1] == '\0')
+	  || (sp[0] == '\r' && sp[2] == '\0'))
 	    (void)snprintf(buf2 + strlen(buf2), 2, "%c", *sp);
 	else
 	    (void)snprintf(buf2 + strlen(buf2), 6, "\\x%02x",
-			   (unsigned)*sp);
+			   0x00ff & (unsigned)*sp);
 }
 
 void gpsd_report(int errlevel, const char *fmt, ...)
@@ -251,6 +161,7 @@ void gpsd_report(int errlevel, const char *fmt, ...)
 #ifndef SQUELCH_ENABLE
     if (errlevel <= debuglevel) {
 	char buf[BUFSIZ], buf2[BUFSIZ];
+	char *err_str;
 	va_list ap;
 
 #if defined(PPS_ENABLE)
@@ -258,7 +169,40 @@ void gpsd_report(int errlevel, const char *fmt, ...)
 	(void)pthread_mutex_lock(&report_mutex);
 	/* +unrecog */
 #endif /* PPS_ENABLE */
-	(void)strlcpy(buf, "gpsd: ", BUFSIZ);
+	switch ( errlevel ) {
+	case LOG_ERROR:
+		err_str = "ERROR: ";
+		break;
+	case LOG_SHOUT:
+		err_str = "SHOUT: ";
+		break;
+	case LOG_WARN:
+		err_str = "WARN: ";
+		break;
+	case LOG_INF:
+		err_str = "INFO: ";
+		break;
+	case LOG_DATA:
+		err_str = "DATA: ";
+		break;
+	case LOG_PROG:
+		err_str = "PROG: ";
+		break;
+	case LOG_IO:
+		err_str = "IO: ";
+		break;
+	case LOG_SPIN:
+		err_str = "SPIN: ";
+		break;
+	case LOG_RAW:
+		err_str = "RAW: ";
+		break;
+	default:
+		err_str = "UNK: ";
+	}
+
+	(void)strlcpy(buf, "gpsd:", BUFSIZ);
+	(void)strncat(buf, err_str, BUFSIZ - strlen(buf) );
 	va_start(ap, fmt);
 	(void)vsnprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), fmt,
 			ap);
@@ -457,7 +401,7 @@ static int filesock(char *filename)
 	return -1;
     }
     (void)strlcpy(addr.sun_path, filename, 104);	/* from sys/un.h */
-    addr.sun_family = AF_UNIX;
+    addr.sun_family = (sa_family_t)AF_UNIX;
     (void)bind(sock, (struct sockaddr *)&addr, (int)sizeof(addr));
     if (listen(sock, QLEN) == -1) {
 	gpsd_report(LOG_ERROR, "can't listen on local socket %s\n", filename);
@@ -470,7 +414,7 @@ static int filesock(char *filename)
 struct subscriber_t
 {
     int fd;			/* client file descriptor. -1 if unused */
-    double active;		/* when subscriber last polled for data */
+    timestamp_t active;		/* when subscriber last polled for data */
     struct policy_t policy;	/* configurable bits */
 };
 
@@ -500,8 +444,8 @@ struct subscriber_t
 #define initialized_device(devp) ((devp)->context != NULL)
 #define subscribed(sub, devp)    (sub->policy.devpath[0]=='\0' || strcmp(sub->policy.devpath, devp->gpsdata.dev.path)==0)
 
-struct gps_device_t devices[MAXDEVICES];
-struct subscriber_t subscribers[MAXSUBSCRIBERS];	/* indexed by client file descriptor */
+static struct gps_device_t devices[MAXDEVICES];
+static struct subscriber_t subscribers[MAXSUBSCRIBERS];	/* indexed by client file descriptor */
 
 static void adjust_max_fd(int fd, bool on)
 /* track the largest fd currently in use */
@@ -564,8 +508,9 @@ static void detach_client(struct subscriber_t *sub)
 		c_ip, sub_index(sub), sub->fd);
     FD_CLR(sub->fd, &all_fds);
     adjust_max_fd(sub->fd, false);
-    sub->active = 0;
+    sub->active = (timestamp_t)0;
     sub->policy.watcher = false;
+    sub->policy.json = false;
     sub->policy.nmea = false;
     sub->policy.raw = 0;
     sub->policy.scaled = false;
@@ -618,7 +563,7 @@ static ssize_t throttled_write(struct subscriber_t *sub, char *buf,
     return status;
 }
 
-static void notify_watchers(struct gps_device_t *device, char *sentence, ...)
+static void notify_watchers(struct gps_device_t *device, const char *sentence, ...)
 /* notify all clients watching a given device of an event */
 {
     va_list ap;
@@ -648,77 +593,65 @@ static void deactivate_device(struct gps_device_t *device)
 }
 
 /* *INDENT-OFF* */
-/*@ -globstate @*/
-/*@null@*//*@observer@*/ static struct gps_device_t *find_device(char
+/*@null@*//*@observer@*/ static struct gps_device_t *find_device(/*@null@*/const char
 								 *device_name)
 /* find the device block for an existing device name */
 {
     struct gps_device_t *devp;
 
     for (devp = devices; devp < devices + MAXDEVICES; devp++)
-	if (allocated_device(devp)
-	    && strcmp(devp->gpsdata.dev.path, device_name) == 0)
-	    return devp;
+    {
+        if (allocated_device(devp) && NULL != device_name &&
+            strcmp(devp->gpsdata.dev.path, device_name) == 0)
+            return devp;
+    }
     return NULL;
 }
 /* *INDENT-ON* */
 
-/*@ -nullret @*/
-/*@ -statictrans @*/
-static bool open_device(char *device_name)
-/* open and initialize a new device block */
+static bool open_device( /*@null@*/struct gps_device_t *device)
 {
-    struct gps_device_t *devp;
-
-    for (devp = devices; devp < devices + MAXDEVICES; devp++)
-	if (!allocated_device(devp)
-	    || (strcmp(devp->gpsdata.dev.path, device_name) == 0
-		&& !initialized_device(devp))) {
-	    goto found;
+	if (NULL == device || gpsd_activate(device) < 0) {
+		return false;
 	}
-    return false;
-  found:
-    gpsd_init(devp, &context, device_name);
-    if (gpsd_activate(devp) < 0)
-	return false;
-    FD_SET(devp->gpsdata.gps_fd, &all_fds);
-    adjust_max_fd(devp->gpsdata.gps_fd, true);
-    return true;
+	gpsd_report(LOG_INF, "device %s activated\n",
+			device->gpsdata.dev.path);
+	FD_SET(device->gpsdata.gps_fd, &all_fds);
+	adjust_max_fd(device->gpsdata.gps_fd, true);
+	return true;
 }
 
-static bool add_device(char *device_name)
+static bool add_device(const char *device_name)
 /* add a device to the pool; open it right away if in nowait mode */
 {
-    if (nowait)
-	return open_device(device_name);
-    else {
 	struct gps_device_t *devp;
+	bool ret = false;
 	/* stash devicename away for probing when the first client connects */
 	for (devp = devices; devp < devices + MAXDEVICES; devp++)
 	    if (!allocated_device(devp)) {
 		gpsd_init(devp, &context, device_name);
 		gpsd_report(LOG_INF, "stashing device %s at slot %d\n",
 			    device_name, (int)(devp - devices));
-		devp->gpsdata.gps_fd = -1;
+		if (nowait) {
+			ret = open_device(devp);
+		} else {
+			devp->gpsdata.gps_fd = -1;
+			ret = true;
+		}
 		notify_watchers(devp,
-				"{\"class\":\"DEVICE\",\"path\":\"%s\",\"activated\":%ld}\r\n",
+				"{\"class\":\"DEVICE\",\"path\":\"%s\",\"activated\":%lf}\r\n",
 				devp->gpsdata.dev.path, timestamp());
-		return true;
+		break;
 	    }
-	return false;
-    }
+	return ret;
 }
 
-/*@ +nullret @*/
-/*@ +statictrans @*/
-/*@ +globstate @*/
-
 static bool awaken(struct gps_device_t *device)
 /* awaken a device and notify all watchers*/
 {
     /* open that device */
     if (!initialized_device(device)) {
-	if (!open_device(device->gpsdata.dev.path)) {
+	if (!open_device(device)) {
 	    gpsd_report(LOG_PROG, "%s: open failed\n",
 			device->gpsdata.dev.path);
 	    free_device(device);
@@ -821,10 +754,21 @@ static void handle_control(int sfd, char *buf)
 	} else {
 	    *eq++ = '\0';
 	    if ((devp = find_device(stash))) {
-		gpsd_report(LOG_INF, "<= control(%d): writing to %s \n", sfd,
-			    stash);
-		ignore_return(write(devp->gpsdata.gps_fd, eq, strlen(eq)));
-		ignore_return(write(sfd, "OK\n", 3));
+		if (devp->context->readonly || (devp->sourcetype <= source_blockdev)) {
+		    gpsd_report(LOG_WARN, "<= control(%d): attempted to write to a read-only device\n",
+				sfd);
+		    ignore_return(write(sfd, "ERROR\n", 6));
+		} else {
+		    gpsd_report(LOG_INF, "<= control(%d): writing to %s \n", sfd,
+				stash);
+		    if (write(devp->gpsdata.gps_fd, eq, strlen(eq)) <= 0) {
+			gpsd_report(LOG_WARN, "<= control(%d): write to device failed\n",
+				    sfd);
+			ignore_return(write(sfd, "ERROR\n", 6));
+		    } else {
+			ignore_return(write(sfd, "OK\n", 3));
+		    }
+		}
 	    } else {
 		gpsd_report(LOG_INF, "<= control(%d): %s not active \n", sfd,
 			    stash);
@@ -844,20 +788,30 @@ static void handle_control(int sfd, char *buf)
 	    *eq++ = '\0';
 	    len = strlen(eq) + 5;
 	    if ((devp = find_device(stash)) != NULL) {
-		/* NOTE: this destroys the original buffer contents */
-		st = gpsd_hexpack(eq, eq, len);
-		if (st <= 0) {
-		    gpsd_report(LOG_INF,
-				"<= control(%d): invalid hex string (error %d).\n",
-				sfd, st);
+		if (devp->context->readonly || (devp->sourcetype <= source_blockdev)) {
+		    gpsd_report(LOG_WARN, "<= control(%d): attempted to write to a read-only device\n",
+				sfd);
 		    ignore_return(write(sfd, "ERROR\n", 6));
-		} else {
-		    gpsd_report(LOG_INF,
-				"<= control(%d): writing %d bytes fromhex(%s) to %s\n",
-				sfd, st, eq, stash);
-		    ignore_return(write
-				  (devp->gpsdata.gps_fd, eq, (size_t) st));
-		    ignore_return(write(sfd, "OK\n", 3));
+                } else {
+                    /* NOTE: this destroys the original buffer contents */
+                    st = gpsd_hexpack(eq, eq, len);
+                    if (st <= 0) {
+                        gpsd_report(LOG_INF,
+                                    "<= control(%d): invalid hex string (error %d).\n",
+                                    sfd, st);
+                        ignore_return(write(sfd, "ERROR\n", 6));
+                    } else {
+                        gpsd_report(LOG_INF,
+                                    "<= control(%d): writing %d bytes fromhex(%s) to %s\n",
+                                    sfd, st, eq, stash);
+                        if (write(devp->gpsdata.gps_fd, eq, (size_t) st) <= 0) {
+                            gpsd_report(LOG_WARN, "<= control(%d): write to device failed\n",
+                                        sfd);
+                            ignore_return(write(sfd, "ERROR\n", 6));
+                        } else {
+                            ignore_return(write(sfd, "OK\n", 3));
+                        }
+                    }
 		}
 	    } else {
 		gpsd_report(LOG_INF, "<= control(%d): %s not active\n", sfd,
@@ -865,6 +819,16 @@ static void handle_control(int sfd, char *buf)
 		ignore_return(write(sfd, "ERROR\n", 6));
 	    }
 	}
+    } else if (strcmp(buf, "?devices")==0) {
+	for (devp = devices; devp < devices + MAXDEVICES; devp++) {
+	    char *path = devp->gpsdata.dev.path;
+	    ignore_return(write(sfd, path, strlen(path)));
+	    ignore_return(write(sfd, "\n", 1));
+	}
+	ignore_return(write(sfd, "OK\n", 6));
+    } else {
+	/* unknown command */
+	ignore_return(write(sfd, "ERROR\n", 6));
     }
     /*@ +sefparams @*/
 }
@@ -996,7 +960,7 @@ static void handle_request(struct subscriber_t *sub,
 		(void)snprintf(reply, replylen,
 			       "{\"class\":\"ERROR\",\"message\":\"Invalid WATCH: %s\"}\r\n",
 			       json_error_string(status));
-		gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+		gpsd_report(LOG_ERROR, "response: %s\n", reply);
 	    } else if (sub->policy.watcher) {
 		if (sub->policy.devpath[0] == '\0') {
 		    /* awaken all devices */
@@ -1007,16 +971,17 @@ static void handle_request(struct subscriber_t *sub,
 		    devp = find_device(sub->policy.devpath);
 		    if (devp == NULL) {
 			(void)snprintf(reply, replylen,
-				       "{\"class\":\"ERROR\",\"message\":\"Do nuch device as %s\"}\r\n",
+				       "{\"class\":\"ERROR\",\"message\":\"No such device as %s\"}\r\n",
 				       sub->policy.devpath);
-			gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+			gpsd_report(LOG_ERROR, "response: %s\n", reply);
 			goto bailout;
-		    } else if (!awaken(devp))
+		    } else if (!awaken(devp)) {
 			(void)snprintf(reply, replylen,
 				       "{\"class\":\"ERROR\",\"message\":\"Can't assign %s\"}\r\n",
 				       sub->policy.devpath);
-		    gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
-		    goto bailout;
+			gpsd_report(LOG_ERROR, "response: %s\n", reply);
+			goto bailout;
+		    }
 		}
 	    }
 	}
@@ -1047,18 +1012,20 @@ static void handle_request(struct subscriber_t *sub,
 	    /*@-branchstate@*/
 	    if (status != 0) {
 		(void)snprintf(reply, replylen,
-			       "{\"class\":\"ERROR\",\"message\":\"Invalid DEVICE: %s\"}\r\n",
+			       "{\"class\":\"ERROR\",\"message\":\"Invalid DEVICE: \"%s\"}\r\n",
 			       json_error_string(status));
-		gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+		gpsd_report(LOG_ERROR, "response: %s\n", reply);
 		goto bailout;
 	    } else {
 		if (devconf.path[0] != '\0') {
 		    /* user specified a path, try to assign it */
-		    if (!awaken(find_device(devconf.path))) {
+		    device = find_device(devconf.path);
+		    /* do not optimize away, we need 'device' later on! */
+		    if (!awaken(device)) {
 			(void)snprintf(reply, replylen,
 				       "{\"class\":\"ERROR\",\"message\":\"Can't open %s.\"}\r\n",
 				       devconf.path);
-			gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+			gpsd_report(LOG_ERROR, "response: %s\n", reply);
 			goto bailout;
 		    }
 		} else {
@@ -1073,13 +1040,13 @@ static void handle_request(struct subscriber_t *sub,
 			(void)strlcat(reply,
 				      "{\"class\":\"ERROR\",\"message\":\"Can't perform DEVICE configuration, no devices attached.\"}\r\n",
 				      replylen);
-			gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+			gpsd_report(LOG_ERROR, "response: %s\n", reply);
 			goto bailout;
 		    } else if (devcount > 1) {
 			(void)snprintf(reply + strlen(reply),
 				       replylen - strlen(reply),
 				       "{\"class\":\"ERROR\",\"message\":\"No path specified in DEVICE, but multiple devices are attached.\"}\r\n");
-			gpsd_report(LOG_ERROR, "ERROR response: %s\n", reply);
+			gpsd_report(LOG_ERROR, "response: %s\n", reply);
 			goto bailout;
 		    }
 		    /* we should have exactly one device now */
@@ -1149,6 +1116,7 @@ static void handle_request(struct subscriber_t *sub,
 				 replylen - strlen(reply));
 	    }
     } else if (strncmp(buf, "POLL;", 5) == 0) {
+	char tbuf[JSON_DATE_MAX+1];
 	int active = 0;
 	buf += 5;
 	for (devp = devices; devp < devices + MAXDEVICES; devp++)
@@ -1156,8 +1124,8 @@ static void handle_request(struct subscriber_t *sub,
 		if ((devp->observed & GPS_TYPEMASK) != 0)
 		    active++;
 	(void)snprintf(reply, replylen,
-		       "{\"class\":\"POLL\",\"timestamp\":%.3f,\"active\":%d,\"fixes\":[",
-		       timestamp(), active);
+		       "{\"class\":\"POLL\",\"time\":\"%s\",\"active\":%d,\"fixes\":[",
+		       unix_to_iso8601(timestamp(), tbuf, sizeof(tbuf)), active);
 	for (devp = devices; devp < devices + MAXDEVICES; devp++) {
 	    if (allocated_device(devp) && subscribed(sub, devp)) {
 		if ((devp->observed & GPS_TYPEMASK) != 0) {
@@ -1169,6 +1137,20 @@ static void handle_request(struct subscriber_t *sub,
 		}
 	    }
 	}
+	if (reply[strlen(reply) - 1] == ',')
+	    reply[strlen(reply) - 1] = '\0';	/* trim trailing comma */
+	(void)strlcat(reply, "],\"gst\":[", replylen);
+	for (devp = devices; devp < devices + MAXDEVICES; devp++) {
+	    if (allocated_device(devp) && subscribed(sub, devp)) {
+		if ((devp->observed & GPS_TYPEMASK) != 0) {
+		    json_noise_dump(&devp->gpsdata,
+				  reply + strlen(reply),
+				  replylen - strlen(reply));
+		    rstrip(reply);
+		    (void)strlcat(reply, ",", replylen);
+		}
+	    }
+	}
 	if (reply[strlen(reply) - 1] == ',')
 	    reply[strlen(reply) - 1] = '\0';	/* trim trailing comma */
 	(void)strlcat(reply, "],\"skyviews\":[", replylen);
@@ -1185,7 +1167,7 @@ static void handle_request(struct subscriber_t *sub,
 	}
 	if (reply[strlen(reply) - 1] == ',')
 	    reply[strlen(reply) - 1] = '\0';	/* trim trailing comma */
-	(void)strlcat(reply, "]}]}\r\n", replylen);
+	(void)strlcat(reply, "]}\r\n", replylen);
     } else if (strncmp(buf, "VERSION;", 8) == 0) {
 	buf += 8;
 	json_version_dump(reply, replylen);
@@ -1256,7 +1238,7 @@ static void raw_report(struct subscriber_t *sub, struct gps_device_t *device)
 static void pseudonmea_report(struct subscriber_t *sub,
 			  gps_mask_t changed,
 			  struct gps_device_t *device)
-/* report pseodo-NMEA in appropriate circumstances */
+/* report pseudo-NMEA in appropriate circumstances */
 {
     if (GPS_PACKET_TYPE(device->packet.type)
 	&& !TEXTUAL_PACKET_TYPE(device->packet.type)) {
@@ -1264,81 +1246,30 @@ static void pseudonmea_report(struct subscriber_t *sub,
 
 	gpsd_report(LOG_PROG, "data mask is %s\n",
 		    gpsd_maskdump(device->gpsdata.set));
+
 	if ((changed & REPORT_IS) != 0) {
 	    nmea_tpv_dump(device, buf, sizeof(buf));
-	    gpsd_report(LOG_IO, "<= GPS (binary1) %s: %s\n",
+	    gpsd_report(LOG_IO, "<= GPS (binary tpv) %s: %s\n",
 			device->gpsdata.dev.path, buf);
 	    (void)throttled_write(sub, buf, strlen(buf));
-	} else if ((changed & SATELLITE_IS) != 0) {
+	}
+
+	if ((changed & SATELLITE_IS) != 0) {
 	    nmea_sky_dump(device, buf, sizeof(buf));
-	    gpsd_report(LOG_IO, "<= GPS (binary2) %s: %s\n",
+	    gpsd_report(LOG_IO, "<= GPS (binary sky) %s: %s\n",
+			device->gpsdata.dev.path, buf);
+	    (void)throttled_write(sub, buf, strlen(buf));
+	}
+
+	if ((changed & SUBFRAME_IS) != 0) {
+	    nmea_subframe_dump(device, buf, sizeof(buf));
+	    gpsd_report(LOG_IO, "<= GPS (binary subframe) %s: %s\n",
 			device->gpsdata.dev.path, buf);
 	    (void)throttled_write(sub, buf, strlen(buf));
 	}
     }
 }
 
-static void json_report(struct subscriber_t *sub,
-			  gps_mask_t changed,
-			  struct gps_device_t *device)
-/* report in JSON format to a subscriber */
-{
-    char buf[GPS_JSON_RESPONSE_MAX * 4];
-
-    buf[0] = '\0';
-    if ((changed & REPORT_IS) != 0) {
-	json_tpv_dump(&device->gpsdata,
-		      buf, sizeof(buf));
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-
-    if ((changed & SATELLITE_IS) != 0) {
-	json_sky_dump(&device->gpsdata,
-		      buf, sizeof(buf));
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-#ifdef COMPASS_ENABLE
-    if ((changed & ATT_IS) != 0) {
-	json_att_dump(&device->gpsdata,
-		      buf, sizeof(buf));
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-#endif /* COMPASS_ENABLE */
-#ifdef RTCM104V2_ENABLE
-    if ((changed & RTCM2_IS) != 0) {
-	rtcm2_json_dump(&device->gpsdata.rtcm2, buf,
-			sizeof(buf));
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-#endif /* RTCM104V2_ENABLE */
-#ifdef AIVDM_ENABLE
-    if ((changed & AIS_IS) != 0) {
-	aivdm_json_dump(&device->gpsdata.ais,
-			sub->policy.scaled,
-			buf, sizeof(buf));
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-#endif /* AIVDM_ENABLE */
-
-#ifdef TIMING_ENABLE
-    if (buf[0] != '\0' && sub->policy.timing) {
-	(void)snprintf(buf, sizeof(buf),
-		       "{\"class\":\"TIMING\","
-		       "\"tag\":\"%s\",\"len\":%d,"
-		       "\"xmit\":%lf,\"recv\":%lf,"
-		       "\"decode\":%lf,"
-		       "\"emit\":%lf}\r\n",
-		       device->gpsdata.tag,
-		       (int)device->packet.outbuflen,
-		       device->d_xmit_time,
-		       device->d_recv_time,
-		       device->d_decode_time,
-		       timestamp());
-	(void)throttled_write(sub, buf, strlen(buf));
-    }
-#endif /* TIMING_ENABLE */
-}
-
 static void consume_packets(struct gps_device_t *device)
 /* consume and report packets from a specified device */
 {
@@ -1347,7 +1278,30 @@ static void consume_packets(struct gps_device_t *device)
     struct subscriber_t *sub;
 
     gpsd_report(LOG_RAW + 1, "polling %d\n",
-		device->gpsdata.gps_fd);
+	    device->gpsdata.gps_fd);
+
+    /*
+     * Strange special case - the opening transaction on an NTRIP connection
+     * may not yet be completed.  Try to ratchet things forward.
+     */
+    if (device->servicetype == service_ntrip
+	    && device->ntrip.conn_state != ntrip_conn_established) {
+
+	/* the socket descriptor might change during connection */
+	if (device->gpsdata.gps_fd != -1) {
+	    FD_CLR(device->gpsdata.gps_fd, &all_fds);
+	}
+	(void)ntrip_open(device, "");
+	if (device->ntrip.conn_state == ntrip_conn_err) {
+	    gpsd_report(LOG_WARN,
+		    "connection to ntrip server failed\n");
+	    device->ntrip.conn_state = ntrip_conn_init;
+	    deactivate_device(device);
+	} else {
+	    FD_SET(device->gpsdata.gps_fd, &all_fds);
+	}
+	return;
+    }
 
     for (fragments = 0; ; fragments++) {
 	changed = gpsd_poll(device);
@@ -1355,23 +1309,32 @@ static void consume_packets(struct gps_device_t *device)
 	if (changed == ERROR_IS) {
 	    gpsd_report(LOG_WARN,
 			"device read of %s returned error or packet sniffer failed sync (flags %s)\n",
-			device->gpsdata.dev.path, 
+			device->gpsdata.dev.path,
 			gpsd_maskdump(changed));
 	    deactivate_device(device);
-	    break;
+    break;
 	} else if (changed == NODATA_IS) {
 	    /*
 	     * No data on the first fragment read means the device
-	     * fd may have been in an end-of-file condition on select. 
+	     * fd may have been in an end-of-file condition on select.
 	     */
 	    if (fragments == 0) {
 		gpsd_report(LOG_DATA,
 			    "%s returned zero bytes\n",
 			    device->gpsdata.dev.path);
-		if (device->zerokill)
+		if (device->zerokill) {
 		    /* failed timeout-and-reawake, kill it */
 		    deactivate_device(device);
-		else {
+		    if (device->ntrip.works) {
+			device->ntrip.works = false; // reset so we try this once only
+			if (gpsd_activate(device) < 0) {
+			    gpsd_report(LOG_WARN, "reconnect to ntrip server failed\n");
+			} else {
+			    gpsd_report(LOG_INFO, "reconnecting to ntrip server\n");
+			    FD_SET(device->gpsdata.gps_fd, &all_fds);
+			}
+		    }
+		} else {
 		    /*
 		     * Disable listening to this fd for long enough
 		     * that the buffer can fill up again.
@@ -1395,14 +1358,14 @@ static void consume_packets(struct gps_device_t *device)
 
 	/* we got actual data, head off the reawake special case */
 	device->zerokill = false;
-	device->reawake = 0;
+	device->reawake = (timestamp_t)0;
 
 	/* must have a full packet to continue */
 	if ((changed & PACKET_IS) == 0)
 	    break;
 
 	gpsd_report(LOG_DATA,
-		    "packet from %s with %s\n", 
+		    "packet from %s with %s\n",
 		    device->gpsdata.dev.path,
 		    gpsd_maskdump(device->gpsdata.set));
 
@@ -1431,20 +1394,21 @@ static void consume_packets(struct gps_device_t *device)
 	    }
 	}
 
-	/* 
-	 * If the device provided an RTCM packet, stash it 
+	/*
+	 * If the device provided an RTCM packet, stash it
 	 * in the context structure for use as a future correction.
 	 */
 	if ((changed & RTCM2_IS) != 0 || (changed & RTCM3_IS) != 0) {
 	    if (device->packet.outbuflen > RTCM_MAX) {
-		gpsd_report(LOG_ERROR, 
+		gpsd_report(LOG_ERROR,
 			    "overlong RTCM packet (%zd bytes)\n",
 			    device->packet.outbuflen);
 	    } else {
 		context.rtcmbytes = device->packet.outbuflen;
-		memcpy(context.rtcmbuf, 
-		       device->packet.outbuffer, 
+		memcpy(context.rtcmbuf,
+		       device->packet.outbuffer,
 		       context.rtcmbytes);
+		context.rtcmtime = timestamp();
 	    }
 	}
 
@@ -1458,17 +1422,27 @@ static void consume_packets(struct gps_device_t *device)
 
 	/* a few things are not per-subscriber reports */
 	if ((changed & REPORT_IS) != 0) {
-	    if (device->gpsdata.fix.mode == MODE_3D)
-		netgnss_report(device);
+	    if (device->gpsdata.fix.mode == MODE_3D) {
+		struct gps_device_t *dgnss;
+		/*
+		 * Pass the fix to every potential caster, here.
+		 * netgnss_report() individual caster types get to
+		 * make filtering decisiona.
+		 */
+		for (dgnss = devices; dgnss < devices + MAXDEVICES; dgnss++)
+		    if (dgnss != device)
+			netgnss_report(&context, device, dgnss);
+	    }
 #ifdef DBUS_ENABLE
-	    send_dbus_fix(device);
+	    if (device->gpsdata.fix.mode > MODE_NO_FIX)
+		send_dbus_fix(device);
 #endif /* DBUS_ENABLE */
 	}
 
 	/* update all subscribers associated with this device */
 	for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++) {
 	    /*@-nullderef@*/
-	    if (sub == NULL || sub->active == 0)
+	    if (sub == NULL || sub->active == 0 || !subscribed(sub, device))
 		continue;
 
 	    /* report raw packets to users subscribed to those */
@@ -1488,19 +1462,37 @@ static void consume_packets(struct gps_device_t *device)
 			pseudonmea_report(sub, changed, device);
 
 		    if (sub->policy.json)
-			json_report(sub, changed, device);
+		    {
+			char buf[GPS_JSON_RESPONSE_MAX * 4];
+
+			json_data_report(changed,
+					 &device->gpsdata, &sub->policy,
+					 buf, sizeof(buf));
+			if (buf[0] != '\0')
+			    (void)throttled_write(sub, buf, strlen(buf));
+
+#ifdef TIMING_ENABLE
+			if (buf[0] != '\0' && sub->policy.timing) {
+			    (void)snprintf(buf, sizeof(buf),
+					   "{\"class\":\"TIMING\","
+					   "\"tag\":\"%s\",\"len\":%d,"
+					   "\"xmit\":%lf,\"recv\":%lf,"
+					   "\"decode\":%lf,"
+					   "\"emit\":%lf}\r\n",
+					   device->gpsdata.tag,
+					   (int)device->packet.outbuflen,
+					   device->d_xmit_time,
+					   device->d_recv_time,
+					   device->d_decode_time,
+					   timestamp());
+			    (void)throttled_write(sub, buf, strlen(buf));
+			}
+#endif /* TIMING_ENABLE */
+		    }
 		}
 	    }
 	    /*@+nullderef@*/
 	} /* subscribers */
-
-	/* 
-	 * Trying to read a nonexistent datagram hangs,
-	 * so only go through the poll loop once after 
-	 * select in this case.
-	 */
-	if (device->sourcetype == source_udp)
-	    break;
     }
 }
 
@@ -1523,17 +1515,96 @@ static int handle_gpsd_request(struct subscriber_t *sub, const char *buf)
     return (int)throttled_write(sub, reply, strlen(reply));
 }
 
+#ifdef __UNUSED_AUTOCONNECT__
+#define DGPS_THRESHOLD	1600000	/* max. useful dist. from DGPS server (m) */
+#define SERVER_SAMPLE	12	/* # of servers within threshold to check */
+
+struct dgps_server_t
+{
+    double lat, lon;
+    char server[257];
+    double dist;
+};
+
+static int srvcmp(const void *s, const void *t)
+{
+    return (int)(((const struct dgps_server_t *)s)->dist - ((const struct dgps_server_t *)t)->dist);	/* fixes: warning: cast discards qualifiers from pointer target type */
+}
+
+static void netgnss_autoconnect(struct gps_context_t *context,
+			double lat, double lon, const char *serverlist)
+/* tell the library to talk to the nearest DGPSIP server */
+{
+    struct dgps_server_t keep[SERVER_SAMPLE], hold, *sp, *tp;
+    char buf[BUFSIZ];
+    FILE *sfp = fopen(serverlist, "r");
+
+    if (sfp == NULL) {
+	gpsd_report(LOG_ERROR, "no DGPS server list found.\n");
+	return;
+    }
+
+    for (sp = keep; sp < keep + SERVER_SAMPLE; sp++) {
+	sp->dist = DGPS_THRESHOLD;
+	sp->server[0] = '\0';
+    }
+    /*@ -usedef @*/
+    while (fgets(buf, (int)sizeof(buf), sfp)) {
+	char *cp = strchr(buf, '#');
+	if (cp != NULL)
+	    *cp = '\0';
+	if (sscanf(buf, "%lf %lf %256s", &hold.lat, &hold.lon, hold.server) ==
+	    3) {
+	    hold.dist = earth_distance(lat, lon, hold.lat, hold.lon);
+	    tp = NULL;
+	    /*
+	     * The idea here is to look for a server in the sample array
+	     * that is (a) closer than the one we're checking, and (b)
+	     * furtherest away of all those that are closer.  Replace it.
+	     * In this way we end up with the closest possible set.
+	     */
+	    for (sp = keep; sp < keep + SERVER_SAMPLE; sp++)
+		if (hold.dist < sp->dist
+		    && (tp == NULL || hold.dist > tp->dist))
+		    tp = sp;
+	    if (tp != NULL)
+		memcpy(tp, &hold, sizeof(struct dgps_server_t));
+	}
+    }
+    (void)fclose(sfp);
+
+    if (keep[0].server[0] == '\0') {
+	gpsd_report(LOG_ERROR, "no DGPS servers within %dm.\n",
+		    (int)(DGPS_THRESHOLD / 1000));
+	return;
+    }
+    /*@ +usedef @*/
+
+    /* sort them and try the closest first */
+    qsort((void *)keep, SERVER_SAMPLE, sizeof(struct dgps_server_t), srvcmp);
+    for (sp = keep; sp < keep + SERVER_SAMPLE; sp++) {
+	if (sp->server[0] != '\0') {
+	    gpsd_report(LOG_INF, "%s is %dkm away.\n", sp->server,
+			(int)(sp->dist / 1000));
+	    if (dgpsip_open(context, sp->server) >= 0)
+		break;
+	}
+    }
+}
+#endif /* __UNUSED_AUTOCONNECT__ */
+
 /*@ -mustfreefresh @*/
 int main(int argc, char *argv[])
 {
-    char *pid_file = NULL;
-    int st, csock = -1;
-    static char *gpsd_service = NULL;	/* static pacifies splint */
-    char *control_socket = NULL;
+    /* some of these statics suppress -W warnings due to longjmp() */
+    static char *pid_file = NULL;
+    static char *control_socket = NULL;
+    static int csock = -1;
+    static char *gpsd_service = NULL;	/* this static pacifies splint */
     struct gps_device_t *device;
     sockaddr_t fsin;
     fd_set rfds, control_fds;
-    int i, option, msocks[2], cfd, dfd;
+    int st, i, option, msocks[2], cfd, dfd;
     bool go_background = true;
     struct timeval tv;
     struct subscriber_t *sub;
@@ -1543,11 +1614,10 @@ int main(int argc, char *argv[])
     pthread_mutex_init(&report_mutex, NULL);
 #endif /* PPS_ENABLE */
 
-#ifdef HAVE_SETLOCALE
     (void)setlocale(LC_NUMERIC, "C");
-#endif
     debuglevel = 0;
     gpsd_hexdump_level = 0;
+    gps_context_init(&context);
     while ((option = getopt(argc, argv, "F:D:S:bGhlNnP:V")) != -1) {
 	switch (option) {
 	case 'D':
@@ -1641,8 +1711,14 @@ int main(int argc, char *argv[])
 		    control_socket);
     }
 
-    if (go_background)
-	(void)daemonize();
+    /* might be time to daemonize */
+    if (go_background) {
+	/* not SuS/POSIX portable, but we have our own fallback version */
+	if (daemon(0, 0) == 0)
+	    in_background = true;
+        else
+	    gpsd_report(LOG_ERROR,"demonization failed: %s\n",strerror(errno));
+    }
 
     if (pid_file) {
 	FILE *fp;
@@ -1758,14 +1834,13 @@ int main(int argc, char *argv[])
 	}
     FD_ZERO(&control_fds);
 
-    /* optimization hack to defer having to read subframe data */
-    if (time(NULL) < START_SUBFRAME)
-	context.valid |= LEAP_SECOND_VALID;
+    /* initialize the GPS context's time fields */
+    gpsd_time_init(&context, time(NULL));
 
     for (i = optind; i < argc; i++) {
 	if (!add_device(argv[i])) {
 	    gpsd_report(LOG_ERROR,
-			"GPS device %s nonexistent or can't be read\n",
+			"GPS device %s open failed\n",
 			argv[i]);
 	}
     }
@@ -1808,7 +1883,7 @@ int main(int argc, char *argv[])
 		if (FD_ISSET(i, &rfds))
 		    (void)snprintf(dbuf + strlen(dbuf),
 				   sizeof(dbuf) - strlen(dbuf), " %d ", i);
-	    gpsd_report(LOG_SPIN, "select() {%s} at %f (errno %d)\n", 
+	    gpsd_report(LOG_SPIN, "select() {%s} at %f (errno %d)\n",
 			dbuf, timestamp(), errno);
 	}
 
@@ -1883,15 +1958,6 @@ int main(int argc, char *argv[])
 	    FD_CLR(csock, &rfds);
 	}
 
-	if (context.dsock >= 0 && FD_ISSET(context.dsock, &rfds)) {
-	    /* be ready for DGPS reports */
-	    if (netgnss_poll(&context) == -1) {
-		FD_CLR(context.dsock, &all_fds);
-		FD_CLR(context.dsock, &rfds);
-		context.dsock = -1;
-	    }
-	}
-
 	/* read any commands that came in over control sockets */
 	for (cfd = 0; cfd < FD_SETSIZE; cfd++)
 	    if (FD_ISSET(cfd, &control_fds)) {
@@ -1926,7 +1992,7 @@ int main(int argc, char *argv[])
 		    gpsd_report(LOG_DATA,
 				"%s reawakened after zero-length read\n",
 				device->gpsdata.dev.path);
-		    device->reawake = 0;
+		    device->reawake = (timestamp_t)0;
 		    device->zerokill = true;
 		    FD_SET(device->gpsdata.gps_fd, &all_fds);
 		    adjust_max_fd(device->gpsdata.gps_fd, true);
@@ -1934,18 +2000,19 @@ int main(int argc, char *argv[])
 		    }
 	} /* devices */
 
-#ifdef NOT_FIXED
-	if (context.fixcnt > 0 && context.dsock == -1) {
+#ifdef __UNUSED_AUTOCONNECT__
+	if (context.fixcnt > 0 && !context.autconnect) {
 	    for (device = devices; device < devices + MAXDEVICES; device++) {
 		if (device->gpsdata.fix.mode > MODE_NO_FIX) {
 		    netgnss_autoconnect(&context,
 					device->gpsdata.fix.latitude,
 					device->gpsdata.fix.longitude);
+		    context.autconnect = True;
 		    break;
 		}
 	    }
 	}
-#endif
+#endif /* __UNUSED_AUTOCONNECT__ */
 
 	/* accept and execute commands for all clients */
 	for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++) {
