@@ -6,7 +6,7 @@
 
 const char *gpsd_maskdump(gps_mask_t set)
 {
-    static char buf[184];
+    static char buf[195];
     const struct {
 	gps_mask_t      mask;
 	const char      *name;
@@ -36,9 +36,11 @@ const char *gpsd_maskdump(gps_mask_t set)
 	{RTCM3_IS,	"RTCM3"},
 	{AIS_IS,	"AIS"},
 	{ATT_IS,	"ATT"},
+	{SUBFRAME_IS,	"SUBFRAME"},
 	{PACKET_IS,	"PACKET"},
 	{CLEAR_IS,	"CLEAR"},
 	{REPORT_IS,	"REPORT"},
+	{NOISE_IS,	"NOISE"},
 	{NODATA_IS,	"NODATA"},
     };
 
