@@ -4,26 +4,18 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/time.h>
 #include <stdio.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <stdarg.h>
+#include <strings.h>   /* for strcasecmp() */
 #include <math.h>
 #include <assert.h>
+#include <locale.h>
 
 #include "gpsd_config.h"
 #include "gps.h"
 #include "gpsdclient.h"
 
-#ifdef HAVE_SETLOCALE
-#include <locale.h>
-#endif
-
 /* convert double degrees to a static string and return a pointer to it
  *
  * deg_str_type:
@@ -105,9 +97,7 @@ enum unit gpsd_units(void)
 {
     char *envu = NULL;
 
-#ifdef HAVE_SETLOCALE
     (void)setlocale(LC_NUMERIC, "C");
-#endif
     if ((envu = getenv("GPSD_UNITS")) != NULL && *envu != '\0') {
 	if (0 == strcasecmp(envu, "imperial")) {
 	    return imperial;
