@@ -2,24 +2,21 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/types.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <stdbool.h>
+#include <string.h>
+#include <stdarg.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#include <stdlib.h>
-#include <string.h>
-#include <stdbool.h>
-#include <stdio.h>
-#include <stdarg.h>
-#include <stdbool.h>
-#include <ctype.h>
 
 #include "gpsd.h"
 #include "gps_json.h"
 
 static int verbose = 0;
 static bool scaled = true;
-static bool json = false;
+static bool json = true;
 
 /**************************************************************************
  *
@@ -43,6 +40,7 @@ void gpsd_report(int errlevel, const char *fmt, ...)
     }
 }
 
+#ifdef AIVDM_ENABLE
 static void aivdm_csv_dump(struct ais_t *ais, char *buf, size_t buflen)
 {
     (void)snprintf(buf, buflen, "%u|%u|%09u|", ais->type, ais->repeat,
@@ -358,69 +356,64 @@ static void aivdm_csv_dump(struct ais_t *ais, char *buf, size_t buflen)
     /*@ +formatcode @*/
     (void)strlcat(buf, "\r\n", buflen);
 }
+#endif
 
-/*@ -compdestroy -compdef -usedef @*/
-static void decode(FILE * fpin, FILE * fpout)
-/* RTCM or AIS packets on fpin to dump format on fpout */
+/*@ -compdestroy -compdef -usedef -uniondef @*/
+static void decode(FILE *fpin, FILE*fpout)
+/* sensor data on fpin to dump format on fpout */
 {
-    struct gps_packet_t lexer;
-    struct rtcm2_t rtcm2;
-    struct rtcm3_t rtcm3;
-    struct ais_t ais;
-    struct aivdm_context_t aivdm[AIVDM_CHANNELS];
-    char buf[BUFSIZ];
+    struct gps_device_t session;
+    struct gps_context_t context;
+    struct policy_t policy;
+    char buf[GPS_JSON_RESPONSE_MAX * 4];
 
-    memset(&aivdm, '\0', sizeof(aivdm));
-    packet_reset(&lexer);
+    memset(&policy, '\0', sizeof(policy));
+    policy.json = json;
 
-    while (packet_get(fileno(fpin), &lexer) > 0) {
-	if (lexer.type == COMMENT_PACKET)
+    gps_context_init(&context);
+    context.readonly = true;
+    gpsd_init(&session, &context, NULL);
+    gpsd_clear(&session);
+    session.gpsdata.gps_fd = fileno(fpin);
+    session.gpsdata.dev.baudrate = 38400;     /* hack to enable subframes */
+
+    for (;;)
+    {
+	gps_mask_t changed = gpsd_poll(&session);
+
+	if (changed == ERROR_IS || changed == NODATA_IS)
+	    break;
+	if (verbose >= 1 && TEXTUAL_PACKET_TYPE(session.packet.type))
+	    (void)fputs((char *)session.packet.outbuffer, fpout);
+	if ((changed & (REPORT_IS|SUBFRAME_IS|AIS_IS|RTCM2_IS|RTCM3_IS)) == 0)
 	    continue;
-#ifdef RTCM104V2_ENABLE
-	else if (lexer.type == RTCM2_PACKET) {
-	    rtcm2_unpack(&rtcm2, (char *)lexer.isgps.buf);
-	    if (json)
-		rtcm2_json_dump(&rtcm2, buf, sizeof(buf));
-	    else
-		rtcm2_sager_dump(&rtcm2, buf, sizeof(buf));
-	    (void)fputs(buf, fpout);
-	}
-#endif
-#ifdef RTCM104V3_ENABLE
-	else if (lexer.type == RTCM3_PACKET) {
-	    rtcm3_unpack(&rtcm3, (char *)lexer.outbuffer);
-	    rtcm3_dump(&rtcm3, stdout);
-	}
-#endif
-	else if (lexer.type == AIVDM_PACKET) {
-	    if (verbose >= 1)
-		(void)fputs((char *)lexer.outbuffer, stdout);
-	    /*@ -uniondef */
-	    if (aivdm_decode
-		((char *)lexer.outbuffer, lexer.outbuflen, aivdm, &ais)) {
-		if (!json)
-		    aivdm_csv_dump(&ais, buf, sizeof(buf));
-		else
-		    aivdm_json_dump(&ais, scaled, buf, sizeof(buf));
+	else if (json) {
+	    json_data_report(changed, 
+			     &session.gpsdata, &policy, 
+			     buf, sizeof(buf));
+	    (void)fputs(buf, fpout);	
+#ifdef AIVDM_ENABLE
+	} else if (session.packet.type == AIVDM_PACKET) {
+	    if ((changed & AIS_IS)!=0) {
+		aivdm_csv_dump(&session.gpsdata.ais, buf, sizeof(buf));
 		(void)fputs(buf, fpout);
 	    }
-
-	    /*@ +uniondef */
+#endif /* AIVDM_ENABLE */
 	}
     }
 }
 
-/*@ +compdestroy +compdef +usedef @*/
-
-/*@ -compdestroy @*/
-static void encode(FILE * fpin, FILE * fpout)
-/* dump format on fpin to RTCM-104 on fpout */
+static void encode(FILE *fpin, FILE *fpout)
+/* JSON format on fpin to JSON on fpout - idempotency test */
 {
     char inbuf[BUFSIZ];
+    struct policy_t policy;
     struct gps_data_t gpsdata;
     int lineno = 0;
 
-    memset(&gpsdata, '\0', sizeof(gpsdata));	/* avoid segfault due to garbage in thread-hook slots */
+    memset(&policy, '\0', sizeof(policy));
+    policy.json = true;
+
     while (fgets(inbuf, (int)sizeof(inbuf), fpin) != NULL) {
 	int status;
 
@@ -430,25 +423,17 @@ static void encode(FILE * fpin, FILE * fpout)
 	status = libgps_json_unpack(inbuf, &gpsdata, NULL);
 	if (status != 0) {
 	    (void)fprintf(stderr,
-			  "gpsdecode: bailing out with status %d (%s) on line %d\n",
+			  "gpsdecode: dying with status %d (%s) on line %d\n",
 			  status, json_error_string(status), lineno);
 	    exit(1);
 	}
-	if ((gpsdata.set & RTCM2_SET) != 0) {
-	    /* this works */
-	    char outbuf[BUFSIZ];
-	    rtcm2_json_dump(&gpsdata.rtcm2, outbuf, sizeof(outbuf));
-	    (void)fputs(outbuf, fpout);
-	}
-	if ((gpsdata.set & AIS_SET) != 0) {
-	    char outbuf[BUFSIZ];
-	    aivdm_json_dump(&gpsdata.ais, false, outbuf, sizeof(outbuf));
-	    (void)fputs(outbuf, fpout);
-	}
+	json_data_report(gpsdata.set, 
+			 &gpsdata, &policy, 
+			 inbuf, sizeof(inbuf));
+	(void)fputs(inbuf, fpout);	
     }
 }
-
-/*@ +compdestroy @*/
+/*@ +compdestroy +compdef +usedef @*/
 
 int main(int argc, char **argv)
 {
@@ -456,7 +441,7 @@ int main(int argc, char **argv)
     enum
     { doencode, dodecode } mode = dodecode;
 
-    while ((c = getopt(argc, argv, "cdejpuVD:")) != EOF) {
+    while ((c = getopt(argc, argv, "cdejpuvVD:")) != EOF) {
 	switch (c) {
 	case 'c':
 	    json = false;
@@ -478,6 +463,10 @@ int main(int argc, char **argv)
 	    scaled = false;
 	    break;
 
+	case 'v':
+	    verbose = 1;
+	    break;
+		
 	case 'D':
 	    verbose = atoi(optarg);
 	    gpsd_hexdump_level = verbose;
