@@ -4,57 +4,41 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
-#include <math.h>
 #include <ctype.h>
+#include <assert.h>
+#include <setjmp.h>
+#include <errno.h>
+#include <signal.h>
+#include <stdarg.h>
+#include <time.h>
+#include <termios.h>
+#include <sys/time.h>		/* expected to declare select(2) a la SuS */
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#include <assert.h>
-#include <signal.h>
-#include <setjmp.h>
+
 /* Cygwin has only _timezone and not timezone unless the following is set */
 #if defined(__CYGWIN__)
 #define timezonevar
 #endif /* defined(__CYGWIN__) */
-#include <time.h>
-#include <termios.h>
-#include <fcntl.h>		/* for O_RDWR */
-#include <stdarg.h>
-#include <stdbool.h>
-#include <errno.h>
-#include <sys/ioctl.h>		/* for O_RDWR */
-#include <setjmp.h>
 
 #include "gpsd_config.h"
-
 #ifdef HAVE_BLUEZ
 #include <bluetooth/bluetooth.h>
 #endif
-
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif /* HAVE_NCURSES_H */
 #include "gpsd.h"
-
-#include "bits.h"
-
-#if defined(HAVE_SYS_TIME_H)
-#include <sys/time.h>
-#endif
-#if defined (HAVE_SYS_SELECT_H)
-#include <sys/select.h>
-#endif
-
 #include "gpsdclient.h"
 #include "gpsmon.h"
 #include "revision.h"
 
+
 #ifdef S_SPLINT_S
 extern struct tm *localtime_r(const time_t *, /*@out@*/ struct tm *tp);
 #endif /* S_SPLINT_S */
@@ -70,7 +54,6 @@ extern struct monitor_object_t oncore_mmt, tnt_mmt;
 /* These are public */
 struct gps_device_t session;
 WINDOW *devicewin;
-int gmt_offset;
 
 /* These are private */
 static struct gps_context_t context;
@@ -79,7 +62,7 @@ static bool serial, curses_active;
 static int debuglevel = 0;
 static WINDOW *statwin, *cmdwin;
 /*@null@*/ static WINDOW *packetwin;
-static FILE *logfile;
+/*@null@*/ static FILE *logfile;
 static char *type_name;
 /*@ -nullassign @*/
 static const struct monitor_object_t *monitor_objects[] = {
@@ -122,7 +105,7 @@ static const struct monitor_object_t *monitor_objects[] = {
     NULL,
 };
 
-static const struct monitor_object_t **active;
+static const struct monitor_object_t **active, **fallback;
 /*@ +nullassign @*/
 
 static jmp_buf terminate;
@@ -151,17 +134,73 @@ void monitor_fixframe(WINDOW * win)
  *
  ******************************************************************************/
 
-void gpsd_report(int errlevel UNUSED, const char *fmt, ...)
+static void visibilize(/*@out@*/char *buf2, size_t len, const char *buf)
+{
+    const char *sp;
+
+    buf2[0] = '\0';
+    for (sp = buf; *sp != '\0' && strlen(buf2)+4 < len; sp++)
+	if (isprint(*sp) || (sp[0] == '\n' && sp[1] == '\0') 
+	  || (sp[0] == '\r' && sp[2] == '\0'))
+	    (void)snprintf(buf2 + strlen(buf2), 2, "%c", *sp);
+	else
+	    (void)snprintf(buf2 + strlen(buf2), 6, "\\x%02x",
+			   0x00ff & (unsigned)*sp);
+}
+
+void gpsd_report(int errlevel, const char *fmt, ...)
 /* our version of the logger */
 {
+    char buf[BUFSIZ]; 
+    char buf2[BUFSIZ];
+    char *err_str;
+
+    switch ( errlevel ) {
+    case LOG_ERROR:
+	err_str = "ERROR: ";
+	break;
+    case LOG_SHOUT:
+	err_str = "SHOUT: ";
+	break;
+    case LOG_WARN:
+	err_str = "WARN: ";
+	break;
+    case LOG_INF:
+	err_str = "INFO: ";
+	break;
+    case LOG_DATA:
+	err_str = "DATA: ";
+	break;
+    case LOG_PROG:
+	err_str = "PROG: ";
+	break;
+    case LOG_IO:
+	err_str = "IO: ";
+	break;
+    case LOG_SPIN:
+	err_str = "SPIN: ";
+	break;
+    case LOG_RAW:
+	err_str = "RAW: ";
+	break;
+    default:
+	err_str = "UNK: ";
+    }
+
+    (void)strlcpy(buf, "gpsd:", BUFSIZ);
+    (void)strncat(buf, err_str, BUFSIZ - strlen(buf) );
     if (errlevel <= debuglevel && packetwin != NULL) {
 	va_list ap;
 	va_start(ap, fmt);
-	if (!curses_active)
-	    (void)vprintf(fmt, ap);
-	else
-	    (void)wprintw(packetwin, (char *)fmt, ap);
+	(void)vsnprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), fmt, ap);
 	va_end(ap);
+	visibilize(buf2, sizeof(buf2), buf);
+	if (!curses_active)
+	    (void)fputs(buf2, stdout);
+	else
+	    (void)waddstr(packetwin, buf2);
+	if (logfile != NULL)
+	    (void)fputs(buf2, logfile);
     }
 }
 
@@ -177,7 +216,12 @@ static ssize_t readpkt(void)
     FD_SET(session.gpsdata.gps_fd, &select_set);
     if (controlfd > -1)
 	FD_SET(controlfd, &select_set);
-    timeval.tv_sec = 3;
+    /*
+     * If the timeout on this select isn't longer than the device's
+     * cycle time, the code will be prone to flaky timing-dependent
+     * failures.
+     */
+    timeval.tv_sec = 2;
     timeval.tv_usec = 0;
     if (select(session.gpsdata.gps_fd + 1, &select_set, NULL, NULL, &timeval)
 	== -1)
@@ -193,7 +237,7 @@ static ssize_t readpkt(void)
     if ((changed & ERROR_IS) != 0)
 	longjmp(terminate, TERM_READ_ERROR);
 
-    if (logfile != NULL) {
+    if (logfile != NULL && session.packet.outbuflen > 0) {
 	/*@ -shiftimplementation -sefparams +charint @*/
 	assert(fwrite
 	       (session.packet.outbuffer, sizeof(char),
@@ -206,7 +250,7 @@ static ssize_t readpkt(void)
 
 /*@ +globstate @*/
 
-static void packet_dump(char *buf, size_t buflen)
+static void packet_dump(const char *buf, size_t buflen)
 {
     if (packetwin != NULL) {
 	size_t i;
@@ -230,21 +274,35 @@ static void packet_dump(char *buf, size_t buflen)
 }
 
 #if defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE)
-static void monitor_dump_send(void)
+static void monitor_dump_send(/*@in@*/ const char *buf, size_t len)
 {
     if (packetwin != NULL) {
 	(void)wattrset(packetwin, A_BOLD);
 	(void)wprintw(packetwin, ">>>");
-	packet_dump(session.msgbuf, session.msgbuflen);
+	packet_dump(buf, len);
 	(void)wattrset(packetwin, A_NORMAL);
     }
 }
+
+static void announce_log(/*@in@*/ const char *str)
+{
+   if (packetwin != NULL) {
+	(void)wattrset(packetwin, A_BOLD);
+	(void)wprintw(packetwin, ">>>");
+	(void)waddstr(packetwin, str);
+	(void)wattrset(packetwin, A_NORMAL);
+	(void)wprintw(packetwin, "\n");
+   }
+   if (logfile != NULL) {
+       (void)fprintf(logfile, ">>>%s\n", str);
+   }
+}
 #endif /* defined(ALLOW_CONTROLSEND) || defined(ALLOW_RECONFIGURE) */
 
 #ifdef ALLOW_CONTROLSEND
 bool monitor_control_send( /*@in@*/ unsigned char *buf, size_t len)
 {
-    if (controlfd == -1)
+    if ((controlfd == -1) || (session.gpsdata.dev.path[0] == '\0'))
 	return false;
     else {
 	int savefd;
@@ -276,14 +334,14 @@ bool monitor_control_send( /*@in@*/ unsigned char *buf, size_t len)
 	    assert(read(controlfd, buf, 8) != -1);
 	    /*@ +sefparams @*/
 	}
-	monitor_dump_send();
+	monitor_dump_send((const char *)buf, len);
 	return (st != -1);
     }
 }
 
 static bool monitor_raw_send( /*@in@*/ unsigned char *buf, size_t len)
 {
-    if (controlfd == -1)
+    if ((controlfd == -1) || (session.gpsdata.dev.path[0] == '\0'))
 	return false;
     else {
 	ssize_t st;
@@ -305,9 +363,7 @@ static bool monitor_raw_send( /*@in@*/ unsigned char *buf, size_t len)
 	    assert(read(controlfd, buf, 8) != -1);
 	    /*@ +sefparams @*/
 	}
-	(void)memcpy(session.msgbuf, buf, len);
-	session.msgbuflen = len;
-	monitor_dump_send();
+	monitor_dump_send((const char *)buf, len);
 	return (st > 0 && (size_t) st == len);
     }
 }
@@ -319,28 +375,6 @@ static bool monitor_raw_send( /*@in@*/ unsigned char *buf, size_t len)
  *
  *****************************************************************************/
 
-static long tzoffset(void)
-{
-    time_t now = time(NULL);
-    struct tm tm;
-    long res = 0;
-
-    tzset();
-#ifdef HAVE_TIMEZONE
-    res = timezone;
-#else
-    res = localtime_r(&now, &tm)->tm_gmtoff;
-#endif
-#ifdef HAVE_DAYLIGHT
-    if (daylight != 0 && localtime_r(&now, &tm)->tm_isdst != 0)
-	res -= 3600;
-#else
-    if (localtime_r(&now, &tm)->tm_isdst)
-	res -= 3600;
-#endif
-    return res;
-}
-
 void monitor_complain(const char *fmt, ...)
 {
     va_list ap;
@@ -434,14 +468,13 @@ int main(int argc, char **argv)
     fd_set select_set;
     unsigned char buf[BUFLEN];
     char line[80], *explanation;
-    int bailout = 0;
+    int bailout = 0, matches = 0;
 
-    gmt_offset = (int)tzoffset();
     /*@ -observertrans @*/
-    (void)putenv("TZ=GMT");	// for ctime()
+    (void)putenv("TZ=UTC");	// for ctime()
     /*@ +observertrans @*/
     /*@ -branchstate @*/
-    while ((option = getopt(argc, argv, "D:F:LVhl:")) != -1) {
+    while ((option = getopt(argc, argv, "D:F:LVhl:t:?")) != -1) {
 	switch (option) {
 	case 'D':
 	    debuglevel = atoi(optarg);
@@ -449,9 +482,6 @@ int main(int argc, char **argv)
 	case 'F':
 	    controlsock = optarg;
 	    break;
-	case 'V':
-	    (void)printf("gpsmon: %s (revision %s)\n", VERSION, REVISION);
-	    exit(0);
 	case 'L':		/* list known device types */
 	    (void)
 		fputs
@@ -493,6 +523,9 @@ int main(int argc, char **argv)
 		(void)fputc('\n', stdout);
 	    }
 	    exit(0);
+	case 'V':
+	    (void)printf("gpsmon: %s (revision %s)\n", VERSION, REVISION);
+	    exit(0);
 	case 'l':		/* enable logging at startup */
 	    logfile = fopen(optarg, "w");
 	    if (logfile == NULL) {
@@ -500,12 +533,31 @@ int main(int argc, char **argv)
 		exit(1);
 	    }
 	    break;
+        case 't':
+	    fallback = NULL;
+	    for (active = monitor_objects; *active; active++) {
+		if (strncmp((*active)->driver->type_name, optarg, strlen(optarg)) == 0)
+		{
+		    fallback = active;
+		    matches++;
+		}
+	    }
+	    if (matches > 1) { 
+		(void)fprintf(stderr, "-T option matched more than one driver.\n");
+		exit(1);
+	    }
+	    else if (matches == 0) { 
+		(void)fprintf(stderr, "-T option didn't match any driver.\n");
+		exit(1);
+	    }
+	    active = NULL;
+	    break;
 	case 'h':
 	case '?':
 	default:
 	    (void)
 		fputs
-		("usage:  gpsmon [-?hVl] [-D debuglevel] [-F controlsock] [server[:port:[device]]]\n",
+		("usage:  gpsmon [-?hVl] [-D debuglevel] [-F controlsock] [-t type] [server[:port:[device]]]\n",
 		 stderr);
 	    exit(1);
 	}
@@ -517,6 +569,7 @@ int main(int argc, char **argv)
     } else
 	gpsd_source_spec(NULL, &source);
 
+    gpsd_time_init(&context, time(NULL));
     gpsd_init(&session, &context, NULL);
 
     /*@ -boolops */
@@ -527,7 +580,7 @@ int main(int argc, char **argv)
 #else
 	) {
 #endif
-	(void)gps_open_r(source.server, source.port, &session.gpsdata);
+	(void)gps_open(source.server, source.port, &session.gpsdata);
 	if (session.gpsdata.gps_fd < 0) {
 	    (void)fprintf(stderr,
 			  "%s: connection failure on %s:%s, error %d = %s.\n",
@@ -537,12 +590,20 @@ int main(int argc, char **argv)
 	    exit(1);
 	}
 	controlfd = open(controlsock, O_RDWR);
-	if (source.device != NULL)
+	if (source.device != NULL) {
 	    (void)gps_send(&session.gpsdata,
 			   "?WATCH={\"raw\":2,\"device\":\"%s\"}\r\n",
 			   source.device);
-	else
+	    /*
+	     *  The gpsdata.dev is filled only in JSON mode,
+	     *  but we in super-raw mode.
+	     */
+	    (void)strlcpy(session.gpsdata.dev.path, source.device,
+			  sizeof(session.gpsdata.dev.path));
+	} else {
 	    (void)gps_send(&session.gpsdata, "?WATCH={\"raw\":2}\r\n");
+	    session.gpsdata.dev.path[0] = '\0';
+	}
 	serial = false;
     } else {
 	(void)strlcpy(session.gpsdata.dev.path, argv[optind],
@@ -702,17 +763,23 @@ int main(int argc, char **argv)
 			monitor_complain("No device defined yet");
 		    else if (serial) {
 			double rate = strtod(arg, NULL);
+			const struct monitor_object_t **switcher = active;
+
+			if (fallback != NULL && (*fallback)->driver->rate_switcher != NULL)
+			    switcher = fallback;
 			/* Ugh...should have a controlfd slot
 			 * in the session structure, really
 			 */
-			if ((*active)->driver->rate_switcher) {
+			if ((*switcher)->driver->rate_switcher) {
 			    int dfd = session.gpsdata.gps_fd;
 			    session.gpsdata.gps_fd = controlfd;
 			    /* *INDENT-OFF* */
-			    if ((*active)->driver->rate_switcher(&session, rate)) {
-				monitor_dump_send();
+			    context.readonly = false;
+			    if ((*switcher)->driver->rate_switcher(&session, rate)) {
+				announce_log("Rate switcher callled.");
 			    } else
 				monitor_complain("Rate not supported.");
+			    context.readonly = true;
 			    /* *INDENT-ON* */
 			    session.gpsdata.gps_fd = dfd;
 			} else
@@ -774,15 +841,21 @@ int main(int argc, char **argv)
 		    if (active == NULL)
 			monitor_complain("No device defined yet");
 		    else if (serial) {
+			const struct monitor_object_t **switcher = active;
+
+			if (fallback != NULL && (*fallback)->driver->mode_switcher != NULL)
+			    switcher = fallback;
 			/* Ugh...should have a controlfd slot
 			 * in the session structure, really
 			 */
-			if ((*active)->driver->mode_switcher) {
+			if ((*switcher)->driver->mode_switcher) {
 			    int dfd = session.gpsdata.gps_fd;
 			    session.gpsdata.gps_fd = controlfd;
-			    (*active)->driver->mode_switcher(&session,
+			    context.readonly = false;
+			    (*switcher)->driver->mode_switcher(&session,
 							     (int)v);
-			    monitor_dump_send();
+			    context.readonly = true;
+			    announce_log("Mode switcher called");
 			    (void)tcdrain(session.gpsdata.gps_fd);
 			    (void)usleep(50000);
 			    session.gpsdata.gps_fd = dfd;
@@ -818,6 +891,10 @@ int main(int argc, char **argv)
 			unsigned int stopbits =
 			    (unsigned int)session.gpsdata.dev.stopbits;
 			char *modespec;
+			const struct monitor_object_t **switcher = active;
+
+			if (fallback != NULL && (*fallback)->driver->speed_switcher != NULL)
+			    switcher = fallback;
 
 			modespec = strchr(arg, ':');
 			/*@ +charint @*/
@@ -846,14 +923,15 @@ int main(int argc, char **argv)
 			 * in the session structure, really
 			 */
 			/* *INDENT-OFF* */
-			if ((*active)->driver->speed_switcher) {
+			if ((*switcher)->driver->speed_switcher) {
 			    int dfd = session.gpsdata.gps_fd;
 			    session.gpsdata.gps_fd = controlfd;
-			    if ((*active)->
+			    context.readonly = false;
+			    if ((*switcher)->
 				driver->speed_switcher(&session, speed,
 						       parity, (int)
 						       stopbits)) {
-				monitor_dump_send();
+				announce_log("Speed switcher called.");
 				/*
 				 * See the comment attached to the 'B'
 				 * command in gpsd.  Allow the control
@@ -869,6 +947,7 @@ int main(int argc, char **argv)
 			    } else
 				monitor_complain
 				    ("Speed/mode combination not supported.");
+			    context.readonly = true;
 			    session.gpsdata.gps_fd = dfd;
 			} else
 			    monitor_complain
