@@ -2,12 +2,6 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
 #include <string.h>
 
 #include "gpsd.h"
