@@ -9,7 +9,7 @@ Notation).  Unlike more general JSON parsers, it doesn't use malloc(3)
 and doesn't support polymorphism; you need to give it a set of
 template structures describing the expected shape of the incoming
 JSON, and it will error out if that shape is not matched.  When the
-parse succeds, attribute values will be extracted into static
+parse succeeds, attribute values will be extracted into static
 locations specified in the template structures.
 
    The "shape" of a JSON object in the type signature of its
@@ -20,6 +20,12 @@ advance what the type of each attribute value will be and where the
 parsed value will be stored. The template structures may supply
 default values to be used when an expected attribute is omitted.
 
+   The preceding paragraph told one fib.  A single attribute may 
+actually have a span of multiple specifications with different
+syntactically distinguishable types (e.g. string vs. real vs. integer
+vs boolean, but not signed integer vs. unsigned integer or strong vs. map).
+The parser will match the right spec against the actual data.
+
    The dialect this parses has some limitations.  First, it cannot
 recognize the JSON "null" value.  Secondly, arrays may only have
 objects or strings - not reals or integers or floats - as elements
@@ -36,15 +42,15 @@ structobject.
 
    Object arrays take one base address per object subfield, and are 
 mapped into parallel C arrays (one per subfield).  Strings are not
-supported in this kind of array, as the don't have a "natural" size
+supported in this kind of array, as they don't have a "natural" size
 to use as an offset multiplier.
 
    Structobjects arrays are a way to parse a list of objects to a set
 of modifications to a corresponding array of C structs.  The trick is
 that the array object initialization has to specify both the C struct
 array's base address and the stride length (the size of the C struct).
-If you initialize the offset fields with the correct offsetof() calls,
-everything will work. Strings are suppported but all string storage
+If you initialize the offset fields with the correct offsetof calls,
+everything will work. Strings are supported but all string storage
 has to be inline in the struct.
 
 PERMISSIONS
@@ -55,7 +61,10 @@ PERMISSIONS
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <stdarg.h>
+#include <locale.h>
+#include <ctype.h>
 
 #include "gpsd_config.h"	/* for strlcpy() prototype */
 #include "json.h"
@@ -362,6 +371,34 @@ static int json_internal_read_object(const char *cp,
 		*pval++ = *cp;
 	    break;
 	case post_val:
+	    /*
+	     * We know that cursor points at the first spec matching
+	     * the current attribute.  We don't know that it's *the*
+	     * correct spec; our dialect allows there to be any number
+	     * of adjacent ones with the same attrname but different
+	     * types.  Here's where we try to seek forward for a
+	     * matching type/attr pair if we're not looking at one.
+	     */
+	    for (;;) {
+		int seeking = cursor->type;
+		if (value_quoted && cursor->type == t_string)
+		    break;
+		if ((strcmp(valbuf, "true")==0 || strcmp(valbuf, "false")==0)
+			&& seeking == t_boolean)
+		    break;
+		if (isdigit(valbuf[0])) {
+		    bool decimal = strchr(valbuf, '.') != NULL;
+		    if (decimal && seeking == t_real)
+			break;
+		    if (!decimal && (seeking == t_integer || seeking == t_uinteger))
+			break;
+		}
+		if (cursor[1].attribute==NULL)	/* out of possiblities */
+		    break;
+		if (strcmp(cursor[1].attribute, attrbuf)!=0)
+		    break;
+		++cursor;
+	    }
 	    if (value_quoted
 		&& (cursor->type != t_string && cursor->type != t_character
 		    && cursor->type != t_check && cursor->map == 0)) {
@@ -555,8 +592,14 @@ int json_read_array(const char *cp, const struct json_array_t *arr,
 int json_read_object(const char *cp, const struct json_attr_t *attrs,
 		     /*@null@*/ const char **end)
 {
+    char *savedlocale = setlocale(LC_ALL, NULL);
+    int st;
+
+    (void)setlocale(LC_ALL, "C");
     json_debug_trace((1, "json_read_object() sees '%s'\n", cp));
-    return json_internal_read_object(cp, attrs, NULL, 0, end);
+    st = json_internal_read_object(cp, attrs, NULL, 0, end);
+    (void)setlocale(LC_ALL, savedlocale);
+    return st;
 }
 
 const /*@observer@*/ char *json_error_string(int err)
