@@ -34,61 +34,35 @@
 
 #define CLIMB 3
 
-#include <stdlib.h>
-#include "gpsd_config.h"
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #ifndef S_SPLINT_S
-#include <unistd.h>
+#include <netdb.h>
+#ifndef AF_UNSPEC
+#include <sys/socket.h>
+#endif /* AF_UNSPEC */
 #endif /* S_SPLINT_S */
+#ifndef INADDR_ANY
+#include <netinet/in.h>
+#endif /* INADDR_ANY */
+#include <stdlib.h>
+#include <string.h>
 #include <math.h>
 #include <errno.h>
-#ifdef HAVE_SYS_SELECT_H
- #include <sys/select.h>
-#endif /* HAVE_SYS_SELECT_H */
+#include <stdio.h>
 #ifndef S_SPLINT_S
- #ifdef HAVE_SYS_SOCKET_H
-  #include <sys/socket.h>
- #endif /* HAVE_SYS_SOCKET_H */
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
+#include <unistd.h>
 #endif /* S_SPLINT_S */
 
-#include <sys/time.h> /* select() */
-#include <sys/stat.h>
-#include <fcntl.h>
-#ifdef HAVE_TERMIOS_H
- #include <termios.h>
-#endif /* HAVE_TERMIOS_H */
-
-#ifndef S_SPLINT_S
- #ifdef HAVE_NETINET_IN_H
-  #include <netinet/in.h>
- #endif /* HAVE_NETINET_IN_H */
- #ifdef HAVE_ARPA_INET_H
-  #include <arpa/inet.h>
- #endif /* HAVE_ARPA_INET_H */
- #ifdef HAVE_NETDB_H
-  #include <netdb.h>
- #endif /* HAVE_NETDB_H */
-#endif /* S_SPLINT_S */
-#include <errno.h>
-
-#include <signal.h>
-
 #include "gps.h"
 #include "gpsdclient.h"
 #include "revision.h"
 
-/* Macro for declaring function arguments unused. */
-#if defined(__GNUC__)
-#  define UNUSED __attribute__((unused)) /* Flag variable as unused */
-#else /* not __GNUC__ */
-#  define UNUSED
-#endif
-
 /* Prototypes. */
 void latlon2maidenhead(char *st,float n,float e);
-static void daemonize(void);
 ssize_t sockreadline(int sockd,void *vptr,size_t maxlen);
 ssize_t sockwriteline(int sockd,const void *vptr,size_t n);
 int send_lcd(char *buf);
@@ -99,10 +73,7 @@ static float altfactor = METERS_TO_FEET;
 static float speedfactor = MPS_TO_MPH;
 static char *altunits = "ft";
 static char *speedunits = "mph";
-
-#ifdef CLIMB
 double avgclimb, climb[CLIMB];
-#endif
 
 /* Global socket descriptor for LCDd. */
 int sd;
@@ -135,42 +106,6 @@ void latlon2maidenhead(char *st,float n,float e)
 #endif
 }
 
-/* Daemonize me. */
-static void daemonize(void) {
-  int i, ignore;
-
-  /* Run as my child. */
-  i=fork();
-  if (i == -1) exit(1); /* fork error */
-  if (i > 0) exit(0); /* parent exits */
-
-  /* Obtain a new process group. */
-  setsid();
-
-  /* Close all open descriptors. */
-  for(i=getdtablesize();i>=0;--i)
-    close(i);
-
-  /* Reopen STDIN, STDOUT, STDERR to /dev/null. */
-  i=open("/dev/null",O_RDWR); /* STDIN */
-  ignore=dup(i); /* STDOUT */
-  ignore=dup(i); /* STDERR */
-
-  /* Know thy mask. */
-  umask(027);
-
-  /* Run from a known location. */
-  ignore=chdir("/");
-
-  /* Catch child sig */
-  signal(SIGCHLD,SIG_IGN);
-
-  /* Ignore tty signals */
-  signal(SIGTSTP,SIG_IGN);
-  signal(SIGTTOU,SIG_IGN);
-  signal(SIGTTIN,SIG_IGN);
-}
-
 /*  Read a line from a socket  */
 ssize_t sockreadline(int sockd,void *vptr,size_t maxlen) {
   ssize_t n,rc;
@@ -272,26 +207,15 @@ static void reset_lcd(void) {
 static enum deg_str_type deg_type = deg_dd;
 
 /* This gets called once for each new sentence. */
-static void update_lcd(struct gps_data_t *gpsdata,
-                       char *message UNUSED,
-                       size_t len UNUSED)
+static void update_lcd(struct gps_data_t *gpsdata)
 {
   char tmpbuf[255];
-#ifdef CLIMB
   char maidenhead[5];
   maidenhead[4]=0;
   int n;
-#else
-  char maidenhead[7];
-  maidenhead[6]=0;
-#endif
   char *s;
   int track;
 
-  /* this is where we implement source-device filtering */
-  if (gpsdata->dev.path[0] && source.device!=NULL && strcmp(source.device, gpsdata->dev.path) != 0)
-      return;
-
   /* Get our location in Maidenhead. */
   latlon2maidenhead(maidenhead,gpsdata->fix.latitude,gpsdata->fix.longitude);
 
@@ -325,7 +249,6 @@ static void update_lcd(struct gps_data_t *gpsdata,
 
   /* Fill in the altitude and fix status. */
   if (gpsdata->fix.mode == MODE_3D) {
-#ifdef CLIMB
     for(n=0;n<CLIMB-2;n++) climb[n]=climb[n+1];
     climb[CLIMB-1]=gpsdata->fix.climb;
     avgclimb=0.0;
@@ -333,10 +256,6 @@ static void update_lcd(struct gps_data_t *gpsdata,
     avgclimb/=CLIMB;
     snprintf(tmpbuf, 254, "widget_set gpsd four 1 4 {%d %s %s %d fpm       }\n",
             (int)(gpsdata->fix.altitude*altfactor), altunits, maidenhead, (int)(avgclimb * METERS_TO_FEET * 60));
-#else
-    snprintf(tmpbuf, 254, "widget_set gpsd four 1 4 {%.1f %s  %s}\n",
-            gpsdata->fix.altitude*altfactor, altunits, maidenhead);
-#endif
   } else {
     snprintf(tmpbuf, 254, "widget_set gpsd four 1 4 {n/a}\n");
   }
@@ -370,14 +289,8 @@ int main(int argc, char *argv[])
     struct sockaddr_in localAddr, servAddr;
     struct hostent *h;
 
-    struct timeval timeout;
-    fd_set rfds;
-    int data;
-
-#ifdef CLIMB
     int n;
     for(n=0;n<CLIMB;n++) climb[n]=0.0;
-#endif 
 
     /*@ -observertrans @*/
     switch (gpsd_units())
@@ -468,12 +381,15 @@ int main(int argc, char *argv[])
       gpsd_source_spec(NULL, &source);
 
     /* Daemonize... */
-    daemonize();
+  if (daemon(0, 0) != 0)
+      (void)fprintf(stderr,
+		    "lcdgps: demonization failed: %s\n",
+		    strerror(errno));
 
     /* Open the stream to gpsd. */
-    if (gps_open_r(source.server, source.port, &gpsdata) != 0) {
+    if (gps_open(source.server, source.port, &gpsdata) != 0) {
 	(void)fprintf( stderr,
-		       "cgps: no gpsd running or network error: %d, %s\n",
+		       "lcdgps: no gpsd running or network error: %d, %s\n",
 		       errno, gps_errstr(errno));
 	exit(2);
     }
@@ -519,28 +435,18 @@ int main(int argc, char *argv[])
     reset_lcd();
 
     /* Here's where updates go. */
-    gps_set_raw_hook(&gpsdata, update_lcd);
-    gps_stream(&gpsdata, WATCH_ENABLE, NULL);
+    unsigned int flags = WATCH_ENABLE;
+    if (source.device != NULL)
+	flags |= WATCH_DEVICE;
+    (void)gps_stream(&gpsdata, flags, source.device);
 
     for (;;) { /* heart of the client */
-
-	/* watch to see when it has input */
-	FD_ZERO(&rfds);
-	FD_SET(gpsdata.gps_fd, &rfds);
-
-	/* wait up to five seconds. */
-	timeout.tv_sec = 5;
-	timeout.tv_usec = 0;
-
-	/* check if we have new information */
-	data = select(gpsdata.gps_fd + 1, &rfds, NULL, NULL, &timeout);
-
-	if (data == -1) {
-	    fprintf( stderr, "cgps: socket error\n");
+	if (!gps_waiting(&gpsdata, 50000000)) {
+	    fprintf( stderr, "lcdgps: error while waiting\n");
 	    exit(2);
-	}
-	else if (data) {
+	} else {
 	    (void)gps_read(&gpsdata);
+	    update_lcd(&gpsdata);
 	}
 
     }
