@@ -3,39 +3,36 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <sys/time.h>
 #include <stdio.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <sys/types.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <fcntl.h>
 #include <string.h>
+#include <ctype.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <math.h>
 #include <locale.h>
 #include <assert.h>
-
-#include "gpsd.h"
-#include "gps_json.h"
+#include <sys/time.h>	 /* expected to have a select(2) prototype a la SuS */
+#include <sys/types.h>
+#include <sys/stat.h>
+#ifndef S_SPLINT_S
+#include <sys/socket.h>
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 
 #ifndef USE_QT
 #ifndef S_SPLINT_S
-#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
-#else
-#define AF_UNSPEC 0
-#endif
-#endif
-#if defined (HAVE_SYS_SELECT_H)
-#include <sys/select.h>
-#endif
+#endif /* S_SPLINT_S */
 #else
 #include <QTcpSocket>
 #endif /* USE_QT */
 
+#include "gpsd.h"
+#include "gps_json.h"
+
 #ifdef S_SPLINT_S
 extern char *strtok_r(char *, const char *, char **);
 #endif /* S_SPLINT_S */
@@ -47,15 +44,21 @@ extern char *strtok_r(char *, const char *, char **);
 struct privdata_t
 {
     bool newstyle;
+    /* data buffered from the last read */
     ssize_t waiting;
     char buffer[GPS_JSON_RESPONSE_MAX * 2];
+
 };
 #define PRIVATE(gpsdata) ((struct privdata_t *)gpsdata->privdata)
 
 #ifdef LIBGPS_DEBUG
+#define DEBUG_CALLS	1	/* shallowest debug level */
+#define DEBUG_JSON	5	/* minimum level for verbose JSON debugging */
 static int debuglevel = 0;
-static int waitcount = 0;
 static FILE *debugfp;
+#ifndef USE_QT
+static int waitcount = 0;
+#endif /* USE_QT */
 
 void gps_enable_debug(int level, FILE * fp)
 /* control the level and destination of debug trace messages */
@@ -63,7 +66,7 @@ void gps_enable_debug(int level, FILE * fp)
     debuglevel = level;
     debugfp = fp;
 #ifdef CLIENTDEBUG_ENABLE
-    json_enable_debug(level - 2, fp);
+    json_enable_debug(level - DEBUG_JSON, fp);
 #endif
 }
 
@@ -90,7 +93,7 @@ static void gps_trace(int errlevel, const char *fmt, ...)
 #endif /* LIBGPS_DEBUG */
 
 /*@-nullderef@*/
-int gps_open_r(/*@null@*/const char *host, /*@null@*/const char *port,
+int gps_open(/*@null@*/const char *host, /*@null@*/const char *port,
 	       /*@out@*/ struct gps_data_t *gpsdata)
 {
     /*@ -branchstate @*/
@@ -101,14 +104,18 @@ int gps_open_r(/*@null@*/const char *host, /*@null@*/const char *port,
     if (!port)
 	port = DEFAULT_GPSD_PORT;
 
-    libgps_debug_trace((1, "gps_open_r(%s, %s)\n", host, port));
+    libgps_debug_trace((DEBUG_CALLS, "gps_open(%s, %s)\n", host, port));
 
 #ifndef USE_QT
     if ((gpsdata->gps_fd =
 	 netlib_connectsock(AF_UNSPEC, host, port, "tcp")) < 0) {
 	errno = gpsdata->gps_fd;
+	libgps_debug_trace((DEBUG_CALLS, "netlib_connectsock() returns error %d\n", errno));
 	return -1;
     }
+    else
+	libgps_debug_trace((DEBUG_CALLS, "netlib_connectsock() returns socket on fd %d\n",
+			    gpsdata->gps_fd));
 #else
     QTcpSocket *sock = new QTcpSocket();
     gpsdata->gps_fd = sock;
@@ -129,30 +136,15 @@ int gps_open_r(/*@null@*/const char *host, /*@null@*/const char *port,
 	return -1;
     PRIVATE(gpsdata)->newstyle = false;
     PRIVATE(gpsdata)->waiting = 0;
-    /*@i2@*/ PRIVATE(gpsdata)->buffer[0] = '\0';
-
     return 0;
     /*@ +branchstate @*/
 }
 
-/*@-compmempass -immediatetrans@*/
-struct gps_data_t *gps_open(const char *host, const char *port)
-/* open a connection to a gpsd daemon */
-{
-    static struct gps_data_t gpsdata;
-    if (gps_open_r(host, port, &gpsdata) == -1)
-	return NULL;
-    else
-	return &gpsdata;
-}
-
-/*@+compmempass +immediatetrans@*/
-
 /*@-compdef -usereleased@*/
 int gps_close(struct gps_data_t *gpsdata)
 /* close a gpsd connection */
 {
-    libgps_debug_trace((1, "gps_close()\n"));
+    libgps_debug_trace((DEBUG_CALLS, "gps_close()\n"));
 #ifndef USE_QT
     free(PRIVATE(gpsdata));
     (void)close(gpsdata->gps_fd);
@@ -169,22 +161,17 @@ int gps_close(struct gps_data_t *gpsdata)
 
 /*@+compdef +usereleased@*/
 
-void gps_set_raw_hook(struct gps_data_t *gpsdata,
-		      void (*hook) (struct gps_data_t *, char *, size_t len))
-{
-    gpsdata->raw_hook = hook;
-}
-
 #ifdef LIBGPS_DEBUG
-static void libgps_dump_state(struct gps_data_t *collect, time_t now)
+static void libgps_dump_state(struct gps_data_t *collect)
 {
-    char *status_values[] = { "NO_FIX", "FIX", "DGPS_FIX" };
-    char *mode_values[] = { "", "NO_FIX", "MODE_2D", "MODE_3D" };
+    const char *status_values[] = { "NO_FIX", "FIX", "DGPS_FIX" };
+    const char *mode_values[] = { "", "NO_FIX", "MODE_2D", "MODE_3D" };
 
     /* no need to dump the entire state, this is a sanity check */
 #ifndef USE_QT
+    /* will fail on a 32-bit macine */
     (void)fprintf(debugfp, "flags: (0x%04x) %s\n",
-		  collect->set, gps_maskdump(collect->set));
+		  (unsigned int)collect->set, gps_maskdump(collect->set));
 #endif
     if (collect->set & ONLINE_SET)
 	(void)fprintf(debugfp, "ONLINE: %lf\n", collect->online);
@@ -265,21 +252,24 @@ static void libgps_dump_state(struct gps_data_t *collect, time_t now)
 
 /*@ -branchstate -usereleased -mustfreefresh -nullstate -usedef @*/
 int gps_unpack(char *buf, struct gps_data_t *gpsdata)
-/* unpack a gpsd response into a status structure, buf must be writeable */
+/* unpack a gpsd response into a status structure, buf must be writeable.
+ * gps_unpack() currently returns 0 in all cases, but should it ever need to
+ * return an error status, it must be < 0.
+ */
 {
-    libgps_debug_trace((1, "gps_unpack(%s)\n", buf));
+    libgps_debug_trace((DEBUG_CALLS, "gps_unpack(%s)\n", buf));
 
     /* detect and process a JSON response */
     if (buf[0] == '{') {
 	const char *jp = buf, **next = &jp;
 	while (next != NULL && *next != NULL && next[0][0] != '\0') {
-	    libgps_debug_trace((1,
+	    libgps_debug_trace((DEBUG_CALLS,
 				"gps_unpack() segment parse '%s'\n", *next));
 	    if (libgps_json_unpack(*next, gpsdata, next) == -1)
 		break;
 #ifdef LIBGPS_DEBUG
 	    if (debuglevel >= 1)
-		libgps_dump_state(gpsdata, time(NULL));
+		libgps_dump_state(gpsdata);
 #endif /* LIBGPS_DEBUG */
 
 	}
@@ -426,7 +416,7 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			break;
 		    case 'X':
 			if (sp[2] == '?')
-			    gpsdata->online = -1;
+			    gpsdata->online = (timestamp_t)-1;
 			else {
 			    (void)sscanf(sp, "X=%lf", &gpsdata->online);
 			    gpsdata->set |= ONLINE_SET;
@@ -487,7 +477,7 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
 
 #ifdef LIBGPS_DEBUG
 		    if (debuglevel >= 1)
-			libgps_dump_state(gpsdata, time(NULL));
+			libgps_dump_state(gpsdata);
 #endif /* LIBGPS_DEBUG */
 
 		    /*
@@ -503,13 +493,8 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
     }
 #endif /* OLDSTYLE_ENABLE */
 
-/*@ -compdef @*/
-    if (gpsdata->raw_hook) {
-	//libgps_debug_trace((stderr, "libgps: raw hook called on '%s'\n", buf));
-	gpsdata->raw_hook(gpsdata, buf, strlen(buf));
-    }
 #ifndef USE_QT
-    libgps_debug_trace((1, "final flags: (0x%04x) %s\n", gpsdata->set,
+    libgps_debug_trace((DEBUG_CALLS, "final flags: (0x%04x) %s\n", gpsdata->set,
 			gps_maskdump(gpsdata->set)));
 #endif
     return 0;
@@ -518,25 +503,28 @@ int gps_unpack(char *buf, struct gps_data_t *gpsdata)
 /*@ +compdef @*/
 /*@ -branchstate +usereleased +mustfreefresh +nullstate +usedef @*/
 
-bool gps_waiting(struct gps_data_t * gpsdata)
+bool gps_waiting(struct gps_data_t * gpsdata, int timeout)
 /* is there input waiting from the GPS? */
 {
 #ifndef USE_QT
     fd_set rfds;
     struct timeval tv;
 
-    libgps_debug_trace((1, "gps_waiting(): %d\n", waitcount++));
+    libgps_debug_trace((DEBUG_CALLS, "gps_waiting(%d): %d\n", timeout, waitcount++));
     if (PRIVATE(gpsdata)->waiting > 0)
 	return true;
 
+    /* we might want to check for EINTR if this returns false */
+    errno = 0;
+
     FD_ZERO(&rfds);
     FD_SET(gpsdata->gps_fd, &rfds);
-    tv.tv_sec = 0;
-    tv.tv_usec = 1;
+    tv.tv_sec = timeout / 1000000;
+    tv.tv_usec = timeout % 1000000;
     /* all error conditions return "not waiting" -- crude but effective */
     return (select(gpsdata->gps_fd + 1, &rfds, NULL, NULL, &tv) == 1);
 #else
-    return ((QTcpSocket *) (gpsdata->gps_fd))->waitForReadyRead(250);
+    return ((QTcpSocket *) (gpsdata->gps_fd))->waitForReadyRead(timeout / 1000);
 #endif
 }
 
@@ -548,34 +536,35 @@ int gps_read(/*@out@*/struct gps_data_t *gpsdata)
     double received = 0;
     ssize_t response_length;
     int status = -1;
-    struct privdata_t *priv = PRIVATE(gpsdata);
 
     gpsdata->set &= ~PACKET_SET;
-    for (eol = priv->buffer;
-	 *eol != '\n' && eol < priv->buffer + priv->waiting; eol++)
+    for (eol = PRIVATE(gpsdata)->buffer;
+	 *eol != '\n' && eol < PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting; eol++)
 	continue;
     if (*eol != '\n')
 	eol = NULL;
 
+    errno = 0;
+
     if (eol == NULL) {
 #ifndef USE_QT
 	/* read data: return -1 if no data waiting or buffered, 0 otherwise */
 	status = (int)recv(gpsdata->gps_fd,
-			   priv->buffer + priv->waiting,
-			   sizeof(priv->buffer) - priv->waiting, 0);
+			   PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting,
+			   sizeof(PRIVATE(gpsdata)->buffer) - PRIVATE(gpsdata)->waiting, 0);
 #else
 	status =
-	    ((QTcpSocket *) (gpsdata->gps_fd))->read(priv->buffer +
-						     priv->waiting,
-						     sizeof(priv->buffer) -
-						     priv->waiting);
+	    ((QTcpSocket *) (gpsdata->gps_fd))->read(PRIVATE(gpsdata)->buffer +
+						     PRIVATE(gpsdata)->waiting,
+						     sizeof(PRIVATE(gpsdata)->buffer) -
+						     PRIVATE(gpsdata)->waiting);
 #endif
 
 	/* if we just received data from the socket, it's in the buffer */
 	if (status > -1)
-	    priv->waiting += status;
+	    PRIVATE(gpsdata)->waiting += status;
 	/* buffer is empty - implies no data was read */
-	if (priv->waiting == 0) {
+	if (PRIVATE(gpsdata)->waiting == 0) {
 	    /* 
 	     * If we received 0 bytes, other side of socket is closing.
 	     * Return -1 as end-of-data indication.
@@ -593,8 +582,8 @@ int gps_read(/*@out@*/struct gps_data_t *gpsdata)
 		return -1;
 	}
 	/* there's buffered data waiting to be returned */
-	for (eol = priv->buffer;
-	     *eol != '\n' && eol < priv->buffer + priv->waiting; eol++)
+	for (eol = PRIVATE(gpsdata)->buffer;
+	     *eol != '\n' && eol < PRIVATE(gpsdata)->buffer + PRIVATE(gpsdata)->waiting; eol++)
 	    continue;
 	if (*eol != '\n')
 	    eol = NULL;
@@ -604,29 +593,24 @@ int gps_read(/*@out@*/struct gps_data_t *gpsdata)
 
     assert(eol != NULL);
     *eol = '\0';
-    response_length = eol - priv->buffer + 1;
+    response_length = eol - PRIVATE(gpsdata)->buffer + 1;
     received = gpsdata->online = timestamp();
-    status = gps_unpack(priv->buffer, gpsdata);
+    status = gps_unpack(PRIVATE(gpsdata)->buffer, gpsdata);
     /*@+matchanyintegral@*/
-    memmove(priv->buffer,
-	    priv->buffer + response_length, priv->waiting - response_length);
+    memmove(PRIVATE(gpsdata)->buffer,
+	    PRIVATE(gpsdata)->buffer + response_length, PRIVATE(gpsdata)->waiting - response_length);
     /*@-matchanyintegral@*/
-    priv->waiting -= response_length;
+    PRIVATE(gpsdata)->waiting -= response_length;
     gpsdata->set |= PACKET_SET;
 
-    return status;
+    return (status == 0) ? (int)response_length : status;
 }
 /*@+compdef -usedef +uniondef@*/
 
-int gps_poll(/*@out@*/struct gps_data_t *gpsdata)
-/* for backwards compatibility */
+const char /*@observer@*/ *gps_data(struct gps_data_t *gpsdata)
+/* return the contents of the client data buffer */
 {
-    int status = gps_read(gpsdata);
-
-    if (status > 0)
-	status = 0;
-
-    return status;
+    return PRIVATE(gpsdata)->buffer;
 }
 
 int gps_send(struct gps_data_t *gpsdata, const char *fmt, ...)
@@ -666,14 +650,10 @@ int gps_stream(struct gps_data_t *gpsdata, unsigned int flags,
     if ((flags & (WATCH_JSON | WATCH_OLDSTYLE | WATCH_NMEA | WATCH_RAW)) == 0) {
 	flags |= WATCH_JSON;
     }
-#ifndef USE_QT
-    if (flags & POLL_NONBLOCK)
-	(void)fcntl(gpsdata->gps_fd, F_SETFL, O_NONBLOCK);
-#endif
     if ((flags & WATCH_DISABLE) != 0) {
 	if ((flags & WATCH_OLDSTYLE) != 0) {
 	    (void)strlcpy(buf, "w-", sizeof(buf));
-	    if (gpsdata->raw_hook != NULL || (flags & WATCH_NMEA) != 0)
+	    if ((flags & WATCH_NMEA) != 0)
 		(void)strlcat(buf, "r-", sizeof(buf));
 	} else {
 	    (void)strlcpy(buf, "?WATCH={\"enable\":false,", sizeof(buf));
@@ -687,17 +667,19 @@ int gps_stream(struct gps_data_t *gpsdata, unsigned int flags,
 		(void)strlcat(buf, "\"raw\":0,", sizeof(buf));
 	    if (flags & WATCH_SCALED)
 		(void)strlcat(buf, "\"scaled\":false,", sizeof(buf));
+	    if (flags & WATCH_TIMING)
+		(void)strlcat(buf, "\"timing\":false,", sizeof(buf));
 	    if (buf[strlen(buf) - 1] == ',')
 		buf[strlen(buf) - 1] = '\0';
 	    (void)strlcat(buf, "};", sizeof(buf));
 	}
-	libgps_debug_trace((1, "gps_stream() disable command: %s\n", buf));
+	libgps_debug_trace((DEBUG_CALLS, "gps_stream() disable command: %s\n", buf));
 	return gps_send(gpsdata, buf);
     } else {			/* if ((flags & WATCH_ENABLE) != 0) */
 
 	if ((flags & WATCH_OLDSTYLE) != 0) {
 	    (void)strlcpy(buf, "w+x", sizeof(buf));
-	    if (gpsdata->raw_hook != NULL || (flags & WATCH_NMEA) != 0)
+	    if ((flags & WATCH_NMEA) != 0)
 		(void)strlcat(buf, "r+", sizeof(buf));
 	} else {
 	    (void)strlcpy(buf, "?WATCH={\"enable\":true,", sizeof(buf));
@@ -711,21 +693,23 @@ int gps_stream(struct gps_data_t *gpsdata, unsigned int flags,
 		(void)strlcat(buf, "\"raw\":2,", sizeof(buf));
 	    if (flags & WATCH_SCALED)
 		(void)strlcat(buf, "\"scaled\":true,", sizeof(buf));
+	    if (flags & WATCH_TIMING)
+		(void)strlcat(buf, "\"timing\":true,", sizeof(buf));
 	    /*@-nullpass@*//* shouldn't be needed, splint has a bug */
 	    if (flags & WATCH_DEVICE)
 		(void)snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-			       "\"device\":%s,", (char *)d);
+			       "\"device\":\"%s\",", (char *)d);
 	    /*@+nullpass@*/
 	    if (buf[strlen(buf) - 1] == ',')
 		buf[strlen(buf) - 1] = '\0';
 	    (void)strlcat(buf, "};", sizeof(buf));
 	}
-	libgps_debug_trace((1, "gps_stream() enable command: %s\n", buf));
+	libgps_debug_trace((DEBUG_CALLS, "gps_stream() enable command: %s\n", buf));
 	return gps_send(gpsdata, buf);
     }
 }
 
-extern char /*@observer@*/ *gps_errstr(const int err)
+extern const char /*@observer@*/ *gps_errstr(const int err)
 {
     /* 
      * We might add our own error codes in the future, e.g for
@@ -734,7 +718,9 @@ extern char /*@observer@*/ *gps_errstr(const int err)
 #ifndef USE_QT
     return netlib_errstr(err);
 #else
-    return "";
+    static char buf[32];
+    (void)snprintf(buf, sizeof(buf), "Qt error %d", err);
+    return buf;
 #endif
 }
 
@@ -744,12 +730,6 @@ extern char /*@observer@*/ *gps_errstr(const int err)
  * Not really useful for anything but debugging.
  */
 
-static void dumpline(struct gps_data_t *ud UNUSED,
-		     char *buf, size_t ulen UNUSED)
-{
-    puts(buf);
-}
-
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
@@ -767,7 +747,7 @@ static struct gps_data_t gpsdata;
 
 int main(int argc, char *argv[])
 {
-    struct gps_data_t *collect;
+    struct gps_data_t collect;
     char buf[BUFSIZ];
     int option;
     bool batchmode = false;
@@ -784,12 +764,12 @@ int main(int argc, char *argv[])
 	case 's':
 	    (void)
 		printf
-		("Sizes: gpsdata=%zd rtcm2=%zd rtcm3=%zd ais=%zd compass=%zd raw=%zd devices=%zd policy=%zd version=%zd\n",
+		("Sizes: gpsdata=%zd rtcm2=%zd rtcm3=%zd ais=%zd compass=%zd raw=%zd devices=%zd policy=%zd version=%zd, noise=%zd\n",
 		 sizeof(struct gps_data_t), sizeof(struct rtcm2_t),
 		 sizeof(struct rtcm3_t), sizeof(struct ais_t),
 		 sizeof(struct attitude_t), sizeof(struct rawdata_t),
-		 sizeof(collect->devices), sizeof(struct policy_t),
-		 sizeof(struct version_t));
+		 sizeof(collect.devices), sizeof(struct policy_t),
+		 sizeof(struct version_t), sizeof(struct gst_t));
 	    exit(0);
 	case 'D':
 	    debug = atoi(optarg);
@@ -807,24 +787,22 @@ int main(int argc, char *argv[])
 	while (fgets(buf, sizeof(buf), stdin) != NULL) {
 	    if (buf[0] == '{' || isalpha(buf[0])) {
 		gps_unpack(buf, &gpsdata);
-		libgps_dump_state(&gpsdata, time(NULL));
+		libgps_dump_state(&gpsdata);
 	    }
 	}
-    } else if ((collect = gps_open(NULL, 0)) == NULL) {
+    } else if (gps_open(NULL, 0, &collect) <= 0) {
 	(void)fputs("Daemon is not running.\n", stdout);
 	exit(1);
     } else if (optind < argc) {
-	gps_set_raw_hook(collect, dumpline);
 	(void)strlcpy(buf, argv[optind], BUFSIZ);
 	(void)strlcat(buf, "\n", BUFSIZ);
-	(void)gps_send(collect, buf);
-	(void)gps_read(collect);
-	libgps_dump_state(collect, time(NULL));
-	(void)gps_close(collect);
+	(void)gps_send(&collect, buf);
+	(void)gps_read(&collect);
+	libgps_dump_state(&collect);
+	(void)gps_close(&collect);
     } else {
 	int tty = isatty(0);
 
-	gps_set_raw_hook(collect, dumpline);
 	if (tty)
 	    (void)fputs("This is the gpsd exerciser.\n", stdout);
 	for (;;) {
@@ -835,12 +813,12 @@ int main(int argc, char *argv[])
 		    putchar('\n');
 		break;
 	    }
-	    collect->set = 0;
-	    (void)gps_send(collect, buf);
-	    (void)gps_read(collect);
-	    libgps_dump_state(collect, time(NULL));
+	    collect.set = 0;
+	    (void)gps_send(&collect, buf);
+	    (void)gps_read(&collect);
+	    libgps_dump_state(&collect);
 	}
-	(void)gps_close(collect);
+	(void)gps_close(&collect);
     }
 
     return 0;
