@@ -14,11 +14,10 @@ PERMISSIONS
 
 ***************************************************************************/
 
+#include <stdbool.h>
 #include <math.h>
-#include <assert.h>
 #include <string.h>
 #include <stddef.h>
-#include <stdio.h>
 
 #include "gpsd.h"
 #include "gps_json.h"
@@ -35,6 +34,7 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
 			 /*@null@*/ const char **endptr)
 {
     int status;
+    char tbuf[JSON_DATE_MAX+1];
     /*@ -fullinitblock @*/
     const struct json_attr_t json_attrs_1[] = {
 	/* *INDENT-OFF* */
@@ -43,6 +43,8 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
 			         .len = sizeof(gpsdata->dev.path)},
 	{"tag",    t_string,  .addr.string = gpsdata->tag,
 			         .len = sizeof(gpsdata->tag)},
+	{"time",   t_string,  .addr.string = tbuf,
+			         .len = sizeof(tbuf)},
 	{"time",   t_real,    .addr.real = &gpsdata->fix.time,
 			         .dflt.real = NAN},
 	{"ept",    t_real,    .addr.real = &gpsdata->fix.ept,
@@ -83,6 +85,14 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
     if (status == 0) {
 	gpsdata->status = STATUS_FIX;
 	gpsdata->set = STATUS_SET;
+	/*@-usedef@*/
+	if (isnan(gpsdata->fix.time)!=0) {
+	    if (tbuf[0] == '\0')
+		gpsdata->fix.time = NAN;
+	    else
+		gpsdata->fix.time = iso8601_to_unix(tbuf);
+	}
+	/*@+usedef@*/
 	if (isnan(gpsdata->fix.time) == 0)
 	    gpsdata->set |= TIME_SET;
 	if (isnan(gpsdata->fix.ept) == 0)
@@ -115,10 +125,64 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
     return status;
 }
 
+static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
+                           /*@null@*/ const char **endptr)
+{
+    int status;
+    char tbuf[JSON_DATE_MAX+1];
+    /*@ -fullinitblock @*/
+    const struct json_attr_t json_attrs_1[] = {
+	/* *INDENT-OFF* */
+	{"class",  t_check,   .dflt.check = "GST"},
+	{"device", t_string,  .addr.string = gpsdata->dev.path,
+			         .len = sizeof(gpsdata->dev.path)},
+	{"tag",    t_string,  .addr.string = gpsdata->tag,
+			         .len = sizeof(gpsdata->tag)},
+	{"time",   t_string,  .addr.string = tbuf,
+			         .len = sizeof(tbuf)},
+	{"time",   t_real,    .addr.real = &gpsdata->gst.utctime,
+			         .dflt.real = NAN},
+	{"rms",    t_real,    .addr.real = &gpsdata->gst.rms_deviation,
+			         .dflt.real = NAN},
+	{"major",  t_real,    .addr.real = &gpsdata->gst.smajor_deviation,
+			         .dflt.real = NAN},
+	{"minor",  t_real,    .addr.real = &gpsdata->gst.sminor_deviation,
+			         .dflt.real = NAN},
+	{"orient", t_real,    .addr.real = &gpsdata->gst.smajor_orientation,
+			         .dflt.real = NAN},
+	{"lat",    t_real,    .addr.real = &gpsdata->gst.lat_err_deviation,
+			         .dflt.real = NAN},
+	{"lon",    t_real,    .addr.real = &gpsdata->gst.lon_err_deviation,
+			         .dflt.real = NAN},
+	{"alt",    t_real,    .addr.real = &gpsdata->gst.alt_err_deviation,
+			         .dflt.real = NAN},
+	{NULL},
+	/* *INDENT-ON* */
+    };
+    /*@ +fullinitblock @*/
+
+    status = json_read_object(buf, json_attrs_1, endptr);
+    if (status != 0)
+	return status;
+
+    /*@-usedef@*/
+    if (isnan(gpsdata->fix.time)!=0) {
+	if (tbuf[0] == '\0')
+	    gpsdata->gst.utctime = NAN;
+	else
+	    gpsdata->gst.utctime = iso8601_to_unix(tbuf);
+    }
+    /*@+usedef@*/
+
+    gpsdata->set |= GST_SET;
+    return 0;
+}
+
 static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 			 /*@null@*/ const char **endptr)
 {
     bool usedflags[MAXCHANNELS];
+    char tbuf[JSON_DATE_MAX+1];
     /*@ -fullinitblock @*/
     const struct json_attr_t json_attrs_2_1[] = {
 	/* *INDENT-OFF* */
@@ -137,8 +201,10 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 	                             .len = sizeof(gpsdata->dev.path)},
 	{"tag",	       t_string,  .addr.string  = gpsdata->tag,
 	                             .len = sizeof(gpsdata->tag)},
-	{"time",       t_real,    .addr.real    = &gpsdata->fix.time,
-	                             .nodefault = true},
+	{"time",       t_string,  .addr.string = tbuf,
+			             .len = sizeof(tbuf)},
+	{"time",       t_real,    .addr.real = &gpsdata->fix.time,
+	      	                     .dflt.real = NAN},
 	{"hdop",       t_real,    .addr.real    = &gpsdata->dop.hdop,
 	                             .dflt.real = NAN},
 	{"xdop",       t_real,    .addr.real    = &gpsdata->dop.xdop,
@@ -163,15 +229,29 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
     /*@ +fullinitblock @*/
     int status, i, j;
 
-    for (i = 0; i < MAXCHANNELS; i++)
+    for (i = 0; i < MAXCHANNELS; i++) {
+	gpsdata->PRN[i] = 0;
 	usedflags[i] = false;
+    }
 
     status = json_read_object(buf, json_attrs_2, endptr);
     if (status != 0)
 	return status;
 
+    /*@-usedef@*/
+    if (isnan(gpsdata->fix.time)!=0) {
+	if (tbuf[0] == '\0')
+	    gpsdata->skyview_time = NAN;
+	else
+	    gpsdata->skyview_time = iso8601_to_unix(tbuf);
+    }
+    /*@+usedef@*/
     gpsdata->satellites_used = 0;
+    gpsdata->satellites_visible = 0;
+    (void)memset(gpsdata->used, '\0', sizeof(gpsdata->used));
     for (i = j = 0; i < MAXCHANNELS; i++) {
+	if(gpsdata->PRN[i] > 0)
+	    gpsdata->satellites_visible++;
 	if (usedflags[i]) {
 	    gpsdata->used[j++] = gpsdata->PRN[i];
 	    gpsdata->satellites_used++;
@@ -359,6 +439,8 @@ int libgps_json_unpack(const char *buf,
 #define STARTSWITH(str, prefix)	strncmp(str, prefix, sizeof(prefix)-1)==0
     if (STARTSWITH(classtag, "\"class\":\"TPV\"")) {
 	return json_tpv_read(buf, gpsdata, end);
+    } else if (STARTSWITH(classtag, "\"class\":\"NOISE\"")) {
+	return json_noise_read(buf, gpsdata, end);
     } else if (STARTSWITH(classtag, "\"class\":\"SKY\"")) {
 	return json_sky_read(buf, gpsdata, end);
     } else if (STARTSWITH(classtag, "\"class\":\"ATT\"")) {
