@@ -7,35 +7,23 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <ctype.h>
+#include <assert.h>
+#include <stdarg.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#include <stdarg.h>
-#include <stdbool.h>
-#include <assert.h>
 
-#include "gpsd_config.h"
-
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif /* HAVE_NCURSES_H */
 #include "gpsd.h"
-
-#include "bits.h"
 #include "gpsmon.h"
 #include "gpsdclient.h"
 
 #ifdef NMEA_ENABLE
 extern const struct gps_type_t nmea;
 
-static WINDOW *cookedwin, *nmeawin, *satwin, *gprmcwin, *gpggawin, *gpgsawin;
-static double last_tick, tick_interval;
+static WINDOW *cookedwin, *nmeawin, *satwin, *gprmcwin, *gpggawin, *gpgsawin, *gpgstwin;
+static timestamp_t last_tick, tick_interval;
 
 /*****************************************************************************
  *
@@ -114,6 +102,23 @@ static bool nmea_initialize(void)
     (void)mvwprintw(gpggawin, 7, 1, "Geoid: ");
     (void)mvwprintw(gpggawin, 8, 12, " GGA ");
     (void)wattrset(gpggawin, A_NORMAL);
+
+    gpgstwin = derwin(devicewin, 6, 30, 15, 50);
+    (void)wborder(gpgstwin, 0, 0, 0, 0, 0, 0, 0, 0),
+	(void)syncok(gpgstwin, true);
+    (void)wattrset(gpgstwin, A_BOLD);
+    (void)mvwprintw(gpgstwin, 1,  1, "UTC: ");
+    (void)mvwprintw(gpgstwin, 1, 16, "RMS: ");
+    (void)mvwprintw(gpgstwin, 2,  1, "MAJ: ");
+    (void)mvwprintw(gpgstwin, 2, 16, "MIN: ");
+    (void)mvwprintw(gpgstwin, 3,  1, "ORI: ");
+    (void)mvwprintw(gpgstwin, 3, 16, "LAT: ");
+    (void)mvwprintw(gpgstwin, 4,  1, "LON: ");
+    (void)mvwprintw(gpgstwin, 4, 16, "ALT: ");
+    (void)mvwprintw(gpgstwin, 5, 12, " GST ");
+    (void)wattrset(gpgstwin, A_NORMAL);
+
+
     /*@ +onlytrans @*/
 
     last_tick = timestamp();
@@ -165,12 +170,13 @@ static void nmea_update(void)
     assert(gpgsawin != NULL);
     assert(gpggawin != NULL);
     assert(gprmcwin != NULL);
+    assert(gpgstwin != NULL);
 
     fields = session.driver.nmea.field;
 
     if (session.packet.outbuffer[0] == (unsigned char)'$') {
 	int ymax, xmax;
-	double now;
+	timestamp_t now;
 	getmaxyx(nmeawin, ymax, xmax);
 	if (strstr(sentences, fields[0]) == NULL) {
 	    char *s_end = sentences + strlen(sentences);
@@ -283,6 +289,16 @@ static void nmea_update(void)
 	    (void)mvwprintw(gpggawin, 6, 12, "%-5.5s", fields[8]);
 	    (void)mvwprintw(gpggawin, 7, 12, "%-5.5s", fields[11]);
 	}
+	if (strcmp(fields[0], "GPGST") == 0) {
+	    (void)mvwprintw(gpgstwin, 1,  6, "%-10s", fields[1]);
+	    (void)mvwprintw(gpgstwin, 1, 21,  "%-8s", fields[2]);
+	    (void)mvwprintw(gpgstwin, 2,  6, "%-10s", fields[3]);
+	    (void)mvwprintw(gpgstwin, 2, 21,  "%-8s", fields[4]);
+	    (void)mvwprintw(gpgstwin, 3,  6, "%-10s", fields[5]);
+	    (void)mvwprintw(gpgstwin, 3, 21,  "%-8s", fields[6]);
+	    (void)mvwprintw(gpgstwin, 4,  6, "%-10s", fields[7]);
+	    (void)mvwprintw(gpgstwin, 4, 21,  "%-8s", fields[8]);
+	}
     }
 }
 
@@ -313,7 +329,7 @@ const struct monitor_object_t nmea_mmt = {
  *
  *****************************************************************************/
 
-#ifdef ALLOW_CONTROLSEND
+#if defined(ALLOW_CONTROLSEND) && defined(ASHTECH_ENABLE)
 static void monitor_nmea_send(const char *fmt, ...)
 {
     char buf[BUFSIZ];
@@ -324,7 +340,7 @@ static void monitor_nmea_send(const char *fmt, ...)
     va_end(ap);
     (void)monitor_control_send((unsigned char *)buf, strlen(buf));
 }
-#endif /* ALLOW_CONTROLSEND */
+#endif /* defined(ALLOW_CONTROLSEND) && defined(ASHTECH_ENABLE) */
 
 /*
  * Yes, it's OK for most of these to be clones of the generic NMEA monitor
