@@ -5,27 +5,15 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  *
  */
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <math.h>
-#include <ctype.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <stdarg.h>
-#include <stdbool.h>
 #include <assert.h>
+#include <time.h>
+#include <sys/time.h>
 
-#include "gpsd_config.h"
-
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif /* HAVE_NCURSES_H */
 #include "gpsd.h"
-
 #include "bits.h"
 #include "gpsmon.h"
 
@@ -34,8 +22,9 @@ extern const struct gps_type_t sirf_binary;
 
 static WINDOW *mid2win, *mid4win, *mid6win, *mid7win, *mid9win, *mid13win;
 static WINDOW *mid19win, *mid27win;
-static bool dispmode = false, subframe_enabled = false;
+static bool dispmode = false, subframe_enabled = false, ppstime_enabled = false;
 static int nfix, fix[20];
+static int leapseconds;
 
 /*@ -nullassign @*/
 static char *verbpat[] = {
@@ -77,14 +66,14 @@ static bool sirf_initialize(void)
     unsigned int i;
 
     /*@ -onlytrans @*/
-    mid2win = subwin(devicewin, 7, 80, 1, 0);
-    mid4win = subwin(devicewin, MAXSATS + 3, 30, 8, 0);
-    mid6win = subwin(devicewin, 3, 50, 8, 30);
-    mid7win = subwin(devicewin, 4, 50, 11, 30);
-    mid9win = subwin(devicewin, 3, 50, 15, 30);
-    mid13win = subwin(devicewin, 3, 50, 18, 30);
-    mid19win = newwin(16, 50, 8, 30);
-    mid27win = subwin(devicewin, 3, 50, 21, 30);
+    mid2win = subwin(devicewin, 6, 80, 1, 0);
+    mid4win = subwin(devicewin, MAXSATS + 3, 30, 7, 0);
+    mid6win = subwin(devicewin, 3, 50, 7, 30);
+    mid7win = subwin(devicewin, 4, 50, 13, 30);
+    mid9win = subwin(devicewin, 3, 50, 10, 30);
+    mid13win = subwin(devicewin, 3, 50, 17, 30);
+    mid19win = newwin(16, 50, 7, 30);
+    mid27win = subwin(devicewin, 3, 50, 20, 30);
     if (mid2win == NULL || mid4win == NULL || mid6win == NULL
 	|| mid9win == NULL || mid13win == NULL || mid19win == NULL
 	|| mid27win == NULL)
@@ -117,13 +106,11 @@ static bool sirf_initialize(void)
 		  "Vel:                            m/s                                  climb m/s");
     (void)wmove(mid2win, 3, 1);
     (void)wprintw(mid2win,
-		  "Week+TOW:               Day:                Heading:                 speed m/s");
+		  "Time:                          Leap:        Heading:                 speed m/s");
     (void)wmove(mid2win, 4, 1);
     (void)wprintw(mid2win,
-		  "Skew:                   TZ:                HDOP:      M1:        M2:    ");
-    (void)wmove(mid2win, 5, 1);
-    (void)wprintw(mid2win, "Fix:");
-    display(mid2win, 6, 24, " Packet type 2 (0x02) ");
+		  "Fix:                                                 HDOP:      M1:     M2:   ");
+    display(mid2win, 5, 24, " Packet type 2 (0x02) ");
     (void)wattrset(mid2win, A_NORMAL);
 
     (void)wborder(mid4win, 0, 0, 0, 0, 0, 0, 0, 0),
@@ -132,6 +119,7 @@ static bool sirf_initialize(void)
     for (i = 0; i < MAXSATS; i++) {
 	display(mid4win, (int)(i + 2), 1, "%2d", i);
     }
+    display(mid4win, 0, 1, " Measured Tracker ");
     display(mid4win, 14, 4, " Packet Type 4 (0x04) ");
     (void)wattrset(mid4win, A_NORMAL);
 
@@ -171,12 +159,13 @@ static bool sirf_initialize(void)
 
     (void)wborder(mid6win, 0, 0, 0, 0, 0, 0, 0, 0),
 	(void)wattrset(mid6win, A_BOLD);
-    display(mid6win, 1, 1, "Version:");
+    display(mid6win, 0, 1, " Firmware Version ");
     display(mid6win, 2, 8, " Packet Type 6 (0x06) ");
     (void)wattrset(mid6win, A_NORMAL);
 
     (void)wborder(mid7win, 0, 0, 0, 0, 0, 0, 0, 0),
 	(void)wattrset(mid7win, A_BOLD);
+    display(mid7win, 0, 1, " Clock Status ");
     display(mid7win, 1, 1, "SVs: ");
     display(mid7win, 1, 9, "Drift: ");
     display(mid7win, 1, 23, "Bias: ");
@@ -186,6 +175,7 @@ static bool sirf_initialize(void)
 
     (void)wborder(mid9win, 0, 0, 0, 0, 0, 0, 0, 0),
 	(void)wattrset(mid9win, A_BOLD);
+    display(mid9win, 0, 1, " CPU Throughput ");
     display(mid9win, 1, 1, "Max: ");
     display(mid9win, 1, 13, "Lat: ");
     display(mid9win, 1, 25, "Time: ");
@@ -195,15 +185,13 @@ static bool sirf_initialize(void)
 
     (void)wborder(mid13win, 0, 0, 0, 0, 0, 0, 0, 0),
 	(void)wattrset(mid13win, A_BOLD);
-    display(mid13win, 1, 1, "SVs: ");
-    display(mid13win, 1, 9, "=");
+    display(mid13win, 0, 1, " Visible List ");
     display(mid13win, 2, 8, " Packet type 13 (0x0D) ");
     (void)wattrset(mid13win, A_NORMAL);
 
     (void)wborder(mid27win, 0, 0, 0, 0, 0, 0, 0, 0),
 	(void)wattrset(mid27win, A_BOLD);
-    display(mid27win, 1, 1, "DGPS source: ");
-    display(mid27win, 1, 31, "Corrections: ");
+    display(mid27win, 0, 1, " DGPS Status ");
     display(mid27win, 2, 8, " Packet type 27 (0x1B) ");
     (void)wattrset(mid27win, A_NORMAL);
     /*@ +nullpass @*/
@@ -216,32 +204,12 @@ static bool sirf_initialize(void)
     /*@ +compdef @*/
 #endif /* ALLOW_CONTROLSEND */
 
+    /* initialize the GPS context's time fields */
+    gpsd_time_init(session.context, time(NULL));
+
     return true;
 }
 
-static void decode_time(int week, int tow)
-{
-    int day = tow / 8640000;
-    int tod = tow % 8640000;
-    int h = tod / 360000;
-    int m = tod % 360000;
-    int s = m % 6000;
-
-    m = (m - s) / 6000;
-
-    (void)wmove(mid2win, 3, 10);
-    (void)wprintw(mid2win, "%4d+%9.2f", week, (double)tow / 100);
-    (void)wmove(mid2win, 3, 30);
-    (void)wprintw(mid2win, "%d %02d:%02d:%05.2f", day, h, m, (double)s / 100);
-    (void)wmove(mid2win, 4, 8);
-    (void)wattrset(mid2win, A_UNDERLINE);
-    (void)wprintw(mid2win, "%f",
-		  timestamp() - gpstime_to_unix(week, tow / 100.0));
-    (void)wmove(mid2win, 4, 29);
-    (void)wprintw(mid2win, "%d", gmt_offset);
-    (void)wattrset(mid2win, A_NORMAL);
-}
-
 static void decode_ecef(double x, double y, double z,
 			double vx, double vy, double vz)
 {
@@ -275,25 +243,24 @@ static void decode_ecef(double x, double y, double z,
     if (heading < 0)
 	heading += 2 * GPS_PI;
 
+    /* North and East position fields */
     (void)wattrset(mid2win, A_UNDERLINE);
     (void)wmove(mid2win, 1, 40);
-    (void)wprintw(mid2win, "%9.5f %9.5f", (double)(RAD_2_DEG * phi),
-		  (double)(RAD_2_DEG * lambda));
+    (void)wprintw(mid2win, "%9.5f %9.5f %9d",
+			(double)(RAD_2_DEG * phi),
+			(double)(RAD_2_DEG * lambda),
+			(int)h);
     (void)mvwaddch(mid2win, 1, 49, ACS_DEGREE);
     (void)mvwaddch(mid2win, 1, 59, ACS_DEGREE);
-    (void)wmove(mid2win, 1, 61);
-    (void)wprintw(mid2win, "%8d", (int)h);
 
+    /* North and East velocity fields */
     (void)wmove(mid2win, 2, 40);
-    (void)wprintw(mid2win, "%9.1f %9.1f", vnorth, veast);
-    (void)wmove(mid2win, 2, 61);
-    (void)wprintw(mid2win, "%8.1f", vup);
+    (void)wprintw(mid2win, "%9.1f %9.1f %9.1f", vnorth, veast, vup);
 
+    /* heading and speed fields */
     (void)wmove(mid2win, 3, 54);
-    (void)wprintw(mid2win, "%5.1f", (double)(RAD_2_DEG * heading));
+    (void)wprintw(mid2win, "%5.1f %9.1f", (double)(RAD_2_DEG * heading), speed);
     (void)mvwaddch(mid2win, 3, 59, ACS_DEGREE);
-    (void)wmove(mid2win, 3, 61);
-    (void)wprintw(mid2win, "%8.1f", speed);
     (void)wattrset(mid2win, A_NORMAL);
 }
 
@@ -304,6 +271,7 @@ static void sirf_update(void)
     unsigned char *buf;
     size_t len;
     uint8_t dgps;
+    char tbuf[128];
 
     assert(mid27win != NULL);
     buf = session.packet.outbuffer + 4;
@@ -311,26 +279,37 @@ static void sirf_update(void)
     switch (buf[0]) {
     case 0x02:			/* Measured Navigation Data */
 	(void)wmove(mid2win, 1, 6);	/* ECEF position */
-	(void)wprintw(mid2win, "%8d %8d %8d", getbesl(buf, 1),
-		      getbesl(buf, 5), getbesl(buf, 9));
+	(void)wprintw(mid2win, "%8d %8d %8d", getbes32(buf, 1),
+		      getbes32(buf, 5), getbes32(buf, 9));
 	(void)wmove(mid2win, 2, 6);	/* ECEF velocity */
 	(void)wprintw(mid2win, "%8.1f %8.1f %8.1f",
-		      (double)getbesw(buf, 13) / 8, (double)getbesw(buf,
+		      (double)getbes16(buf, 13) / 8, (double)getbes16(buf,
 								    15) / 8,
-		      (double)getbesw(buf, 17) / 8);
-	decode_ecef((double)getbesl(buf, 1), (double)getbesl(buf, 5),
-		    (double)getbesl(buf, 9), (double)getbesw(buf, 13) / 8,
-		    (double)getbesw(buf, 15) / 8, (double)getbesw(buf,
+		      (double)getbes16(buf, 17) / 8);
+	decode_ecef((double)getbes32(buf, 1), (double)getbes32(buf, 5),
+		    (double)getbes32(buf, 9), (double)getbes16(buf, 13) / 8,
+		    (double)getbes16(buf, 15) / 8, (double)getbes16(buf,
 								  17) / 8);
-	decode_time((int)getbeuw(buf, 22), getbesl(buf, 24));
+	/* line 3 */
+	(void)wmove(mid2win, 3, 7);
+	(void)wprintw(mid2win, "%-22s",
+			unix_to_iso8601(session.gpsdata.fix.time, tbuf, sizeof(tbuf))
+			);
+	(void)wmove(mid2win, 3, 38);
+	(void)wattrset(mid2win, A_UNDERLINE);
+	if (ppstime_enabled)
+	   (void)wprintw(mid2win, "%02d", leapseconds);
+	else
+	   (void)wprintw(mid2win, "??");
+	(void)wattrset(mid2win, A_NORMAL);
 	/* line 4 */
-	(void)wmove(mid2win, 4, 49);
+	(void)wmove(mid2win, 4, 59);
 	(void)wprintw(mid2win, "%4.1f", (double)getub(buf, 20) / 5);	/* HDOP */
-	(void)wmove(mid2win, 4, 58);
+	(void)wmove(mid2win, 4, 69);
 	(void)wprintw(mid2win, "%02x", getub(buf, 19));	/* Mode 1 */
-	(void)wmove(mid2win, 4, 70);
+	(void)wmove(mid2win, 4, 77);
 	(void)wprintw(mid2win, "%02x", getub(buf, 21));	/* Mode 2 */
-	(void)wmove(mid2win, 5, 7);
+	(void)wmove(mid2win, 4, 7);
 	nfix = (int)getub(buf, 28);
 	(void)wprintw(mid2win, "%d = ", nfix);	/* SVs in fix */
 	for (i = 0; i < MAXSATS; i++) {	/* SV list */
@@ -344,7 +323,6 @@ static void sirf_update(void)
 	break;
 
     case 0x04:			/* Measured Tracking Data */
-	decode_time((int)getbeuw(buf, 1), getbesl(buf, 3));
 	ch = (int)getub(buf, 7);
 	for (i = 0; i < ch; i++) {
 	    int sv, st;
@@ -358,10 +336,10 @@ static void sirf_update(void)
 			  ((int)getub(buf, off + 1) * 3) / 2, (int)getub(buf,
 									 off +
 									 2) /
-			  2, (int)getbesw(buf, off + 3));
+			  2, (int)getbes16(buf, off + 3));
 
 	    st = ' ';
-	    if ((int)getbeuw(buf, off + 3) == 0xbf)
+	    if ((int)getbeu16(buf, off + 3) == 0xbf)
 		st = 'T';
 	    for (j = 0; j < nfix; j++)
 		if (sv == fix[j]) {
@@ -385,7 +363,7 @@ static void sirf_update(void)
 #ifdef __UNUSED__
     case 0x05:			/* raw track data */
 	for (off = 1; off < len; off += 51) {
-	    ch = getbeul(buf, off);
+	    ch = getbeu32(buf, off);
 	    (void)wmove(mid4win, ch + 2, 19);
 	    cn = 0;
 
@@ -394,41 +372,40 @@ static void sirf_update(void)
 
 	    printw("%5.1f", (double)cn / 10);
 
-	    printw("%9d%3d%5d", getbeul(buf, off + 8),
-		   (int)getbeuw(buf, off + 12), (int)getbeuw(buf, off + 14));
-	    printw("%8.5f %10.5f", (double)getbeul(buf, off + 16) / 65536,
-		   (double)getbeul(buf, off + 20) / 1024);
+	    printw("%9d%3d%5d", getbeu32(buf, off + 8),
+		   (int)getbeu16(buf, off + 12), (int)getbeu16(buf, off + 14));
+	    printw("%8.5f %10.5f", (double)getbeu32(buf, off + 16) / 65536,
+		   (double)getbeu32(buf, off + 20) / 1024);
 	}
 	monitor_log("RTD 0x05=");
 	break;
 #endif /* __UNUSED */
 
     case 0x06:			/* firmware version */
-	display(mid6win, 1, 10, "%s", buf + 1);
+	display(mid6win, 1, 1, "%s", buf + 1);
 	monitor_log("FV  0x06=");
 	break;
 
     case 0x07:			/* Response - Clock Status Data */
-	decode_time((int)getbeuw(buf, 1), getbesl(buf, 3));
 	display(mid7win, 1, 5, "%2d", getub(buf, 7));	/* SVs */
-	display(mid7win, 1, 16, "%lu", getbeul(buf, 8));	/* Clock drift */
-	display(mid7win, 1, 29, "%lu", getbeul(buf, 12));	/* Clock Bias */
-	display(mid7win, 2, 21, "%lu", getbeul(buf, 16));	/* Estimated Time */
+	display(mid7win, 1, 16, "%lu", getbeu32(buf, 8));	/* Clock drift */
+	display(mid7win, 1, 29, "%lu", getbeu32(buf, 12));	/* Clock Bias */
+	display(mid7win, 2, 21, "%lu", getbeu32(buf, 16));	/* Estimated Time */
 	monitor_log("CSD 0x07=");
 	break;
 
     case 0x08:			/* 50 BPS data */
 	ch = (int)getub(buf, 1);
 	display(mid4win, ch + 2, 27, "Y");
-	monitor_log("50B 0x08=");
 	subframe_enabled = true;
+	monitor_log("50B 0x08=");
 	break;
 
     case 0x09:			/* Throughput */
-	display(mid9win, 1, 6, "%.3f", (double)getbeuw(buf, 1) / 186);	/*SegStatMax */
-	display(mid9win, 1, 18, "%.3f", (double)getbeuw(buf, 3) / 186);	/*SegStatLat */
-	display(mid9win, 1, 31, "%.3f", (double)getbeuw(buf, 5) / 186);	/*SegStatTime */
-	display(mid9win, 1, 42, "%3d", (int)getbeuw(buf, 7));	/* Last Millisecond */
+	display(mid9win, 1, 6, "%.3f", (double)getbeu16(buf, 1) / 186);	/*SegStatMax */
+	display(mid9win, 1, 18, "%.3f", (double)getbeu16(buf, 3) / 186);	/*SegStatLat */
+	display(mid9win, 1, 31, "%.3f", (double)getbeu16(buf, 5) / 186);	/*SegStatTime */
+	display(mid9win, 1, 42, "%3d", (int)getbeu16(buf, 7));	/* Last Millisecond */
 	monitor_log("THR 0x09=");
 	break;
 
@@ -441,14 +418,13 @@ static void sirf_update(void)
 	break;
 
     case 0x0d:			/* Visible List */
-	display(mid13win, 1, 6, "%02d", getub(buf, 1));
-	(void)wmove(mid13win, 1, 10);
-	for (i = 0; i < MAXSATS; i++) {
-	    if (i < (int)getub(buf, 1))
-		(void)wprintw(mid13win, " %2d", getub(buf, 2 + 5 * i));
-	    else
-		(void)wprintw(mid13win, "   ");
-	}
+	display(mid13win, 1, 1, "%02d =                                            ",
+				 getub(buf, 1));
+	(void)wmove(mid13win, 1, 5);
+	for (i = 0; i < MAXSATS; i++)
+	    (void)wprintw(mid13win, " %2d", getub(buf, 2 + 5 * i));
+	if (MAXSATS < (int)getub(buf, 1))
+	    (void)wprintw(mid13win, " ...");
 	monitor_log("VL  0x0d=");
 	break;
 
@@ -456,7 +432,7 @@ static void sirf_update(void)
 #define YESNO(n)	(((int)getub(buf, n) != 0)?'Y':'N')
 	display(mid19win, 1, 20, "%d", getub(buf, 5));	/* Alt. hold mode */
 	display(mid19win, 2, 20, "%d", getub(buf, 6));	/* Alt. hold source */
-	display(mid19win, 3, 20, "%dm", (int)getbeuw(buf, 7));	/* Alt. source input */
+	display(mid19win, 3, 20, "%dm", (int)getbeu16(buf, 7));	/* Alt. source input */
 	if (getub(buf, 9) != (uint8_t) '\0')
 	    display(mid19win, 4, 20, "%dsec", getub(buf, 10));	/* Degraded timeout */
 	else
@@ -466,26 +442,27 @@ static void sirf_update(void)
 	display(mid19win, 7, 20, "%c", YESNO(13));	/* Static Nav. */
 	display(mid19win, 8, 20, "0x%x", getub(buf, 14));	/* 3SV Least Squares */
 	display(mid19win, 9, 20, "0x%x", getub(buf, 19));	/* DOP Mask mode */
-	display(mid19win, 10, 20, "0x%x", (int)getbeuw(buf, 20));	/* Nav. Elev. mask */
+	display(mid19win, 10, 20, "0x%x", (int)getbeu16(buf, 20));	/* Nav. Elev. mask */
 	display(mid19win, 11, 20, "0x%x", getub(buf, 22));	/* Nav. Power mask */
 	display(mid19win, 12, 20, "0x%x", getub(buf, 27));	/* DGPS Source */
 	display(mid19win, 13, 20, "0x%x", getub(buf, 28));	/* DGPS Mode */
 	display(mid19win, 14, 20, "%dsec", getub(buf, 29));	/* DGPS Timeout */
 	display(mid19win, 1, 42, "%c", YESNO(34));	/* LP Push-to-Fix */
-	display(mid19win, 2, 42, "%dms", getbeul(buf, 35));	/* LP On Time */
-	display(mid19win, 3, 42, "%d", getbeul(buf, 39));	/* LP Interval */
+	display(mid19win, 2, 42, "%dms", getbeu32(buf, 35));	/* LP On Time */
+	display(mid19win, 3, 42, "%d", getbeu32(buf, 39));	/* LP Interval */
 	display(mid19win, 4, 42, "%c", YESNO(43));	/* User Tasks enabled */
-	display(mid19win, 5, 42, "%d", getbeul(buf, 44));	/* User Task Interval */
+	display(mid19win, 5, 42, "%d", getbeu32(buf, 44));	/* User Task Interval */
 	display(mid19win, 6, 42, "%c", YESNO(48));	/* LP Power Cycling Enabled */
-	display(mid19win, 7, 42, "%d", getbeul(buf, 49));	/* LP Max Acq Search Time */
-	display(mid19win, 8, 42, "%d", getbeul(buf, 53));	/* LP Max Off Time */
+	display(mid19win, 7, 42, "%d", getbeu32(buf, 49));	/* LP Max Acq Search Time */
+	display(mid19win, 8, 42, "%d", getbeu32(buf, 53));	/* LP Max Off Time */
 	display(mid19win, 9, 42, "%c", YESNO(57));	/* APM Enabled */
-	display(mid19win, 10, 42, "%d", (int)getbeuw(buf, 58));	/* # of fixes */
-	display(mid19win, 11, 42, "%d", (int)getbeuw(buf, 60));	/* Time Between fixes */
+	display(mid19win, 10, 42, "%d", (int)getbeu16(buf, 58));	/* # of fixes */
+	display(mid19win, 11, 42, "%d", (int)getbeu16(buf, 60));	/* Time Between fixes */
 	display(mid19win, 12, 42, "%d", getub(buf, 62));	/* H/V Error Max */
 	display(mid19win, 13, 42, "%d", getub(buf, 63));	/* Response Time Max */
 	display(mid19win, 14, 42, "%d", getub(buf, 64));	/* Time/Accu & Duty Cycle Priority */
 #undef YESNO
+	monitor_log("NP  0x13=");
 	break;
 
     case 0x1b:
@@ -531,34 +508,36 @@ static void sirf_update(void)
 	total               3 x 12 = 36 bytes
 	******************************************************************/
 	dgps = getub(buf, 1);
-	display(mid27win, 1, 14, "%d (%s)",
-		dgps, (CHECK_RANGE(dgpsvec, dgps) ? dgpsvec[dgps] : "???"));
 	/*@ -type @*/
-	//(void) wmove(mid27win, 2, 0);
-	for (i = j = 0; i < 12; i++) {
-	    if (getub(buf, 16 + 3 * i) != '\0') {
-		//(void)wprintw(mid27win, " %d=%d", getub(buf, 16+3*i), getbesw(buf, 16+3*i+1));
-		j++;
-	    }
-	}
+	display(mid27win, 1, 1, "%8s =                                      ",
+		(CHECK_RANGE(dgpsvec, dgps) ? dgpsvec[dgps] : "???"));
+	(void)wmove(mid27win, 1, 12);
+	for (i = 0; i < MAXSATS; i++)
+	    if (getub(buf, 16 + 3 * i) != '\0')
+		(void)wprintw(mid27win, " %2d", getub(buf, 16+3*i));
 	/*@ +type @*/
-	display(mid27win, 1, 44, "%d", j);
 	monitor_log("DST 0x1b=");
 	break;
 
-    case 0x1C:			/* NL Measurement Data */
-    case 0x1D:			/* DGPS Data */
-    case 0x1E:			/* SV State Data */
-    case 0x1F:			/* NL Initialized Data */
+    case 0x1c:			/* NL Measurement Data */
+    case 0x1d:			/* NL DGPS Data */
+    case 0x1e:			/* NL SV State Data */
+    case 0x1f:			/* NL Initialized Data */
 	subframe_enabled = true;
+	monitor_log("NL  0x%02x=", buf[0]);
 	break;
+
     case 0x29:			/* Geodetic Navigation Message */
 	monitor_log("GNM 0x29=");
 	break;
+
     case 0x32:			/* SBAS Parameters */
 	monitor_log("SBP 0x32=");
 	break;
+
     case 0x34:			/* PPS Time */
+	ppstime_enabled = true;
+	leapseconds = (int)getbeu16(buf, 8);
 	monitor_log("PPS 0x34=");
 	break;
 
@@ -566,18 +545,18 @@ static void sirf_update(void)
     case 0x62:
 	attrset(A_BOLD);
 	move(2, 40);
-	printw("%9.5f %9.5f", (double)(RAD_2_DEG * 1e8 * getbesl(buf, 1)),
-	       (double)(RAD_2_DEG * 1e8 * getbesl(buf, 5)));
+	printw("%9.5f %9.5f", (double)(RAD_2_DEG * 1e8 * getbes32(buf, 1)),
+	       (double)(RAD_2_DEG * 1e8 * getbes32(buf, 5)));
 	move(2, 63);
-	printw("%8d", getbesl(buf, 9) / 1000);
+	printw("%8d", getbes32(buf, 9) / 1000);
 
 	move(3, 63);
 
-	printw("%8.1f", (double)getbesl(buf, 17) / 1000);
+	printw("%8.1f", (double)getbes32(buf, 17) / 1000);
 
 	move(4, 54);
-	if (getbeul(buf, 13) > 50) {
-	    double heading = RAD_2_DEG * 1e8 * getbesl(buf, 21);
+	if (getbeu32(buf, 13) > 50) {
+	    double heading = RAD_2_DEG * 1e8 * getbes32(buf, 21);
 	    if (heading < 0)
 		heading += 360;
 	    printw("%5.1f", heading);
@@ -585,15 +564,15 @@ static void sirf_update(void)
 	    printw("  0.0");
 
 	move(4, 63);
-	printw("%8.1f", (double)getbesl(buf, 13) / 1000);
+	printw("%8.1f", (double)getbes32(buf, 13) / 1000);
 	attrset(A_NORMAL);
 
 	move(5, 13);
 	printw("%04d-%02d-%02d %02d:%02d:%02d.%02d",
-	       (int)getbeuw(buf, 26), getub(buf, 28), getub(buf, 29),
-	       getub(buf, 30), getub(buf, 31), (unsigned short)getbeuw(buf,
+	       (int)getbeu16(buf, 26), getub(buf, 28), getub(buf, 29),
+	       getub(buf, 30), getub(buf, 31), (unsigned short)getbeu16(buf,
 								       32) /
-	       1000, ((unsigned short)getbeuw(buf, 32) % 1000) / 10);
+	       1000, ((unsigned short)getbeu16(buf, 32) % 1000) / 10);
 	{
 	    struct timeval clk, gps;
 	    struct tm tm;
@@ -601,16 +580,16 @@ static void sirf_update(void)
 	    gettimeofday(&clk, NULL);
 
 	    memset(&tm, 0, sizeof(tm));
-	    tm.tm_sec = (unsigned short)getbeuw(buf, 32) / 1000;
+	    tm.tm_sec = (unsigned short)getbeu16(buf, 32) / 1000;
 	    tm.tm_min = (int)getub(buf, 31);
 	    tm.tm_hour = (int)getub(buf, 30);
 	    tm.tm_mday = (int)getub(buf, 29);
 	    tm.tm_mon = (int)getub(buf, 28) - 1;
-	    tm.tm_year = (int)getbeuw(buf, 26) - 1900;
+	    tm.tm_year = (int)getbeu16(buf, 26) - 1900;
 
 	    gps.tv_sec = mkgmtime(&tm);
 	    gps.tv_usec =
-		(((unsigned short)getbeuw(buf, 32) % 1000) / 10) * 10000;
+		(((unsigned short)getbeu16(buf, 32) % 1000) / 10) * 10000;
 
 	    move(5, 2);
 	    printw("           ");
@@ -648,7 +627,7 @@ static void sirf_update(void)
 	break;
 
     default:
-	monitor_log("    0x%02x=", buf[4]);
+	monitor_log("UNK 0x%02x=", buf[0]);
 	break;
     }
 
@@ -679,9 +658,14 @@ static int sirf_command(char line[])
     case 'A':			/* toggle 50bps subframe data */
 	(void)memset(buf, '\0', sizeof(buf));
 	putbyte(buf, 0, 0x80);
-	putbyte(buf, 23, 12);
+	putbyte(buf, 23, 0x0c);
 	putbyte(buf, 24, subframe_enabled ? 0x00 : 0x10);
 	(void)monitor_control_send(buf, 25);
+	/*
+	 * The subframe_enabled off because we're counting on receipt of
+	 * a subframe packet to turn it on if the command succeeds.
+	 */
+	subframe_enabled = false;
 	return COMMAND_MATCH;
 
     case 'M':			/* static navigation */
@@ -707,6 +691,24 @@ static int sirf_command(char line[])
 
     case 'P':			/* poll navigation params */
 	dispmode = !dispmode;
+	if (dispmode) {
+	    (void)syncok(mid6win, false);
+	    (void)syncok(mid7win, false);
+	    (void)syncok(mid9win, false);
+	    (void)syncok(mid13win, false);
+	    (void)syncok(mid27win, false);
+	} else {
+	    (void)syncok(mid6win, true);
+	    (void)wsyncup(mid6win);
+	    (void)syncok(mid7win, true);
+	    (void)wsyncup(mid7win);
+	    (void)syncok(mid9win, true);
+	    (void)wsyncup(mid9win);
+	    (void)syncok(mid13win, true);
+	    (void)wsyncup(mid13win);
+	    (void)syncok(mid27win, true);
+	    (void)wsyncup(mid27win);
+	}
 	return COMMAND_MATCH;
     }
 
@@ -735,7 +737,7 @@ const struct monitor_object_t sirf_mmt = {
     .command = NULL,
 #endif /* ALLOW_CONTROLSEND */
     .wrap = sirf_wrap,
-    .min_y = 23,.min_x = 80,
+    .min_y = 22,.min_x = 80,
     .driver = &sirf_binary,
 };
 #endif /* defined(SIRF_ENABLE) && defined(BINARY_ENABLE) */
