@@ -4,26 +4,20 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 
-#include <stdlib.h>
-#include "gpsd_config.h"
-#include <sys/types.h>
-#ifndef S_SPLINT_S
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#include <unistd.h>
-#endif /* S_SPLINT_S */
 #include <string.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#ifndef S_SPLINT_S
+#include <sys/socket.h>
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 
 #define NETGNSS_DGPSIP	"dgpsip://"
 #define NETGNSS_NTRIP	"ntrip://"
 
-/* Where to find the list of DGPSIP correction servers, if there is one */
-#define DGPSIP_SERVER_LIST	"/usr/share/gpsd/dgpsip-servers"
-
 bool netgnss_uri_check(char *name)
 /* is given string a valid URI for GNSS/DGPS service? */
 {
@@ -34,19 +28,21 @@ bool netgnss_uri_check(char *name)
 
 
 /*@ -branchstate */
-int netgnss_uri_open(struct gps_context_t *context, char *netgnss_service)
+int netgnss_uri_open(struct gps_device_t *dev, char *netgnss_service)
 /* open a connection to a DGNSS service */
 {
 #ifdef NTRIP_ENABLE
-    if (strncmp(netgnss_service, NETGNSS_NTRIP, strlen(NETGNSS_NTRIP)) == 0)
-	return ntrip_open(context, netgnss_service + strlen(NETGNSS_NTRIP));
+    if (strncmp(netgnss_service, NETGNSS_NTRIP, strlen(NETGNSS_NTRIP)) == 0) {
+	dev->ntrip.conn_state = ntrip_conn_init;
+	return ntrip_open(dev, netgnss_service + strlen(NETGNSS_NTRIP));
+    }
 #endif
 
     if (strncmp(netgnss_service, NETGNSS_DGPSIP, strlen(NETGNSS_DGPSIP)) == 0)
-	return dgpsip_open(context, netgnss_service + strlen(NETGNSS_DGPSIP));
+	return dgpsip_open(dev, netgnss_service + strlen(NETGNSS_DGPSIP));
 
 #ifndef REQUIRE_DGNSS_PROTO
-    return dgpsip_open(context, netgnss_service);
+    return dgpsip_open(dev, netgnss_service);
 #else
     gpsd_report(LOG_ERROR,
 		"Unknown or unspecified DGNSS protocol for service %s\n",
@@ -57,44 +53,18 @@ int netgnss_uri_open(struct gps_context_t *context, char *netgnss_service)
 
 /*@ +branchstate */
 
-int netgnss_poll(struct gps_context_t *context)
-/* poll the DGNSS service for a correction report */
-{
-    if (context->dsock > -1) {
-	ssize_t rtcmbytes =
-	    read(context->dsock, context->rtcmbuf, sizeof(context->rtcmbuf));
-	if ((rtcmbytes == -1 && errno != EAGAIN) || (rtcmbytes == 0)) {
-	    (void)shutdown(context->dsock, SHUT_RDWR);
-	    (void)close(context->dsock);
-	    context->rtcmbytes = 0;
-	    return -1;
-	} else {
-	    context->rtcmbytes = (size_t) rtcmbytes;
-	    context->rtcmtime = timestamp();
-	}
-    }
-    return 0;
-}
-
-void netgnss_report(struct gps_device_t *session)
+void netgnss_report(struct gps_context_t *context,
+		    struct gps_device_t *gps, struct gps_device_t *dgnss)
 /* may be time to ship a usage report to the DGNSS service */
 {
-    if (session->context->netgnss_service == netgnss_dgpsip)
-	dgpsip_report(session);
+    if (dgnss->servicetype == service_dgpsip)
+	dgpsip_report(context, gps, dgnss);
 #ifdef NTRIP_ENABLE
-    else if (session->context->netgnss_service == netgnss_ntrip)
-	ntrip_report(session);
+    else if (dgnss->servicetype == service_ntrip)
+	ntrip_report(context, gps, dgnss);
 #endif
 }
 
-void netgnss_autoconnect(struct gps_context_t *context, double lat,
-			 double lon)
-{
-    if (context->netgnss_service == netgnss_dgpsip) {
-	dgpsip_autoconnect(context, lat, lon, DGPSIP_SERVER_LIST);
-    }
-}
-
 /* *INDENT-OFF* */
 void rtcm_relay(struct gps_device_t *session)
 /* pass a DGNSS connection report to a session */
