@@ -3,57 +3,25 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <stdlib.h>
-#include "gpsd_config.h"
-#include <sys/types.h>
-#ifndef S_SPLINT_S
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#else
-#define AF_UNSPEC 0
-#endif /* HAVE_SYS_SOCKET_H */
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <sys/time.h>
 #include <stdio.h>
-#include <math.h>
-#ifndef S_SPLINT_S
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#endif /* S_SPLINT_S */
+#include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <math.h>
+#include <strings.h>
+#ifndef S_SPLINT_S
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <netdb.h>
+#include <sys/socket.h>
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #include "bsd-base64.h"
 
-struct ntrip_stream_t
-{
-    char mountpoint[101];
-    enum
-    {
-	fmt_rtcm2,
-	fmt_rtcm2_0,
-	fmt_rtcm2_1,
-	fmt_rtcm2_2,
-	fmt_rtcm2_3,
-	fmt_rtcm3,
-	fmt_unknown
-    } format;
-    int carrier;
-    double latitude;
-    double longitude;
-    int nmea;
-    enum
-    { cmp_enc_none, cmp_enc_unknown } compr_encryp;
-    enum
-    { auth_none, auth_basic, auth_digest, auth_unknown } authentication;
-    int fee;
-    int bitrate;
-};
-
 #define NTRIP_SOURCETABLE	"SOURCETABLE 200 OK\r\n"
 #define NTRIP_ENDSOURCETABLE	"ENDSOURCETABLE"
 #define NTRIP_CAS		"CAS;"
@@ -64,8 +32,6 @@ struct ntrip_stream_t
 #define NTRIP_ICY		"ICY 200 OK"
 #define NTRIP_UNAUTH		"401 Unauthorized"
 
-static struct ntrip_stream_t ntrip_stream;
-
 /*@ -temptrans -mustfreefresh @*/
 static /*@null@*/ char *ntrip_field_iterate( /*@null@ */ char *start,
 					     /*@null@*/ char *prev,
@@ -123,6 +89,9 @@ static void ntrip_str_parse(char *str, size_t len,
 	    hold->format = fmt_rtcm2_2;
 	else if (strcasecmp("RTCM 2.3", s) == 0)
 	    hold->format = fmt_rtcm2_3;
+	/* required for the SAPOS derver in Gemany, confirmed as RTCM2.3 */
+	else if (strcasecmp("RTCM1_", s) == 0)
+	    hold->format = fmt_rtcm2_3;
 	else if (strcasecmp("RTCM 3.0", s) == 0)
 	    hold->format = fmt_rtcm3;
 	else
@@ -185,59 +154,70 @@ static void ntrip_str_parse(char *str, size_t len,
     while ((s = ntrip_field_iterate(NULL, s, eol)));
 }
 
-/*@ +mustfreefresh @*/
-
-static int ntrip_sourcetable_parse(int fd, char *buf, ssize_t blen,
-				   const char *stream,
-				   struct ntrip_stream_t *keep)
+static int ntrip_sourcetable_parse(struct gps_device_t *device)
 {
     struct ntrip_stream_t hold;
     ssize_t llen, len = 0;
     char *line;
-    bool srctbl = false;
+    bool sourcetable = false;
     bool match = false;
+    char buf[BUFSIZ];
+    size_t blen = BUFSIZ;
+    int fd = device->gpsdata.gps_fd;
 
-    for (;;) {
+    while (1) {
 	char *eol;
 	ssize_t rlen;
 
 	memset(&buf[len], 0, (size_t) (blen - len));
 
-	if ((rlen = recv(fd, &buf[len], (size_t) (blen - 1 - len), 0)) == -1) {
-	    if (errno == EINTR)
+	rlen = read(fd, &buf[len], (size_t) (blen - 1 - len));
+	if (rlen == -1) {
+	    if (errno == EINTR) {
 		continue;
+	    }
+	    if (sourcetable && !match && errno == EAGAIN) { // we have not yet found a match, but there currently is no more data
+		return 0;
+	    }
+	    if (match) {
+		return 1;
+	    }
+	    gpsd_report(LOG_ERROR, "ntrip stream read error %d on fd %d\n",
+		    errno, fd);
+	    return -1;
+	} else if (rlen == 0) { // server closed the connection
+	    gpsd_report(LOG_ERROR, "ntrip stream unexpected close %d on fd %d during sourcetable read\n",
+		    errno, fd);
 	    return -1;
 	}
-	if (rlen == 0)
-	    continue;
 
 	line = buf;
 	rlen = len += rlen;
 
 	gpsd_report(LOG_RAW, "Ntrip source table buffer %s\n", buf);
 
-	if (!srctbl) {
+	sourcetable = device->ntrip.sourcetable_parse;
+	if (!sourcetable) {
 	    /* parse SOURCETABLE */
 	    if (strncmp(line, NTRIP_SOURCETABLE, strlen(NTRIP_SOURCETABLE)) ==
-		0) {
-		srctbl = true;
+		    0) {
+		sourcetable = true;
+		device->ntrip.sourcetable_parse = true;
 		llen = (ssize_t) strlen(NTRIP_SOURCETABLE);
 		line += llen;
 		len -= llen;
 	    } else {
 		gpsd_report(LOG_WARN, "Received unexpexted Ntrip reply %s.\n",
-			    buf);
+			buf);
 		return -1;
 	    }
 	}
-	if (!srctbl)
-	    return -1;
 
 	while (len > 0) {
 	    /* parse ENDSOURCETABLE */
 	    if (strncmp
-		(line, NTRIP_ENDSOURCETABLE,
-		 strlen(NTRIP_ENDSOURCETABLE)) == 0)
+		    (line, NTRIP_ENDSOURCETABLE,
+		     strlen(NTRIP_ENDSOURCETABLE)) == 0)
 		goto done;
 
 	    if (!(eol = strstr(line, NTRIP_BR)))
@@ -253,31 +233,41 @@ static int ntrip_sourcetable_parse(int fd, char *buf, ssize_t blen,
 	    /* parse STR */
 	    if (strncmp(line, NTRIP_STR, strlen(NTRIP_STR)) == 0) {
 		ntrip_str_parse(line + strlen(NTRIP_STR),
-				(size_t) (llen - strlen(NTRIP_STR)), &hold);
-		if (stream != NULL && strcmp(stream, hold.mountpoint) == 0) {
+			(size_t) (llen - strlen(NTRIP_STR)), &hold);
+		if (strcmp(device->ntrip.stream.mountpoint, hold.mountpoint) == 0) {
 		    /* todo: support for RTCM 3.0, SBAS (WAAS, EGNOS), ... */
 		    if (hold.format == fmt_unknown) {
 			gpsd_report(LOG_ERROR,
-				    "Ntrip stream %s format not supported\n",
-				    line);
+				"Ntrip stream %s format not supported\n",
+				line);
 			return -1;
 		    }
 		    /* todo: support encryption and compression algorithms */
 		    if (hold.compr_encryp != cmp_enc_none) {
 			gpsd_report(LOG_ERROR,
-				    "Ntrip stream %s compression/encryption algorithm not supported\n",
-				    line);
+				"Ntrip stream %s compression/encryption algorithm not supported\n",
+				line);
 			return -1;
 		    }
 		    /* todo: support digest authentication */
 		    if (hold.authentication != auth_none
-			&& hold.authentication != auth_basic) {
+			    && hold.authentication != auth_basic) {
 			gpsd_report(LOG_ERROR,
-				    "Ntrip stream %s authentication method not supported\n",
-				    line);
+				"Ntrip stream %s authentication method not supported\n",
+				line);
 			return -1;
 		    }
-		    memcpy(keep, &hold, sizeof(hold));
+		    /* no memcpy, so we can keep the other infos */
+		    device->ntrip.stream.format = hold.format;
+		    device->ntrip.stream.carrier = hold.carrier;
+		    device->ntrip.stream.latitude = hold.latitude;
+		    device->ntrip.stream.longitude = hold.longitude;
+		    device->ntrip.stream.nmea = hold.nmea;
+		    device->ntrip.stream.compr_encryp = hold.compr_encryp;
+		    device->ntrip.stream.authentication = hold.authentication;
+		    device->ntrip.stream.fee = hold.fee;
+		    device->ntrip.stream.bitrate = hold.bitrate;
+		    device->ntrip.stream.set = true;
 		    match = true;
 		}
 		/* todo: compare stream location to own location to
@@ -293,52 +283,51 @@ static int ntrip_sourcetable_parse(int fd, char *buf, ssize_t blen,
 	    line += llen;
 	    len -= llen;
 	    gpsd_report(LOG_RAW,
-			"Remaining Ntrip source table buffer %zd %s\n", len,
-			line);
+		    "Remaining Ntrip source table buffer %zd %s\n", len,
+		    line);
 	}
 	/* message too big to fit into buffer */
-	if (len == blen - 1)
+	if ((size_t)len == blen - 1)
 	    return -1;
 
 	if (len > 0)
-	    memcpy(buf, &buf[rlen - len], (size_t) len);
+	    memmove(buf, &buf[rlen - len], (size_t) len);
     }
 
-  done:
-    return match ? 0 : -1;
+done:
+    return match ? 1 : -1;
 }
 
-static int ntrip_stream_probe(const char *caster,
-			      const char *port,
-			      const char *stream, struct ntrip_stream_t *keep)
+static int ntrip_stream_req_probe(const struct ntrip_stream_t *stream)
 {
-    int ret;
-    socket_t dsock;
+    int dsock;
+    ssize_t r;
     char buf[BUFSIZ];
 
-    if ((dsock = netlib_connectsock(AF_UNSPEC, caster, port, "tcp")) == -1) {
-	printf("ntrip stream connect error %d\n", dsock);
+    dsock = netlib_connectsock(AF_UNSPEC, stream->url, stream->port, "tcp");
+    if (dsock < 0) {
+	gpsd_report(LOG_ERROR, "ntrip stream connect error %d in req probe\n", dsock);
 	return -1;
     }
+    gpsd_report(LOG_SPIN, "ntrip stream for req probe connected on fd %d\n", dsock);
     (void)snprintf(buf, sizeof(buf),
-		   "GET / HTTP/1.1\r\n"
-		   "User-Agent: NTRIP gpsd/%s\r\n"
-		   "Connection: close\r\n" "\r\n", VERSION);
-    if (write(dsock, buf, strlen(buf)) != (ssize_t) strlen(buf)) {
-	printf("ntrip stream write error %d\n", dsock);
+	    "GET / HTTP/1.1\r\n"
+	    "User-Agent: NTRIP gpsd/%s\r\n"
+	    "Connection: close\r\n"
+	    "\r\n", VERSION);
+    r = write(dsock, buf, strlen(buf));
+    if (r != (ssize_t)strlen(buf)) {
+	gpsd_report(LOG_ERROR, "ntrip stream write error %d on fd %d during probe request %zd\n",
+		errno, dsock, r);
 	return -1;
     }
-    ret =
-	ntrip_sourcetable_parse(dsock, buf, (ssize_t) sizeof(buf), stream,
-				keep);
-    (void)close(dsock);
-    return ret;
+    return dsock;
 }
 
 static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
-			     const char *auth,
-			     /*@out@*/ char buf[],
-			     size_t size)
+	const char *auth,
+	/*@out@*/ char buf[],
+	size_t size)
 {
     memset(buf, 0, size);
     if (stream->authentication == auth_none)
@@ -349,8 +338,8 @@ static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
 	    return -1;
 	memset(authenc, 0, sizeof(authenc));
 	if (b64_ntop
-	    ((unsigned char *)auth, strlen(auth), authenc,
-	     sizeof(authenc) - 1) < 0)
+		((unsigned char *)auth, strlen(auth), authenc,
+		 sizeof(authenc) - 1) < 0)
 	    return -1;
 	(void)snprintf(buf, size - 1, "Authorization: Basic %s\r\n", authenc);
     } else {
@@ -359,161 +348,220 @@ static int ntrip_auth_encode(const struct ntrip_stream_t *stream,
     return 0;
 }
 
-/* *INDENT-OFF* */
-/*@ -nullpass @*//* work around a splint bug */
-static int ntrip_stream_open(const char *caster, const char *port,
-			     /*@null@*/ const char *auth,
-			     struct gps_context_t *context,
-			     struct ntrip_stream_t *stream)
-{
-    char buf[BUFSIZ];
-    char authstr[128];
-    int opts;
+/* *INDENT-ON* */
 
-    if (ntrip_auth_encode(stream, auth, authstr, sizeof(authstr)) < 0) {
-	gpsd_report(LOG_ERROR, "User-ID and password needed for %s:%s/%s\n",
-		    caster, port, stream->mountpoint);
+static int ntrip_stream_get_req(const struct ntrip_stream_t *stream)
+{
+    int dsock;
+    char buf[BUFSIZ];
+
+    dsock = netlib_connectsock(AF_UNSPEC, stream->url, stream->port, "tcp");
+    if (dsock == -1) {
+	gpsd_report(LOG_ERROR, "ntrip stream connect error %d\n", dsock);
 	return -1;
     }
-    if ((context->dsock =
-	 netlib_connectsock(AF_UNSPEC, caster, port, "tcp")) < 0)
-	return -1;
+
+    gpsd_report(LOG_SPIN, "netlib_connectsock() returns socket on fd %d\n",
+	    dsock);
 
     (void)snprintf(buf, sizeof(buf),
-		   "GET /%s HTTP/1.1\r\n"
-		   "User-Agent: NTRIP gpsd/%s\r\n"
-		   "Accept: rtk/rtcm, dgps/rtcm\r\n"
-		   "%s"
-		   "Connection: close\r\n"
-		   "\r\n", stream->mountpoint, VERSION, authstr);
-    if (write(context->dsock, buf, strlen(buf)) != (ssize_t) strlen(buf)) {
-	printf("ntrip stream write error on %d\n", context->dsock);
+	    "GET /%s HTTP/1.1\r\n"
+	    "User-Agent: NTRIP gpsd/%s\r\n"
+	    "Accept: rtk/rtcm, dgps/rtcm\r\n"
+	    "%s"
+	    "Connection: close\r\n"
+	    "\r\n", stream->mountpoint, VERSION, stream->authStr);
+    if (write(dsock, buf, strlen(buf)) != (ssize_t) strlen(buf)) {
+	gpsd_report(LOG_ERROR, "ntrip stream write error %d on fd %d during get request\n", errno,
+		dsock);
+	(void)close(dsock);
 	return -1;
     }
+    return dsock;
+}
 
+static int ntrip_stream_get_parse(const struct ntrip_stream_t *stream, int dsock)
+{
+/*@-nullpass@*/
+    char buf[BUFSIZ];
+    int opts;
     memset(buf, 0, sizeof(buf));
-    if (read(context->dsock, buf, sizeof(buf) - 1) == -1)
+    while (read(dsock, buf, sizeof(buf) - 1) == -1) {
+	if (errno == EINTR)
+	    continue;
+	gpsd_report(LOG_ERROR, "ntrip stream read error %d on fd %d during get rsp\n", errno,
+		dsock);
 	goto close;
+    }
 
     /* parse 401 Unauthorized */
-    if (strstr(buf, NTRIP_UNAUTH)) {
+    if (strstr(buf, NTRIP_UNAUTH)!=NULL) {
 	gpsd_report(LOG_ERROR,
-		    "%s not authorized for Ntrip stream %s:%s/%s\n", auth,
-		    caster, port, stream->mountpoint);
+		"not authorized for Ntrip stream %s/%s\n", stream->url,
+		stream->mountpoint);
 	goto close;
     }
     /* parse SOURCETABLE */
-    if (strstr(buf, NTRIP_SOURCETABLE)) {
+    if (strstr(buf, NTRIP_SOURCETABLE)!=NULL) {
 	gpsd_report(LOG_ERROR,
-		    "Broadcaster doesn't recognize Ntrip stream %s:%s/%s\n",
-		    caster, port, stream->mountpoint);
+		"Broadcaster doesn't recognize Ntrip stream %s:%s/%s\n",
+		stream->url, stream->port, stream->mountpoint);
 	goto close;
     }
     /* parse ICY 200 OK */
-    if (!strstr(buf, NTRIP_ICY)) {
+    if (strstr(buf, NTRIP_ICY)==NULL) {
 	gpsd_report(LOG_ERROR,
-		    "Unknown reply %s from Ntrip service %s:%s/%s\n", buf,
-		    caster, port, stream->mountpoint);
+		"Unknown reply %s from Ntrip service %s:%s/%s\n", buf,
+		stream->url, stream->port, stream->mountpoint);
 	goto close;
     }
-    opts = fcntl(context->dsock, F_GETFL);
+    opts = fcntl(dsock, F_GETFL);
 
     if (opts >= 0)
-	(void)fcntl(context->dsock, F_SETFL, opts | O_NONBLOCK);
+	(void)fcntl(dsock, F_SETFL, opts | O_NONBLOCK);
 
-    context->netgnss_service = netgnss_ntrip;
-#ifndef S_SPLINT_S
-    context->netgnss_privdata = stream;
-#endif
-    return context->dsock;
-  close:
-    (void)close(context->dsock);
+    return dsock;
+close:
+    (void)close(dsock);
     return -1;
+/*@+nullpass@*/
 }
-/* *INDENT-ON* */
 
-/*@ +nullpass @*/
-
-/*@ -branchstate @*/
-int ntrip_open(struct gps_context_t *context, char *caster)
-/* open a connection to a Ntrip broadcaster */
+/*@ -branchstate -nullpass @*/
+int ntrip_open(struct gps_device_t *device, char *caster)
+    /* open a connection to a Ntrip broadcaster */
 {
     char *amp, *colon, *slash;
     char *auth = NULL;
     char *port = NULL;
     char *stream = NULL;
+    char *url = NULL;
     int ret;
+    char t[strlen(caster + 1)];
+    char *tmp = t;
 
-    /*@ -boolops @*/
-    if ((amp = strchr(caster, '@')) != NULL) {
-	if (((colon = strchr(caster, ':')) != NULL) && colon < amp) {
-	    auth = caster;
-	    *amp = '\0';
-	    caster = amp + 1;
-	} else {
-	    gpsd_report(LOG_ERROR,
-			"can't extract user-ID and password from %s\n",
+    switch (device->ntrip.conn_state) {
+	case ntrip_conn_init:
+	    /* this has to be done here, because it is needed for multi-stage connection */
+	    device->servicetype = service_ntrip;
+	    device->ntrip.works = false;
+	    device->ntrip.sourcetable_parse = false;
+	    device->ntrip.stream.set = false;
+	    (void)strlcpy(tmp, caster, strlen(caster) + 1);
+
+	    /*@ -boolops @*/
+	    if ((amp = strchr(tmp, '@')) != NULL) {
+		if (((colon = strchr(tmp, ':')) != NULL) && colon < amp) {
+		    auth = tmp;
+		    *amp = '\0';
+		    tmp = amp + 1;
+		    url = tmp;
+		} else {
+		    gpsd_report(LOG_ERROR,
+			    "can't extract user-ID and password from %s\n",
+			    caster);
+		    device->ntrip.conn_state = ntrip_conn_err;
+		    return -1;
+		}
+	    }
+	    /*@ +boolops @*/
+	    if ((slash = strchr(tmp, '/')) != NULL) {
+		*slash = '\0';
+		stream = slash + 1;
+	    } else {
+		/* todo: add autoconnect like in dgpsip.c */
+		gpsd_report(LOG_ERROR, "can't extract Ntrip stream from %s\n",
 			caster);
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    if ((colon = strchr(tmp, ':')) != NULL) {
+		port = colon + 1;
+		*colon = '\0';
+	    }
+	    if (!port) {
+		port = "rtcm-sc104";
+		if (!getservbyname(port, "tcp"))
+		    port = DEFAULT_RTCM_PORT;
+	    }
+
+	    strncpy(device->ntrip.stream.mountpoint, stream, 101); /* magic numbers from struct definitions */
+	    strncpy(device->ntrip.stream.credentials, auth, 128); /* magic numbers from struct definitions */
+	    strncpy(device->ntrip.stream.url, url, 256);
+	    strncpy(device->ntrip.stream.port, port, 32);
+
+	    ret = ntrip_stream_req_probe(&device->ntrip.stream);
+	    if (ret == -1) {
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    device->gpsdata.gps_fd = ret;
+	    device->ntrip.conn_state = ntrip_conn_sent_probe;
+	    return ret;
+	case ntrip_conn_sent_probe:
+	    ret = ntrip_sourcetable_parse(device);
+	    if (ret == -1) {
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    if (ret == 0 && device->ntrip.stream.set == false) {
+		return ret;
+	    }
+	    (void)close(device->gpsdata.gps_fd);
+	    if (ntrip_auth_encode(&device->ntrip.stream, device->ntrip.stream.credentials, device->ntrip.stream.authStr, 128) != 0) {
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    ret = ntrip_stream_get_req(&device->ntrip.stream);
+	    if (ret == -1) {
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    device->gpsdata.gps_fd = ret;
+	    device->ntrip.conn_state = ntrip_conn_sent_get;
+	    break;
+	case ntrip_conn_sent_get:
+	    ret = ntrip_stream_get_parse(&device->ntrip.stream,
+					 device->gpsdata.gps_fd);
+	    if (ret == -1) {
+		device->ntrip.conn_state = ntrip_conn_err;
+		return -1;
+	    }
+	    device->ntrip.conn_state = ntrip_conn_established;
+	    device->ntrip.works = true; // we know, this worked.
+	    break;
+	case ntrip_conn_established:
+	case ntrip_conn_err:
 	    return -1;
-	}
     }
-    /*@ +boolops @*/
-    if ((slash = strchr(caster, '/')) != NULL) {
-	*slash = '\0';
-	stream = slash + 1;
-    } else {
-	/* todo: add autoconnect like in dgpsip.c */
-	gpsd_report(LOG_ERROR, "can't extract Ntrip stream from %s\n",
-		    caster);
-	return -1;
-    }
-    if ((colon = strchr(caster, ':')) != NULL) {
-	port = colon + 1;
-	*colon = '\0';
-    }
-    if (!port) {
-	port = "rtcm-sc104";
-	if (!getservbyname(port, "tcp"))
-	    port = DEFAULT_RTCM_PORT;
-    }
-    if (ntrip_stream_probe(caster, port, stream, &ntrip_stream)) {
-	gpsd_report(LOG_ERROR,
-		    "unable to probe for data about stream %s:%s/%s\n",
-		    caster, port, stream);
-	return -1;
-    }
-    ret = ntrip_stream_open(caster, port, auth, context, &ntrip_stream);
-    if (ret >= 0)
-	gpsd_report(LOG_PROG,
-		    "connection to Ntrip broadcaster %s established.\n",
-		    caster);
-    else
-	gpsd_report(LOG_ERROR, "can't connect to Ntrip stream %s:%s/%s.\n",
-		    caster, port, stream);
     return ret;
 }
+/*@ +branchstate +nullpass @*/
 
-/*@ +branchstate @*/
-
-void ntrip_report(struct gps_device_t *session)
-/* may be time to ship a usage report to the Ntrip caster */
+void ntrip_report(struct gps_context_t *context, 
+		  struct gps_device_t *gps,
+		  struct gps_device_t *caster)
+    /* may be time to ship a usage report to the Ntrip caster */
 {
-    struct ntrip_stream_t *stream = session->context->netgnss_privdata;
+    static int count;
     /*
      * 10 is an arbitrary number, the point is to have gotten several good
      * fixes before reporting usage to our Ntrip caster.
+     *
+     * count % 5 is as arbitrary a number as the fixcnt. But some delay
+     * was needed here
      */
-    if (stream != NULL && stream->nmea != 0
-	&& session->context->fixcnt > 10 && !session->context->sentdgps) {
-	session->context->sentdgps = true;
-	if (session->context->dsock > -1) {
+    count ++;
+    if (caster->ntrip.stream.nmea != 0 && context->fixcnt > 10 && (count % 5)==0) {
+	if (caster->gpsdata.gps_fd > -1) {
 	    char buf[BUFSIZ];
-	    gpsd_position_fix_dump(session, buf, sizeof(buf));
-	    if (write(session->context->dsock, buf, strlen(buf)) ==
-		(ssize_t) strlen(buf))
+	    gpsd_position_fix_dump(gps, buf, sizeof(buf));
+	    if (write(caster->gpsdata.gps_fd, buf, strlen(buf)) ==
+		    (ssize_t) strlen(buf)) {
 		gpsd_report(LOG_IO, "=> dgps %s\n", buf);
-	    else
+	    } else {
 		gpsd_report(LOG_IO, "ntrip report write failed\n");
+	    }
 	}
     }
 }
