@@ -7,30 +7,19 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 
-#include <stdlib.h>
-#include "gpsd_config.h"
-#include <sys/types.h>
+#include <string.h>
+#include <stdbool.h>
+#include <math.h>
+#include <errno.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-#include <stdio.h>
-#include <string.h>
-#include <math.h>
-#include <errno.h>
 
 #include "gpsd.h"
 #ifdef NTPSHM_ENABLE
-
-#if defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
-#endif
-
-#ifdef HAVE_SYS_IPC_H
 #include <sys/ipc.h>
-#endif /* HAVE_SYS_IPC_H */
-#ifdef HAVE_SYS_SHM_H
 #include <sys/shm.h>
-#endif /* HAVE_SYS_SHM_H */
 
 #define PPS_MAX_OFFSET	100000	/* microseconds the PPS can 'pull' */
 #define PUT_MAX_OFFSET	1000000	/* microseconds for lost lock */
@@ -116,10 +105,11 @@ struct shmTime
  * ipcrm  -M 0x4e545032
  * ipcrm  -M 0x4e545033
  */
-static /*@null@*/ struct shmTime *getShmTime(int unit)
+static /*@null@*/ volatile struct shmTime *getShmTime(int unit)
 {
     int shmid;
     unsigned int perms;
+    volatile struct shmTime *p;
     // set the SHM perms the way ntpd does
     if (unit < 2) {
 	// we are root, be careful
@@ -136,19 +126,18 @@ static /*@null@*/ struct shmTime *getShmTime(int unit)
 		    (long int)(NTPD_BASE + unit), sizeof(struct shmTime),
 		    (int)perms, strerror(errno));
 	return NULL;
-    } else {
-	struct shmTime *p = (struct shmTime *)shmat(shmid, 0, 0);
-	/*@ -mustfreefresh */
-	if ((int)(long)p == -1) {
-	    gpsd_report(LOG_ERROR, "NTPD shmat failed: %s\n",
-			strerror(errno));
-	    return NULL;
-	}
-	gpsd_report(LOG_PROG, "NTPD shmat(%d,0,0) succeeded, segment %d\n",
-		    shmid, unit);
-	return p;
-	/*@ +mustfreefresh */
+    } 
+    p = (struct shmTime *)shmat(shmid, 0, 0);
+    /*@ -mustfreefresh */
+    if ((int)(long)p == -1) {
+	gpsd_report(LOG_ERROR, "NTPD shmat failed: %s\n",
+		    strerror(errno));
+	return NULL;
     }
+    gpsd_report(LOG_PROG, "NTPD shmat(%d,0,0) succeeded, segment %d\n",
+		shmid, unit);
+    return p;
+    /*@ +mustfreefresh */
 }
 
 void ntpshm_init(struct gps_context_t *context, bool enablepps)
@@ -204,7 +193,10 @@ bool ntpshm_free(struct gps_context_t * context, int segment)
 int ntpshm_put(struct gps_device_t *session, double fixtime, double fudge)
 /* put a received fix time into shared memory for NTP */
 {
-    struct shmTime *shmTime = NULL;
+    /* shmTime is volatile to try to prevent C compiler from reordering
+     * writes, or optimizing some 'dead code'.  but CPU cache may still 
+     *write out of order since we do not use memory barriers, yet */
+    volatile struct shmTime *shmTime = NULL;
     struct timeval tv;
     double seconds, microseconds;
 
@@ -239,6 +231,8 @@ int ntpshm_put(struct gps_device_t *session, double fixtime, double fudge)
      */
     shmTime->valid = 0;
     shmTime->count++;
+    /* FIXME need a memory barrier here to prevent write reordering by
+     * the compiler or CPU cache */
     shmTime->clockTimeStampSec = (time_t) seconds;
     shmTime->clockTimeStampUSec = (int)microseconds;
     shmTime->receiveTimeStampSec = (time_t) tv.tv_sec;
@@ -248,6 +242,8 @@ int ntpshm_put(struct gps_device_t *session, double fixtime, double fudge)
      * Any NMEA will be about -1 or -2. 
      * Garmin GPS-18/USB is around -6 or -7.
      */
+    /* FIXME need a memory barrier here to prevent write reordering by
+     * the compiler or CPU cache */
     shmTime->count++;
     shmTime->valid = 1;
 
@@ -260,11 +256,14 @@ int ntpshm_put(struct gps_device_t *session, double fixtime, double fudge)
 }
 
 #ifdef PPS_ENABLE
-/* put NTP shared memory info based on received PPS pulse */
-
+/* put NTP shared memory info based on received PPS pulse
+ *
+ * good news is that kernel PPS gives us nSec resolution
+ * bad news is that ntpshm only has uSec resolution
+ */
 int ntpshm_pps(struct gps_device_t *session, struct timeval *tv)
 {
-    struct shmTime *shmTime = NULL, *shmTimeP = NULL;
+    volatile struct shmTime *shmTime = NULL, *shmTimeP = NULL;
     time_t seconds;
     /* FIX-ME, microseconds needs to be set for 5Hz PPS */
     int microseconds = 0;
