@@ -2,32 +2,27 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <stdbool.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <ctype.h>
+#ifndef S_SPLINT_S
+#include <unistd.h>
+#include <sys/socket.h>
+#endif /* S_SPLINT_S */
 
 #include "gpsd_config.h"
-
 #ifdef HAVE_BLUEZ
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 #include <bluetooth/rfcomm.h>
-#endif
-
-#if defined(HAVE_SYS_MODEM_H)
-#include <sys/modem.h>
-#endif /* HAVE_SYS_MODEM_H */
+#endif /* HAVE_BLUEZ */
 
 #include "gpsd.h"
 
@@ -53,13 +48,13 @@ static sourcetype_t gpsd_classify(const char *path)
     else if (S_ISSOCK(sb.st_mode))
 	return source_tcp;
     else if (S_ISCHR(sb.st_mode)) {
-	sourcetype_t devtype = source_unknown;
+	sourcetype_t devtype = source_rs232;
 #ifdef __linux__
 	/* Linux major device numbers live here
 	 * ftp://ftp.kernel.org/pub/linux/docs/device-list/devices-2.6+.txt
 	 */
 	int devmajor = major(sb.st_rdev);
-	if (devmajor == 4)
+	if (devmajor == 4 || devmajor == 204)
 	    devtype = source_rs232;
 	else if (devmajor == 188)
 	    devtype = source_usb;
@@ -73,6 +68,46 @@ static sourcetype_t gpsd_classify(const char *path)
 	return source_unknown;
 }
 
+#ifdef __linux__
+#include <dirent.h>
+#include <ctype.h>
+
+static int fusercount(const char *path)
+/* return true if any process has the specified path open */
+{
+    DIR *procd, *fdd;
+    struct dirent *procentry, *fdentry;
+    char procpath[32], fdpath[64], linkpath[64];
+    int cnt = 0;
+
+    if ((procd = opendir("/proc")) == NULL)
+	return false;
+    while ((procentry = readdir(procd)) != NULL) {
+	if (isdigit(procentry->d_name[0])==0)
+	    continue;
+	(void)snprintf(procpath, sizeof(procpath), 
+		       "/proc/%s/fd/", procentry->d_name);
+	if ((fdd = opendir(procpath)) == NULL)
+	    continue;
+	while ((fdentry = readdir(fdd)) != NULL) {
+	    (void)strlcpy(fdpath, procpath, sizeof(fdpath));
+	    (void)strlcat(fdpath, fdentry->d_name, sizeof(fdpath));
+	    (void)memset(linkpath, '\0', sizeof(linkpath));
+	    if (readlink(fdpath, linkpath, sizeof(linkpath)) == -1)
+		continue;
+	    if (strcmp(linkpath, path) == 0) {
+		(void)closedir(procd);
+		return true;
+	    }
+	}
+	(void)closedir(fdd);
+    }
+    (void)closedir(procd);
+
+    return cnt;
+}
+#endif /* __linux__ */
+
 void gpsd_tty_init(struct gps_device_t *session)
 /* to be called on allocating a device */
 {
@@ -87,7 +122,7 @@ void gpsd_tty_init(struct gps_device_t *session)
 # endif	/* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
     session->zerokill = false;
-    session->reawake = 0;
+    session->reawake = (timestamp_t)0;
 }
 
 #if defined(__CYGWIN__)
@@ -274,12 +309,17 @@ void gpsd_set_speed(struct gps_device_t *session,
     session->gpsdata.dev.parity = parity;
     session->gpsdata.dev.stopbits = stopbits;
 
-    if (!session->context->readonly) {
-	/*
-	 * The device might need a wakeup string before it will send data.
-	 * If we don't know the device type, ship it every driver's wakeup
-	 * in hopes it will respond.
-	 */
+    /*
+     * The device might need a wakeup string before it will send data.
+     * If we don't know the device type, ship it every driver's wakeup
+     * in hopes it will respond.  But not to USB or Bluetooth, because
+     * shipping probe strings to unknown USB serial adaptors or
+     * Bluetooth devices may spam devices that aren't GPSes at all and
+     * could become confused.
+     */
+    if (!session->context->readonly 
+		&& session->sourcetype != source_usb 
+		&& session->sourcetype != source_bluetooth) {
 	if (isatty(session->gpsdata.gps_fd) != 0
 	    && !session->context->readonly) {
 	    const struct gps_type_t **dp;
@@ -294,11 +334,13 @@ void gpsd_set_speed(struct gps_device_t *session,
     packet_reset(&session->packet);
 }
 
-int gpsd_open(struct gps_device_t *session)
+int gpsd_serial_open(struct gps_device_t *session)
+/* open a device for access to its data */
 {
     mode_t mode = (mode_t) O_RDWR;
 
     session->sourcetype = gpsd_classify(session->gpsdata.dev.path);
+    session->servicetype = service_sensor;
 
     /*@ -boolops -type @*/
     if (session->context->readonly
@@ -353,6 +395,38 @@ int gpsd_open(struct gps_device_t *session)
 	}
     }
 
+    /*
+     * Ideally we want to exclusion-lock the device before doing any reads.
+     * It would have been best to do this at open(2) time, but O_EXCL
+     * doesn't work wuthout O_CREAT.
+     *
+     * We have to make an exception for ptys, which are intentionally
+     * opened by another process on the master side, otherwise we'll
+     * break all our regression tests.
+     */
+    if (session->sourcetype != source_pty) {
+	/*
+	 * Try to block other processes from using this device while we
+	 * have it open (later opens should return EBUSY).  Won't work
+	 * against anything with root privileges, alas.
+	 */
+	(void)ioctl(session->gpsdata.gps_fd, (unsigned long)TIOCEXCL);
+
+#ifdef __linux__
+	/*
+	 * Don't touch devices already opened by another process.
+	 */
+	if (fusercount(session->gpsdata.dev.path) > 1) {
+            gpsd_report(LOG_ERROR, 
+			"%s already opened by another process\n",
+			session->gpsdata.dev.path);
+	    (void)close(session->gpsdata.gps_fd);
+	    session->gpsdata.gps_fd = -1;
+	    return -1;
+	}
+#endif /* __linux__ */
+    }
+
 #ifdef FIXED_PORT_SPEED
     session->saved_baud = FIXED_PORT_SPEED;
 #endif
@@ -394,13 +468,12 @@ int gpsd_open(struct gps_device_t *session)
 	session->baudindex = 0;
 	gpsd_set_speed(session, gpsd_get_speed(&session->ttyset_old), 'N', 1);
     }
-    session->is_serial = true;
     gpsd_report(LOG_SPIN, "open(%s) -> %d in gpsd_open()\n",
 		session->gpsdata.dev.path, session->gpsdata.gps_fd);
     return session->gpsdata.gps_fd;
 }
 
-ssize_t gpsd_write(struct gps_device_t * session, void const *buf, size_t len)
+ssize_t gpsd_write(struct gps_device_t * session, char const *buf, size_t len)
 {
     ssize_t status;
     bool ok;
