@@ -4,8 +4,6 @@
  * BSD terms apply: see the file COPYING in the distribution root for details. 
  */
 
-#include <sys/types.h>
-#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 
