@@ -4,16 +4,15 @@
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
 
-#include <assert.h>
 #include <stdio.h>
+#include <assert.h>
 #include <stdlib.h>
-#include <stddef.h>
 #include <string.h>
+#include <stddef.h>
 
 #include "gpsd.h"
 #include "gps_json.h"
 
-#include "strl.c"
 
 static void assert_case(int num, int status)
 {
@@ -89,13 +88,13 @@ static struct gps_data_t gpsdata;
 /* *INDENT-OFF* */
 static const char json_str1[] = "{\"class\":\"TPV\",\
     \"device\":\"GPS#1\",\"tag\":\"MID2\",				\
-    \"time\":1119197561.890,\"lon\":46.498203637,\"lat\":7.568074350,\
+    \"time\":\"2005-06-19T08:12:41.89Z\",\"lon\":46.498203637,\"lat\":7.568074350,\
     \"alt\":1327.780,\"epx\":21.000,\"epy\":23.000,\"epv\":124.484,\"mode\":3}";
 
 /* Case 2: SKY report */
 
 static const char *json_str2 = "{\"class\":\"SKY\",\
-         \"tag\":\"MID4\",\"time\":1119197562.890,   \
+         \"tag\":\"MID4\",\"time\":\"2005-06-19T12:12:42.03Z\",   \
          \"satellites\":[\
          {\"PRN\":10,\"el\":45,\"az\":196,\"ss\":34,\"used\":true},\
          {\"PRN\":29,\"el\":67,\"az\":310,\"ss\":40,\"used\":true},\
@@ -220,7 +219,7 @@ int main(int argc UNUSED, char *argv[]UNUSED)
     assert_string("device", gpsdata.dev.path, "GPS#1");
     assert_string("tag", gpsdata.tag, "MID2");
     assert_integer("mode", gpsdata.fix.mode, 3);
-    assert_real("time", gpsdata.fix.time, 1119197561.890);
+    assert_real("time", gpsdata.fix.time, 1119168761.8900001);
     assert_real("lon", gpsdata.fix.longitude, 46.498203637);
     assert_real("lat", gpsdata.fix.latitude, 7.568074350);
 
